@@ -71,16 +71,15 @@ module FileOption = struct
           let get_value name conv = conv (List.assoc name assocs) in
 	  let get_value_nil name conv =
 	    try conv (List.assoc name assocs) with Not_found -> [] in
+          let filename = get_value "file_filename" value_to_string in
           let network = try get_value "file_network" value_to_string
             with _ -> "Donkey" in
           let network = 
             try network_find_by_name network with e ->
                 lprintf_nl
 		  "Error %s for network %s while parsing file %s"
-		    (Printexc2.to_string e)
-		    network
-		    (get_value "file_filename" value_to_string);
-    lprintf_nl "This core is lacking support for network %s, exiting" network;
+		    (Printexc2.to_string e) network filename;
+                lprintf_nl "This core is lacking support for network %s, exiting" network;
                 exit_properly 70
           in
           let file_state = 
@@ -93,24 +92,7 @@ module FileOption = struct
             with _ -> Int64.zero
           in
           
-          let file = network_file_of_option network file_size 
-              file_state assocs in
-          let priority = try get_value "file_priority" value_to_int 
-            with _ -> 0 in
-          
-          let impl = as_file_impl file in
-          (try
-              impl.impl_file_age <- 
-                normalize_time (get_value "file_age" value_to_int)
-            with _ -> ());
-
-          (try
-              impl.impl_file_release <- 
-                get_value "file_release" value_to_bool
-            with _ -> ());
-
 	  let file_user =
-	    let filename = get_value "file_filename" value_to_string in
 	    try
 	      let u = get_value "file_owner" value_to_string in
 		begin
@@ -118,21 +100,19 @@ module FileOption = struct
 		    user2_user_find u
 		  with Not_found ->
 		    lprintf_nl "file_owner %s of %s does not exist, changing to %s"
-		      u filename admin_user.user_name;
-		    admin_user
+		      u filename (admin_user ()).user_name;
+		    admin_user ()
 		end
 	    with Not_found ->
 		lprintf_nl "file_owner of %s is empty, changing to %s"
-		  filename admin_user.user_name;
-		admin_user
+		  filename (admin_user ()).user_name;
+		admin_user ()
 	  in
-	  set_file_owner file file_user;
 
 	  let file_group =
-	    let filename = get_value "file_filename" value_to_string in
 	    let dgroup = user2_print_user_default_group file_user in
 	    try
-	      match (get_value "file_group" stringvalue_to_option) with
+	      match (get_value "file_group" value_to_stringoption) with
 		None -> None
 	      | Some g ->
 		  begin
@@ -156,13 +136,25 @@ module FileOption = struct
 		  filename dgroup file_user.user_name;
 		file_user.user_default_group
 	  in
-	  set_file_group file file_group;
+
+          let file = network_file_of_option network file_size 
+              file_state file_user file_group assocs in
+          
+          let impl = as_file_impl file in
+          (try
+              impl.impl_file_age <- 
+                normalize_time (get_value "file_age" value_to_int)
+            with _ -> ());
+
+          (try
+              impl.impl_file_release <- 
+                get_value "file_release" value_to_bool
+            with _ -> ());
 
           set_file_state file file_state;       
 
           (try
-              set_file_best_name file
-              (get_value "file_filename" value_to_string) "" 0
+              set_file_best_name file filename "" 0
             with _ -> ());
 
           (try
@@ -170,14 +162,12 @@ module FileOption = struct
 	      (get_value_nil "file_filenames" (value_to_list value_to_string))
             with _ -> ());
 
+          let priority = try get_value "file_priority" value_to_int 
+            with _ -> 0 in
           set_file_priority file priority;
 
-          if !verbose then lprintf_nl "New %s file %s"
-	      (match file_state with
-                 FileDownloading -> "downloading"
-               | FileDownloaded -> "downloaded"
-               | _ -> "other")
-	     (file_best_name file);
+          if !verbose && !CommonGlobals.is_startup_phase then
+            lprintf_nl "Started download of %s" (file_best_name file);
 
           file
       | _ -> assert false
@@ -196,7 +186,7 @@ module FileOption = struct
         ("file_age", IntValue (Int64.of_int impl.impl_file_age)) ::
         ("file_release", bool_to_value impl.impl_file_release) ::
         ("file_owner", string_to_value (file_owner file).user_name) ::
-        ("file_group", option_to_stringvalue (match file_group file with Some g -> Some g.group_name | None -> None)) ::
+        ("file_group", stringoption_to_value (match file_group file with Some g -> Some g.group_name | None -> None)) ::
           (file_to_option file)
         )
           
@@ -339,7 +329,7 @@ module ServerOption = struct
           in
           let network = 
             try network_find_by_name network with e ->
-                lprintf_nl "Network %s not supported" network;
+                lprintf_nl "Loading servers, network %s not supported, deleting server" network;
                 raise e
               in
           let server = network_server_of_option network assocs in
@@ -962,7 +952,30 @@ let search_incoming_directories () =
         [default_incoming_directories]
   | l -> l
 
-exception Incoming_full
+let shared_directories_including_user_commit () =
+(* This function is to be used in bTInteractive.try_share_file which is not recursive.
+   Its a replacement for !!shared_directories and provides the same list, but with
+   sub-directories added based on user_commit_dir.
+   This function works without disc access to avoid overhead. *)
+  let list = ref [] in
+  List.iter (fun s ->
+    let user_commit_dir_list = ref [] in
+    if (sharing_strategy s.shdir_strategy).sharing_incoming then
+      begin
+        user2_users_iter (fun u ->
+          if u.user_commit_dir <> "" then
+            user_commit_dir_list := !user_commit_dir_list @
+              [{
+                shdir_dirname = Filename.concat s.shdir_dirname u.user_commit_dir;
+                shdir_priority = s.shdir_priority;
+                shdir_networks = s.shdir_networks;
+                shdir_strategy = s.shdir_strategy;
+              }]
+        )
+      end;
+    list := !list @ [s] @ !user_commit_dir_list
+  ) !!shared_directories;
+  !list
 
 let incoming_dir usedir ?user ?needed_space ?network () =
 
@@ -994,12 +1007,13 @@ let incoming_dir usedir ?user ?needed_space ?network () =
   in
 
   let checkdir =
+    let module U = Unix.LargeFile in
     try
       List.find (fun d ->
 	let dirname = compute_dir_name d.shdir_dirname in
 (* check if temp_directory and incoming are on different partitions *)
 	try
-          if (Unix.stat dirname).Unix.st_dev <> (Unix.stat !!temp_directory).Unix.st_dev then
+          if (U.stat dirname).U.st_dev <> (U.stat !!temp_directory).U.st_dev then
             begin
               match needed_space with
               | None -> true
@@ -1097,14 +1111,15 @@ open Zip
 
 let backup_zip archive files =
   try 
-    Unix2.tryopen_write_zip archive (fun oc ->
-      List.iter (fun file ->
-	try
-          let s = Unix.stat file in
-	  Zip.copy_file_to_entry file oc ~level:9 ~mtime:s.Unix.st_mtime file
+    Unix2.tryopen_umask 0o066 (fun _old_umask ->
+      Unix2.tryopen_write_zip archive (fun oc ->
+        List.iter (fun file -> try
+          let module U = Unix.LargeFile in
+          let s = U.stat file in
+	  Zip.copy_file_to_entry file oc ~level:9 ~mtime:s.U.st_mtime file
 	with e ->
 	  failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) file)
-      ) files)
+      ) files))
   with e ->
     failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) archive)
 
@@ -1112,21 +1127,23 @@ open Tar
 
 let backup_tar archive files =
   let failed_files = ref [] in
-  Unix2.tryopen_write_tar ~compress:`Gzip archive (fun otar ->
-    List.iter (fun arg ->
-      try
+  Unix2.tryopen_umask 0o066 (fun _old_umask ->
+    Unix2.tryopen_write_tar ~compress:`Gzip archive (fun otar ->
+      List.iter (fun arg -> try
 	let header, s =
 	  Unix2.tryopen_read_bin arg (fun ic ->
 	    let stat = Unix.stat arg in
 	    let size = stat.Unix.st_size in
 	    if size > Sys.max_string_length then
-	      failwith (Printf.sprintf "Tar: file %s too big, limit %d byte" arg Sys.max_string_length);
+	      failwith (Printf.sprintf
+                  "Tar: file %s too big, system limit %d byte, use .zip to avoid this limit"
+                    arg Sys.max_string_length);
 	    let header = 
 	      { Tar.t_name = arg;
-	      t_mode = 0o644;
+	      t_mode = stat.Unix.st_perm;
 	      t_uid = stat.Unix.st_uid;
 	      t_gid = stat.Unix.st_gid;
-	      t_size = 0;
+	      t_size = stat.Unix.st_size;
 	      t_mtime = Int32.of_float stat.Unix.st_mtime;
 	      t_chksum = 0;
 	      t_typeflag = REGULAR;
@@ -1150,11 +1167,13 @@ let backup_tar archive files =
             lprintf_nl "Tar: Windows specific pseudo error %s in %s" (Printexc2.to_string e) arg
       | e ->
 	  failed_files := arg :: !failed_files;
-	  lprintf_nl "Tar: error %s in %s" (Printexc2.to_string e) arg
-    ) files);
-    if !failed_files <> [] then
-      failwith (Printf.sprintf "Tar: error backing up %s"
-	(String.concat " " (List.rev !failed_files)))
+	  lprintf_nl "Tar: skipping %s, error %s" arg (Printexc2.to_string e)
+      ) files
+    )
+  );
+  if !failed_files <> [] then
+    failwith (Printf.sprintf "Tar: skipped %s due to backup errors"
+      (String.concat " " (List.rev !failed_files)))
 
 let backup_options () =
   let counter = ref 1 in
@@ -1180,17 +1199,21 @@ let backup_options () =
   in
   begin
     try
-    let archive = Filename.concat "old_config" (backup_prefix ^ Date.reverse (Unix.time ()) ^ format) in
-      let files = List.sort (fun o -> compare o) (List.filter (fun o ->
+      let archive =
+        Filename.concat "old_config" (backup_prefix ^ Date.reverse (Unix.time ()) ^ format)
+      in
+      let files =
+        List.sort (fun o -> compare o) (List.filter (fun o ->
 	  String.lowercase (Filename2.last_extension o) = ".ini"
-	  && o <> "file_sources.ini") 
-	    (Unix2.list_directory file_basedir)) in
+	  && o <> "file_sources.ini")
+	    (Unix2.list_directory file_basedir))
+      in
       begin
         match (Filename2.last_extension archive) with
           ".zip" -> backup_zip archive files
         | _ -> backup_tar archive files
       end
-    with e -> lprintf_nl "Exception %s while options backup" (Printexc2.to_string e); raise e
+    with e -> lprintf_nl "Exception %s while options backup" (Printexc2.to_string e)
   end;
   lprintf_nl (_b "Options backup as %s correctly saved") format
              
@@ -1217,63 +1240,77 @@ let _ =
   option_hook max_filenames (fun _ ->
     shorten_all_file_filenames !!max_filenames
   );
+
+  let max_opened_connections_pass = ref 0 in
   option_hook max_opened_connections (fun _ ->
+
+  incr max_opened_connections_pass;
+
+(* let users see if the option is called again *)
+  let lprintf_nl s = lprintf_nl2
+      (Printf.sprintf "%s pass %d:" log_prefix !max_opened_connections_pass) s in
+
   if !verbose then lprintf_nl
     "checking max_opened_connections = %d for validity" !!max_opened_connections;
-(* original code from ./src/config/unix/MlUnix.ml
-let max_all_sockets = getdtablesize ()
-let max_sockets = max (max_all_sockets - 100) (max_all_sockets / 2)
-let max_filedescs = (max_all_sockets - max_sockets) / 2 *)
 
-  (* ulimit open files. minimum 150, most systems have 1024 *)
-  let max_all_sockets = Unix2.c_getdtablesize () in
+(* maximum value of open sockets/files allowed *)
+  let max_all_fds = Unix2.c_getdtablesize () in
 
-  (* old max_sockets code: max (150 - 100) (150 / 2),
-     minimum number of max_opened_connections *)
-  let min_conns = 75 in
+(* ini files, dynamic libs, etc. *)
+  let reserved_fds = max CommonOptions.min_reserved_fds (max_all_fds / 50) in
 
-  if min_conns > !!max_opened_connections then begin
-    lprintf_nl "max_opened_connections is set too low (%d), raising to %d"
-      !!max_opened_connections min_conns;
-    max_opened_connections =:= min_conns
-  end;
+(* minimum number of max_opened_connections, p2p needs some sockets *)
+  let min_conns = CommonOptions.min_connections in
+(* max_conns *should* be greater than min_conns at that point, because of
+   the sanity check at start time in CommonOptions;
+   taking the max is just a safety belt from a paranoid :) *)
+  let max_conns = max min_conns
+    (max_all_fds - reserved_fds - Unix32.max_cache_size_default) in
 
-  let reserved_fds = 40 in (* ini files, dynamic libs, etc. *)
-
-  let total_files = (* maximum number of files in use at the same time *)
-    (max (List.length !!files) !!max_concurrent_downloads) + !!max_upload_slots + reserved_fds
+  let print_stats verbose =
+    if verbose then begin
+      lprintf_nl "file descriptors status: total allowed (ulimit -n) %d" max_all_fds;
+      lprintf_nl "- max_opened_connections %d (%d%% indirect)"
+        !!max_opened_connections !!max_indirect_connections;
+      lprintf_nl "- file cache size %d" (Unix32.get_max_cache_size ());
+      lprintf_nl "- reserved %d" reserved_fds;
+      let s,v =
+        let v1 =
+          max_all_fds - !!max_opened_connections - (Unix32.get_max_cache_size ()) - reserved_fds
+        in
+        if v1 >= 0 then "left", v1 else "missing", (abs v1)
+      in
+      lprintf_nl "= %d descriptors %s" v s
+    end
   in
 
-  let wanted_socks = !!max_opened_connections + total_files in
+  if !!max_opened_connections < min_conns then begin
+    lprintf_nl "max_opened_connections is set too low (%d), raising to %d"
+      !!max_opened_connections min_conns;
+    print_stats true;
+    max_opened_connections =:= min_conns
+  end
+  else if !!max_opened_connections > max_conns then begin
+    lprintf_nl "max_opened_connections is set too high (%d), lowering to %d"
+      !!max_opened_connections max_conns;
+    print_stats true;
+    max_opened_connections =:= max_conns
+  end
+  else begin
+    TcpBufferedSocket.set_max_opened_connections (fun _ -> !!max_opened_connections);
 
-  if max_all_sockets < wanted_socks then
-    if max_all_sockets < total_files + min_conns then (* check if ulimit is enough to allow total_files + min_conns *)
-      begin
-        lprintf_nl "only %d file descriptors available, raise ulimit open files to at least %d"
-          max_all_sockets wanted_socks;
-        lprintf_nl "FD info: max_opened_connections %d, number of (possible) concurrent downloads %d, = %d fd needed"
-          !!max_opened_connections total_files wanted_socks;
-        CommonGlobals.exit_properly 71
-      end
-  else
-    begin
-      let new_max_opened_connections =
-        max (max_all_sockets - total_files) (max_all_sockets / 2)
-      in
-      lprintf_nl "max_opened_connections is set too high (%d), reducing to %d"
-        !!max_opened_connections new_max_opened_connections;
-      max_opened_connections =:= new_max_opened_connections;
-    end;
+    let unused_fds = max_conns - !!max_opened_connections in
 
-  if !verbose then lprintf_nl
-    "max_opened_connections %d, total_files %d, max_concurrent_downloads %d, !!files %d"
-      !!max_opened_connections total_files !!max_concurrent_downloads (List.length !!files);
+    Unix32.set_max_cache_size
+      (Unix32.max_cache_size_default + unused_fds * 75 / 100);
 
-  TcpBufferedSocket.set_max_opened_connections
-    (fun _ -> !!max_opened_connections);
+    calc_real_max_indirect_connections ();
 
-  Unix32.max_cache_size := total_files - reserved_fds;
-  calc_real_max_indirect_connections ()
+    print_stats !verbose
+  end;
+
+  if !verbose then lprintf_nl "checking max_opened_connections finished";
+  decr max_opened_connections_pass
 )
 
 let _ =
