@@ -746,7 +746,7 @@ parent.fstatus.location.href='submit?q=chown+'+v+'+%d';
 
       Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
       if user2_allow_file_admin file o.conn_user.ui_user &&
-         o.conn_user.ui_user.user_groups <> [] then
+         (file_owner file).user_groups <> [] then begin
         let optionlist =
           if (file_group file) = None then
             ref ""
@@ -777,6 +777,7 @@ parent.fstatus.location.href='submit?q=chgrp+'+v+'+%d';
    ^ Printf.sprintf "\\<option value=\\\"%s\\\" selected\\>%s\\</option\\>\n" (user2_print_group (file_group file)) (user2_print_group (file_group file))
    ^ !optionlist ^ "\\</select\\>\\</td\\>\\</form\\>\\</tr\\>\\</table\\>"
   ) ];
+        end
 
       else
         html_mods_td buf [("File group", "sr br", "Group");
@@ -826,6 +827,13 @@ parent.fstatus.location.href='submit?q=chgrp+'+v+'+%d';
             ) ]
       );
 
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+      html_mods_td buf [
+        ("Chunk size", "sr br", "Chunk size");
+        ("", "sr", (match info.G.file_chunk_size with
+          Some v -> String.concat " " (List.map (fun v -> Printf.sprintf "%Ld" v) v)
+        | None -> "unknown"))];
+
       (match file_magic file with
         Some magic ->
 	    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
@@ -1232,6 +1240,7 @@ let impl_file_info impl =
     T.file_active_sources = 0;
     T.file_sources = None;
     T.file_chunks = None;
+    T.file_chunk_size = None;
     T.file_availability = [];
     T.file_format = FormatNotComputed 0;
     T.file_chunks_age = [||];
