@@ -40,7 +40,6 @@ open CommonResult
 open CommonServer
 open CommonTypes
 open CommonComplexOptions
-open CommonUserDb
 
 let log_prefix = "[cInt]"
 
@@ -96,6 +95,7 @@ let hdd_full_log_closed = ref false
 let all_temp_queued = ref false
 
 let send_dirfull_warning dir full line1 =
+  if !!smtp_server <> "" && !!smtp_port <> 0 then begin
   let status = if full then "is full" else "has enough space again" in
   lprintf_nl "WARNING: Directory %s %s, %s" dir status line1;
   if (not (keep_console_output ())) then
@@ -125,15 +125,16 @@ let send_dirfull_warning dir full line1 =
           M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
 	with _ -> ()
     end
+  end
 
 let file_commited_name incoming_dir file =
   (try Unix2.safe_mkdir incoming_dir with _ -> ());
   let fs = Unix32.filesystem incoming_dir in
   let namemax =
     match Unix32.fnamelen incoming_dir with
-      None -> 0
-    | Some v -> Int64.to_int v in
-
+    | None -> 0
+    | Some v -> v
+  in
   let new_name =
     Filename2.filesystem_compliant 
       (canonize_basename (file_best_name file)) fs namemax in
@@ -163,6 +164,22 @@ let script_for_file file incoming new_name =
   in
   let network = network_find_by_num info.G.file_network in
   let filename = Filename.basename new_name in
+  let file_group_info =
+    match file_group file with
+    | None -> 0, []
+    | Some _ ->
+      let users = ref [] in
+      let counter = ref 0 in
+      user2_users_iter (fun u ->
+        if file_owner file <> u &&
+          user2_can_view_file u (file_owner file) (file_group file) then
+            begin
+              incr counter;
+              users :=  (Printf.sprintf "FILE_GROUP_USER_%d" !counter, u.user_name) ::
+                        (Printf.sprintf "FILE_GROUP_DIR_%d" !counter, u.user_commit_dir) :: !users
+            end);
+      !counter, !users
+  in
   begin try
   MlUnix.fork_and_exec !!file_completed_cmd
       [|  (* keep those for compatibility *)
@@ -171,7 +188,7 @@ let script_for_file file incoming new_name =
       size; (* $2 *)
       filename (* $3 *)
     |]
-     ~vars:[("TEMPNAME",  temp_name);
+     ~vars:([("TEMPNAME",  temp_name);
 	    ("FILEID",    file_id);
 	    ("FILESIZE",  size);
 	    ("FILENAME",  filename);
@@ -183,7 +200,13 @@ let script_for_file file incoming new_name =
 	    ("ED2K_HASH", (file_print_ed2k_link filename (file_size file) info.G.file_md4));
 	    ("FILE_OWNER",(file_owner file).user_name);
 	    ("FILE_GROUP",user2_print_group (file_group file));
+	    ("USER_MAIL", ( if (file_owner file).user_mail <> "" then
+                              (file_owner file).user_mail
+                            else
+                              if !!mail <> "" then !!mail else ""));
+	    ("FILE_GROUP_CNT", string_of_int (fst (file_group_info)));
 	    ]
+            @ snd (file_group_info))
 
   with e -> 
       lprintf_nl "Exception %s while executing %s"
@@ -200,8 +223,6 @@ file has already been moved to the incoming/ directory under its new
 name.
 *)
 
-exception Incoming_full
-
 let file_commit file =
   let impl = as_file_impl file in
   if impl.impl_file_state = FileDownloaded then
@@ -222,7 +243,7 @@ let file_commit file =
 	  let new_name = file_commited_name incoming.shdir_dirname file in
 	    if Unix2.is_directory file_name then begin
 	      Unix2.safe_mkdir new_name;
-	      Unix2.chmod new_name (Misc.int_of_octal_string !!create_dir_mask)
+	      Unix2.chmod new_name !Unix32.create_dir_mode;
 	    end;
 
 (*          the next line really moves the file *)
@@ -305,7 +326,7 @@ let file_cancel file user =
 
 let mail_for_completed_file file =
   let usermail = (file_owner file).user_mail in
-  if !!mail <> "" || usermail <> "" then begin
+  if (!!mail <> "" || usermail <> "") && !!smtp_server <> "" && !!smtp_port <> 0 then begin
     let module M = Mailer in
     let info = file_info file in
     let line1 = "mldonkey has completed the download of:\r\n\r\n" in
@@ -331,7 +352,12 @@ let mail_for_completed_file file =
     let incoming = incoming_dir (Unix2.is_directory (file_disk_name file)) () in
 
     let line4 = if !!url_in_mail = "" then "" else
-      Printf.sprintf "\r\n<%s/%s/%s>\r\n" !!url_in_mail incoming.shdir_dirname (Url.encode (file_best_name file))
+      Printf.sprintf "\r\n<%s/%s%s/%s>\r\n"
+        !!url_in_mail
+        incoming.shdir_dirname
+        (if (file_owner file).user_commit_dir = "" then ""
+         else Printf.sprintf "/%s" (file_owner file).user_commit_dir)
+        (Url.encode (file_best_name file))
     in
 
     let line5 = if !!auto_commit then "" else
@@ -359,6 +385,7 @@ let file_completed (file : file) =
   try
     let impl = as_file_impl file in
     if impl.impl_file_state = FileDownloading then begin
+        CommonSwarming.duplicate_chunks ();
         files =:= List2.removeq file !!files;
         done_files =:= file :: !!done_files;
         update_file_state impl FileDownloaded;
@@ -385,7 +412,7 @@ let file_add impl state =
           | FileDownloading
           | FileQueued
           | FilePaused ->
-              files =:= file :: !!files);
+              files =:= !!files @ [file]);
         update_file_state impl state
       end
   with e ->
@@ -496,7 +523,7 @@ let display_vd = ref false
 let display_bw_stats = ref false
 
 let start_download file =
-  if !!pause_new_downloads then file_pause file admin_user;
+  if !!pause_new_downloads then file_pause file (admin_user ());
   if !!file_started_cmd <> "" then
       MlUnix.fork_and_exec  !!file_started_cmd
       [|
@@ -541,11 +568,11 @@ let start_search user query buf =
 let network_must_update n =
    CommonEvent.add_event (Network_info_event n)
 
-let network_display_stats buf o =
+let network_display_stats o =
   networks_iter_all (fun r ->
     try
       if List.mem NetworkHasStats r.network_flags then
-        network_display_stats r buf o
+        network_display_stats r o
     with _ -> ())
 
 let print_connected_servers o =
@@ -714,7 +741,7 @@ let send_custom_query user buf query args =
       | Q_MP3_BITRATE _ ->
           let bitrate = get_arg "bitrate" in
           if bitrate = "" then raise Not_found;
-          QHasMinVal(Field_UNKNOWN "bitrate", Int64.of_string bitrate)
+          QHasMinVal(Field_KNOWN "bitrate", Int64.of_string bitrate)
 
     in
     try
@@ -752,12 +779,11 @@ let sort_options l =
 
 let opfile_args r opfile =
   let prefix = r.network_shortname ^ "-" in
-  let args = simple_options prefix opfile in
-  args
+  simple_options prefix opfile true
 
 let all_simple_options () =
   let options = ref (sort_options
-      (simple_options "" downloads_ini)
+      (simple_options "" downloads_ini true)
     )
   in
   networks_iter_all (fun r ->
@@ -805,11 +831,9 @@ let all_active_network_opfile_network_names () =
        !names
 
 let apply_on_fully_qualified_options name f =
-  if !verbose then lprintf_nl "Change option %s" name;
   let rec iter prefix opfile =
-    let args = simple_options prefix opfile in
+    let args = simple_options prefix opfile true in
     List.iter (fun o ->
-(*        lprintf "Compare [%s] [%s]\n" o.option_name name; *)
         if o.option_name = name then
           (f opfile o.option_shortname o.option_value; raise Exit))
     args
@@ -821,7 +845,6 @@ let apply_on_fully_qualified_options name f =
             try
               List.iter (fun opfile ->
                   let prefix = r.network_shortname ^ "-" in
-(*                  lprintf "Prefix [%s]\n" prefix; *)
                   iter prefix opfile;
               )
               r.network_config_file ;
@@ -833,10 +856,6 @@ let apply_on_fully_qualified_options name f =
       end
   with Exit -> ()
 
-let set_fully_qualified_options name value =
-  apply_on_fully_qualified_options name (fun opfile old_name old_value ->
-      set_simple_option opfile old_name value)
-
 let get_fully_qualified_options name =
   let value = ref None in
   (try
@@ -845,14 +864,30 @@ let get_fully_qualified_options name =
       );
     with _ -> ());
   match !value with
-    None -> "????"
+    None -> "unknown"
   | Some s -> s
 
-let add_item_to_fully_qualified_options name value =
-  ()
-
-let del_item_from_fully_qualified_options name value =
-  ()
+let set_fully_qualified_options name value ?(user = None) ?(ip = None) ?(port = None) ?(gui_type = None) () =
+  let old_value = get_fully_qualified_options name in
+  apply_on_fully_qualified_options name
+    (fun opfile old_name old_value -> set_simple_option opfile old_name value);
+  if !verbose && old_value <> get_fully_qualified_options name then
+    begin
+      let ip_port_text =
+        match ip with
+        | None -> "IP unknown"
+        | Some ip ->
+            Printf.sprintf "from host %s%s" (Ip.to_string ip)
+              (match port with | None -> "" | Some port -> Printf.sprintf ":%d" port)
+      in
+      lprintf_nl "User %s changed option %s %s %s, old: %s, new %s"
+        (match user with | None -> "unknown" | Some user -> user)
+        name ip_port_text
+        (match gui_type with
+        | None -> "GUI type unknown"
+        | Some gt -> Printf.sprintf "using %s interface" (connection_type_to_text gt))
+        old_value (get_fully_qualified_options name)
+    end
 
 let keywords_of_query query =
   let keywords = ref [] in
@@ -874,14 +909,14 @@ let keywords_of_query query =
     | QHasMinVal (field, value) ->
         begin
           match field with
-            Field_UNKNOWN "bitrate"
+            Field_KNOWN "bitrate"
           | Field_Size
           | _ -> ()
         end
     | QHasMaxVal (field, value) ->
         begin
           match field with
-            Field_UNKNOWN "bitrate"
+            Field_KNOWN "bitrate"
           | Field_Size
           | _ -> ()
         end
@@ -947,7 +982,7 @@ let force_download_quotas () =
   let queue_files files =
     List.iter (fun file ->
       if file_state file = FileDownloading then
-	set_file_state file FileQueued
+	file_queue file
     ) files in
 
   let queue_user_file_list (_user, user_file_list) =
@@ -1032,7 +1067,7 @@ let force_download_quotas () =
 	      match file_state first_file with
 	      | FileDownloading -> true
 	      | FileQueued ->
-		  set_file_state first_file FileDownloading;
+		  file_resume first_file (admin_user ());
 		  true
 	      | _ -> false in
 	    if is_downloading then
