@@ -126,10 +126,10 @@ let network_disable n = n.op_network_disable ()
 let network_share n s = n.op_network_share s
 let network_recover_temp n = n.op_network_recover_temp ()
 let network_search n s buf = n.op_network_search s buf
-let network_download n r user = n.op_network_download r user
+let network_download n r user group = n.op_network_download r user group
 let network_add_server n s = n.op_network_add_server s
 let network_server_of_option n s = n.op_network_server_of_option s
-let network_file_of_option n f = n.op_network_file_of_option f 
+let network_file_of_option n f user group = n.op_network_file_of_option f user group
 let network_client_of_option n f = n.op_network_client_of_option f
 (* is returning true (successful) when an exception is caught the
    right thing to do ? *)
@@ -273,7 +273,7 @@ let new_network shortname name ?comment flags =
       op_network_update_options = (fun _ -> ni_ok name "update_options");
       op_network_disable =  (fun _ -> ni_ok name "disable");
       op_network_server_of_option =  (fun _ -> fni name "op_network_server_of_option");
-      op_network_file_of_option =  (fun _ _ _ -> fni name "op_network_file_of_option");
+      op_network_file_of_option =  (fun _ _ _ _ _ -> fni name "op_network_file_of_option");
       op_network_client_of_option =  (fun _ -> fni name "op_network_client_of_option");
       op_network_recover_temp = (fun _ -> ni_ok name "recover_temp");
       op_network_search = (fun _ _ -> ni_ok name "search");
@@ -284,12 +284,12 @@ let new_network shortname name ?comment flags =
       op_network_close_search = (fun _ -> ni_ok name "close_search");
       op_network_extend_search = (fun _ _ -> ni_ok name "extend search");
       op_network_clean_servers = (fun _ -> ni_ok name "clean servers");
-      op_network_parse_url = (fun _ _ -> ni_ok name "parse_url"; "", false);
+      op_network_parse_url = (fun _ _ _ -> ni_ok name "parse_url"; "", false);
       op_network_info = (fun _ -> fni name "network_info");
       op_network_connected = (fun _ -> ni_ok name "connected"; false);
       op_network_add_server = (fun _ -> fni name "op_network_add_server");
       op_network_gui_message = (fun _ _ -> ni_ok name "gui_message");
-      op_network_download = (fun _ _ -> fni name "network_download");
+      op_network_download = (fun _ _ _ -> fni name "network_download");
       op_network_display_stats = (fun _ -> ni_ok name "display_stats");
       op_network_stat_info_list = (fun _ -> []);
       op_network_clean_exit = (fun _ -> true);
