@@ -24,6 +24,7 @@ open BasicSocket
 open TcpBufferedSocket
 open Options
 open Unix
+open CommonTypes
 
 let log_prefix = "[cO]"
 
@@ -127,6 +128,9 @@ let windows_sleep seconds =
   lprintf_nl "waiting %d seconds to exit..." seconds;
   Unix.sleep seconds
 
+let min_reserved_fds = 50
+let min_connections = 50
+
 let _ =
   lprintf_nl "Starting MLDonkey %s ... " Autoconf.current_version;
   let ulof_old = Unix2.c_getdtablesize () in
@@ -139,8 +143,10 @@ let _ =
   let ulof = Unix2.c_getdtablesize () in
   if ulof_old <> ulof then
     lprintf_nl "raised ulimit for open files from %d to %d" ulof_old ulof;
-  if ulof < 150 then begin
-    lprintf_nl "ulimit for open files is set to %d, at least 150 is required, exiting..." ulof;
+  let absolute_min = Unix32.max_cache_size_default +
+    min_reserved_fds + min_connections in
+  if ulof < absolute_min then begin
+    lprintf_nl "ulimit for open files is set to %d, at least %d is required, exiting..." ulof absolute_min;
     exit 2
   end;
 
@@ -273,22 +279,15 @@ let _ =
     end
   end
 
-let define_option a b ?desc c d e =
+let define_option a b ?desc ?restart ?public ?internal c d e =
   match desc with
-    None -> define_option a b (_s c) d e
-  | Some desc -> define_option a b ~desc: (_s desc) (_s c) d e
-let define_expert_option a b ?desc c d e =
-  match desc with
-    None -> define_expert_option a b (_s c) d e
-  | Some desc -> define_expert_option a b ~desc: (_s desc) (_s c) d e
+    None -> define_option a b (_s c) d e ?restart ?public ?internal
+  | Some desc -> define_option a b ~desc: (_s desc) (_s c) d e ?restart ?public ?internal
 
-let string_list_option = define_option_class "String"
-    (fun v ->
-      match v with
-        List _ | SmallList _ -> ""
-      | _ -> value_to_string v
-  )
-  string_to_value
+let define_expert_option a b ?desc ?restart ?public ?internal c d e =
+  match desc with
+    None -> define_expert_option a b (_s c) d e ?restart ?public ?internal
+  | Some desc -> define_expert_option a b ~desc: (_s desc) (_s c) d e ?restart ?public ?internal
 
 let html_themes_dir = "html_themes"
 let downloads_ini = create_options_file "downloads.ini"
@@ -308,7 +307,7 @@ let ip_range_list_option = list_option Ip.range_option
 
 let int_list_option = list_option int_option
 
-let country_list_option = list_option string_option
+let string_list_option = list_option string_option
 
 let allow_browse_share_option = define_option_class "Integer"
     (fun v ->
@@ -356,7 +355,7 @@ let _ =
       let list = String2.tokens s in
       List.map (fun i -> int_of_string i) list
   );
-  Options.set_string_wrappers country_list_option
+  Options.set_string_wrappers string_list_option
   (String.concat " ")
   String2.tokens
 
@@ -478,34 +477,42 @@ let _ =
 
 let gui_port = define_option current_section ["gui_port"]
   ~desc: "The port to connect the GUI"
-  "port for Graphical Interfaces"
-    int_option 4001
+  ~restart: true
+  "port for Graphical Interfaces, 0 to deactivate GUI interface"
+    port_option 4001
 
 let gift_port = define_option current_section ["gift_port"]
   ~desc: "The port to connect for GiFT GUIs."
+  ~restart: true
   "port for GiFT Graphical Interfaces interaction. It was 1213, but the default is
   now 0 for disabled, because it does not check for a password." 
-    int_option 0
+    port_option 0
 
 let http_port = define_option current_section ["http_port"]
   ~desc: "The port to connect via HTTP"
-  "The port used to connect to your client with a WEB browser"
-    int_option 4080
+  ~public: true
+  ~restart: true
+  "The port used to connect to your client with a web browser, 0 to deactivate web interface"
+    port_option 4080
 
 let telnet_port = define_option current_section ["telnet_port"]
   ~desc: "The port to connect via telnet"
-  "port for user interaction"
-    int_option 4000
+  ~restart: true
+  "port for user interaction, 0 to deactivate telnet interface"
+    port_option 4000
 
 let http_bind_addr = define_expert_option current_section ["http_bind_addr"]
+  ~restart: true
   "The IP address used to bind the http server"
     Ip.option (Ip.any)
 
 let gui_bind_addr = define_expert_option current_section ["gui_bind_addr"]
+  ~restart: true
   "The IP address used to bind the gui server"
     Ip.option (Ip.of_inet_addr Unix.inet_addr_any)
 
 let telnet_bind_addr = define_expert_option current_section ["telnet_bind_addr"]
+  ~restart: true
   "The IP address used to bind the telnet server"
     Ip.option (Ip.of_inet_addr Unix.inet_addr_any)
 
@@ -552,8 +559,10 @@ let verbosity = define_expert_option current_section ["verbosity"]
   swarming : debug swarming
   hc : http_client messages
   hs : http_server messages
+  com : commands by non-admin users
   act : debug activity
   bw : debug bandwidth
+  geo : debug GeoIP
   unexp : debug unexpected messages"
     string_option ""
 
@@ -626,7 +635,7 @@ let max_connections_per_second = define_option current_section ["max_connections
 let loop_delay = define_expert_option current_section ["loop_delay"]
   "The delay in milliseconds to wait in the event loop. Can be decreased to
   increase the bandwidth usage, or increased to lower the CPU usage."
-    int_option 20
+    int_option 5
 
 let nolimit_ips = define_option current_section ["nolimit_ips"]
   ~desc: "No-limit IPs"
@@ -758,6 +767,10 @@ let html_mods_vd_age = define_expert_option current_section ["html_mods_vd_age"]
   "Whether to display the Age column in vd output"
     bool_option true
 
+let html_flags = define_expert_option current_section ["html_flags"]
+  "Whether to display flags instead of country codes"
+    bool_option true
+
 let html_mods_vd_gfx = define_expert_option current_section ["html_mods_vd_gfx"]
   "Show graph in vd output"
     bool_option true
@@ -807,6 +820,7 @@ let html_mods_vd_gfx_y_size = define_expert_option current_section ["html_mods_v
     int_option 200
 
 let html_mods_vd_gfx_h_intervall = define_expert_option current_section ["html_mods_vd_gfx_h_intervall"]
+  ~restart: true
   "compute values for hourly graph every 1,2,3,4,5,10,15,20,30,60 min
 	Changes to this option require a core restart."
      int_option 60
@@ -925,7 +939,7 @@ let html_mods_theme = define_option current_section ["html_mods_theme"]
     string_option ""
 
 let use_html_mods o =
-  o.CommonTypes.conn_output = CommonTypes.HTML && !!html_mods
+  o.conn_output = HTML && !!html_mods
 
 let html_checkbox_vd_file_list = define_expert_option current_section ["html_checkbox_vd_file_list"]
   "Whether to use checkboxes in the WEB interface for download list"
@@ -975,6 +989,10 @@ let force_client_ip = define_option current_section ["force_client_ip"]
   ourself. Don't set this option to true if you have dynamic IP."
     bool_option false
 
+let discover_ip = define_option current_section ["discover_ip"]
+  "Use http://ip.discoveryvip.com/ip.asp to obtain WAN IP"
+    bool_option true
+
 let user_agent = define_option current_section ["user_agent"]
   "User agent string (default = \"default\")"
     string_option "default"
@@ -1001,9 +1019,11 @@ let web_infos = define_option current_section ["web_infos"]
   EXAMPLE:
     web_infos = [
     (\"server.met\", 0, \"http://www.gruk.org/server.met.gz\");
+    (\"hublist\", 0, \"http://dchublist.com/hublist.config.bz2\");
     (\"guarding.p2p\", 96, \"http://www.bluetack.co.uk/config/level1.gz\");
     (\"ocl\", 24, \"http://members.lycos.co.uk/appbyhp2/FlockHelpApp/contact-files/contact.ocl\");
     (\"contact.dat\", 168, \"http://download.overnet.org/contact.dat\");
+    (\"geoip.dat\", 168, \"http://www.maxmind.com/download/geoip/database/GeoIP.dat.gz\");
     ]
   "
     (list_option (tuple3_option (string_option, int_option, string_option)))
@@ -1018,6 +1038,8 @@ let web_infos = define_option current_section ["web_infos"]
         "http://www.maxmind.com/download/geoip/database/GeoIP.dat.gz");
       ("nodes.gzip", 0,
         "http://update.kceasy.com/update/fasttrack/nodes.gzip");
+      ("hublist", 0,
+        "http://dchublist.com/hublist.config.bz2");
 (*
     ("slsk_boot", 0,
       "http://www.slsknet.org/slskinfo2");
@@ -1045,7 +1067,7 @@ let ip_blocking_countries = define_expert_option current_section ["ip_blocking_c
   "List of countries to block connections from/to (requires Geoip).
   Names are in ISO 3166 format, see http://www.maxmind.com/app/iso3166
   You can also at your own risk use \"Unknown\" for IPs Geoip won't recognize."
-    country_list_option []
+    string_list_option []
 
 let ip_blocking_countries_block = define_expert_option current_section ["ip_blocking_countries_block"]
   "false: use ip_blocking_countries as block list, all other countries are allowed
@@ -1070,10 +1092,6 @@ let mtu_packet_size = define_expert_option current_section ["mtu_packet_size"]
   "The size of the MTU of a TCP/IP packet on your connection"
     int_option 1500
 
-let packet_frame_size = define_expert_option current_section ["packet_frame_size"]
-  "The size of the frame packet on your network (on my cable link, it is 250)"
-    int_option 250
-
 let minimal_packet_size = define_expert_option current_section ["minimal_packet_size"]
   "The size of the minimal packet you want mldonkey to send when data is
   available on the connection"
@@ -1098,7 +1116,7 @@ let http_proxy_server = define_option current_section ["http_proxy_server"]
 
 let http_proxy_port = define_option current_section ["http_proxy_port"]
   "Port of HTTP proxy"
-    int_option 8080
+    port_option 8080
 
 let http_proxy_tcp = define_option current_section ["http_proxy_tcp"]
   "Direct TCP connections to HTTP proxy (the proxy should support CONNECT)"
@@ -1121,7 +1139,7 @@ let smtp_server = define_option current_section ["smtp_server"]
 
 let smtp_port = define_option current_section ["smtp_port"]
   "The port to use on the mail server (default 25)"
-    int_option 25
+    port_option 25
 
 let mail = define_option current_section ["mail"]
   "Your e-mail if you want to receive mails when downloads are completed"
@@ -1176,15 +1194,6 @@ let sources_per_chunk = define_expert_option current_section ["sources_per_chunk
   "How many sources to use to download each chunk"
     int_option 3
 
-let swarming_block_selection_algorithm = define_expert_option current_section ["swarming_block_selection_algorithm"]
-  "What algorithm to use to select blocks (currently 1 or 2)"
-    int_option 1
-
-let block_switching = define_expert_option current_section ["block_switching"]
-  "Allows swarmer to switch a source to another block if current block is already
-  totally selected by other sources"
-    bool_option true
-
 let max_recover_gap = define_option current_section ["max_recover_zeroes_gap"]
   "The maximal length of zero bytes between non-zero bytes in a file that
   should be interpreted as downloaded during a recovery"
@@ -1225,12 +1234,14 @@ let file_started_cmd = define_option current_section ["file_started_cmd"]
 let current_section = startup_section
 
 let run_as_user = define_option current_section ["run_as_user"]
+  ~restart: true
   "The login of the user you want mldonkey to run as, after the ports
   have been bound (can be use not to run with root priviledges when
   a port < 1024 is needed)"
     string_option ""
 
 let run_as_useruid = define_option current_section ["run_as_useruid"]
+  ~restart: true
   "The UID of the user (0=disabled) you want mldonkey to run as, after the ports
   have been bound (can be use not to run with root priviledges when
   a port < 1024 is needed)"
@@ -1249,6 +1260,7 @@ let recover_temp_on_startup = define_option current_section ["recover_temp_on_st
     bool_option true
 
 let config_files_security_space = define_expert_option current_section ["config_files_security_space"]
+  ~restart: true
   "How many megabytes should MLdonkey keep for saving configuration files."
     int_option 10
 
@@ -1267,8 +1279,19 @@ let temp_directory = define_option current_section ["temp_directory"]
   "The directory where temporary files should be put"
     string_option "temp"
 
-let create_dir_mask = define_option current_section ["create_dir_mask"]
-  "New directories in incoming_directories are created with these rights"
+let share_scan_interval = define_option current_section ["share_scan_interval"]
+  ~restart: true
+  "How often (in minutes) should MLDonkey scan all shared directories for new/removed files.
+  Minimum 5, 0 to disable. Use command reshare to manually scan shares.
+  When core starts, shared directories are scanned once, independent of this option."
+    int_option 30
+
+let create_file_mode = define_option current_section ["create_file_mode"]
+  "New download files are created with these rights (in octal)"
+    string_option "664"
+
+let create_dir_mode = define_option current_section ["create_dir_mode"]
+  "New directories in incoming_directories are created with these rights (in octal)"
     string_option "755"
 
 let create_file_sparse = define_option current_section ["create_file_sparse"]
@@ -1340,7 +1363,7 @@ let allow_browse_share = define_option current_section ["allow_browse_share"]
 
 let messages_filter = define_option current_section ["messages_filter"]
   "Regexp of messages to filter out, example: string1|string2|string3"
-    string_option "Your client is connecting too fast"
+    string_option "DI-Emule|ZamBoR|Ketamine|eMule FX|AUTOMATED MESSAGE|Hi Honey!|Do you live in my area|download HyperMule"
 
 let comments_filter = define_option current_section ["comments_filter"]
   "Regexp of comments to filter out, example: string1|string2|string3"
@@ -1361,19 +1384,12 @@ let save_results = define_option current_section ["save_results"]
   "(experimental)" 
     int_option 0
 
-let use_result_history = define_expert_option current_section ["use_file_history"]
-  "keep seen files in history to allow local search (can be expensive in memory)"
-    bool_option false
-
-let filters = define_option current_section ["filters"]
-  "filters on replies (replies will be kept)."
-    string_list_option ""
-
 let buffer_writes = define_option current_section ["buffer_writes"]
   "Buffer writes and flush after buffer_writes_delay seconds (experimental)"
     bool_option false
 
 let buffer_writes_delay = define_expert_option current_section ["buffer_writes_delay"]
+  ~restart: true
   "Buffer writes and flush after buffer_writes_delay seconds (experimental)"
     float_option 30.
 
@@ -1484,6 +1500,7 @@ let vd_reload_delay = define_expert_option current_section ["vd_reload_delay"]
     int_option 120
 
 let client_bind_addr = define_option current_section ["client_bind_addr"]
+  ~restart: true
   "The IP address used to bind the p2p clients"
     Ip.option (Ip.of_inet_addr Unix.inet_addr_any)
 
@@ -1497,9 +1514,12 @@ let _ =
       TcpBufferedSocket.copy_read_buffer := !!copy_read_buffer
   )
 
-let _ =
-  option_hook create_dir_mask (fun _ ->
-      Unix32.create_dir_mask := !!create_dir_mask
+let () =
+  option_hook create_file_mode (fun _ ->
+      Unix32.create_file_mode := Misc.int_of_octal_string !!create_file_mode
+  );
+  option_hook create_dir_mode (fun _ ->
+      Unix32.create_dir_mode := Misc.int_of_octal_string !!create_dir_mode
   )
 
 let create_mlsubmit = define_expert_option current_section ["create_mlsubmit"]
@@ -1527,10 +1547,12 @@ let filter_table_threshold = define_expert_option current_section ["filter_table
     int_option 50
 
 let client_buffer_size = define_expert_option current_section ["client_buffer_size"]
-  "Maximal size of the buffers of a client"
+  "Maximal size in byte of the buffers of a client, minimum 50.000 byte.
+For high-volume links raise this value to 1.000.000 or higher."
     int_option 500000
 
 let save_options_delay = define_expert_option current_section ["save_options_delay"]
+  ~restart: true
   "The delay between two saves of the 'downloads.ini' file (default is 15 minutes).
   Changes to this option require a core restart."
     float_option 900.0
@@ -1540,12 +1562,13 @@ let server_connection_timeout = define_expert_option current_section ["server_co
     float_option 30.
 
 let download_sample_rate = define_expert_option current_section ["download_sample_rate"]
+  ~restart: true
   "The delay between one glance at a file and another"
     float_option 1.
 
 let download_sample_size = define_expert_option current_section ["download_sample_size"]
   "How many samples go into an estimate of transfer rates"
-    int_option 10
+    int_option 100
 
 let calendar = define_expert_option current_section ["calendar"]
   "This option defines a set of date at which some commands have to be executed.
@@ -1558,7 +1581,7 @@ let calendar = define_expert_option current_section ["calendar"]
 
 let compaction_overhead = define_expert_option current_section ["compaction_overhead"]
   "The percentage of free memory before a compaction is triggered"
-    percent_option 25
+    int_option 25
 
 let space_overhead = define_expert_option current_section ["space_overhead"]
   "The major GC speed is computed from this parameter. This is the memory
@@ -1573,9 +1596,9 @@ let max_displayed_results = define_expert_option current_section ["max_displayed
     int_option 1000
 
 let options_version = define_expert_option current_section ["options_version"]
+  ~internal: true
   "(internal option)"
-    int_option 14
-
+    int_option 20
 
 let max_comments_per_file = define_expert_option current_section ["max_comments_per_file"]
   "Maximum number of comments per file"
@@ -1643,7 +1666,7 @@ let client_ip sock =
   else
     if !last_high_id <> Ip.null then
       begin
-        if !last_high_id <> Ip.localhost && !!set_client_ip <> !last_high_id then
+        if Ip.usable !last_high_id && !!set_client_ip <> !last_high_id then
           set_client_ip =:= !last_high_id;
         !last_high_id
       end
@@ -1652,8 +1675,8 @@ let client_ip sock =
         None -> !!set_client_ip
       | Some sock ->
           let ip = TcpBufferedSocket.my_ip sock in
-          if ip <> Ip.localhost && !!set_client_ip <> ip then
-          set_client_ip =:= ip;
+          if Ip.usable ip && !!set_client_ip <> ip then
+            set_client_ip =:= ip;
           ip
 
 let start_running_plugins = ref false
@@ -1708,6 +1731,9 @@ let _ =
   option_hook min_reask_delay (fun _ ->
     if !!min_reask_delay < 600 then min_reask_delay =:= 600
   );
+  option_hook share_scan_interval (fun _ ->
+    if !!share_scan_interval < 5 && !!share_scan_interval <> 0 then share_scan_interval =:= 5
+  );
   option_hook global_login (fun _ ->
       let len = String.length !!global_login in
       let prefix = "mldonkey_" in
@@ -1742,11 +1768,6 @@ let _ =
           close_log ()
         end
   );
-  option_hook swarming_block_selection_algorithm (fun _ ->
-    match !!swarming_block_selection_algorithm with
-    | 1 | 2 -> ()
-    | _ -> swarming_block_selection_algorithm =:= 1;
-  );
   option_hook max_upload_slots (fun _ ->
       if !!max_upload_slots < 3 then
         max_upload_slots =:= 3);
@@ -1769,16 +1790,6 @@ let _ =
       let gc_control = Gc.get () in
       Gc.set { gc_control with Gc.space_overhead = !!space_overhead };
   );
-  option_hook web_infos (fun _ ->
-      List.iter (fun remove ->
-          if List.mem remove !!web_infos then
-            web_infos =:= List2.remove remove !!web_infos
-      )
-      [
-        ("server.met", 1, "http://savannah.nongnu.org/download/mldonkey/network/servers.met");
-        ("ocl",1, "http://savannah.nongnu.org/download/mldonkey/network/peers.ocl");
-      ]
-  );
   option_hook tcpip_packet_size (fun _ ->
       TcpBufferedSocket.ip_packet_size := !!tcpip_packet_size
   );
@@ -1788,16 +1799,13 @@ let _ =
   option_hook minimal_packet_size (fun _ ->
       TcpBufferedSocket.minimal_packet_size := !!minimal_packet_size
   );
-  option_hook packet_frame_size (fun _ ->
-      TcpBufferedSocket.packet_frame_size := !!packet_frame_size
-  );
   option_hook minor_heap_size (fun _ ->
       let gc_control = Gc.get () in
       Gc.set { gc_control with Gc.minor_heap_size =
         (!!minor_heap_size * 1024) };
   );
   option_hook client_buffer_size (fun _ ->
-      TcpBufferedSocket.max_buffer_size := max 10000000 !!client_buffer_size
+      TcpBufferedSocket.max_buffer_size := max 50000 !!client_buffer_size
   );
   if Autoconf.has_gd then begin
     option_hook html_mods_vd_gfx_png (fun _ ->
@@ -1830,6 +1838,8 @@ let verbose_udp = ref false
 let verbose_supernode = ref false
 let verbose_swarming = ref false
 let verbose_activity = ref false
+let verbose_user_commands = ref false
+let verbose_geoip = ref false
 let verbose_unexpected_messages = ref false
 
 let set_all v =
@@ -1858,6 +1868,8 @@ let set_all v =
   Http_client.verbose := v;
   Http_server.verbose := v;
   verbose_activity := v;
+  verbose_user_commands := v;
+  Geoip.verbose := v;
   verbose_unexpected_messages := v
 
 let _ =
@@ -1893,6 +1905,8 @@ let _ =
           | "act" -> verbose_activity := true
           | "bw" -> incr BasicSocket.verbose_bandwidth
           | "unexp" -> verbose_unexpected_messages := true
+          | "com" -> verbose_user_commands := true
+          | "geo" -> Geoip.verbose := true
 
           | "all" ->
 
@@ -2007,21 +2021,54 @@ let _ =
   option_hook allow_local_network (fun _ ->
       Ip.allow_local_network := !!allow_local_network)
 
-let web_infos_exists url =
-  List.exists (fun (_, _, weburl) ->
-    weburl = url) !!web_infos
+let web_infos_table = Hashtbl.create 10
 
-let web_infos_remove outdated_web_infos =
-  web_infos =:=
-  List.fold_left (fun acc owi ->
-    let (kind2, _, url2) = owi in
-    List.filter (fun (_,_,url) -> url <> url2) acc
-  ) !!web_infos outdated_web_infos
+exception Found_web_infos of web_infos
+
+let web_infos_find url =
+  let found = ref None in
+  (try
+    Hashtbl.iter (fun key w ->
+      if w.url = url then raise (Found_web_infos w)
+    ) web_infos_table
+  with Found_web_infos w -> found := Some w);
+  !found
+
+let web_infos_remove url =
+  let delete_list = ref [] in
+  Hashtbl.iter (fun key w -> 
+    if w.url = url then delete_list := !delete_list @ [key]
+  ) web_infos_table;
+  List.iter (fun key -> Hashtbl.remove web_infos_table key) !delete_list
 
 let web_infos_add kind period url =
-  let web_info = (kind,period,url) in
-  if web_infos_exists url then web_infos_remove [web_info];
-  web_infos =:=  web_info :: !!web_infos
+  (match web_infos_find url with
+  | None -> ()
+  | Some w -> web_infos_remove w.url);
+  Hashtbl.add web_infos_table (kind, period, url)
+    {
+      kind = kind;
+      period = period;
+      url = url;
+      state = None;
+    }
+
+let _ =
+(* convert list option web_infos to a hashtable for better usage *)
+  set_after_load_hook downloads_ini (fun _ ->
+    List.iter (fun (kind, period, url) ->
+      web_infos_add kind period url
+    ) !!web_infos;
+    web_infos =:= []
+  );
+  set_before_save_hook downloads_ini (fun _ ->
+    Hashtbl.iter (fun _ w ->
+      web_infos =:= !!web_infos @ [(w.kind, w.period, w.url)]
+    ) web_infos_table
+  );
+  set_after_save_hook downloads_ini (fun _ ->
+    web_infos =:= []
+  )
 
 let rec update_options () =
   let update v =
@@ -2033,16 +2080,11 @@ let rec update_options () =
   match !!options_version with
     0 ->
       web_infos =:= List.map (fun (kind, period, url) ->
-          kind, period * 24, url
+          kind, period * Date.day_in_hours, url
       ) !!web_infos;
-      web_infos =:= !!web_infos @ [
-        ("rss", 6,
-          "http://www.ed2k-it.com/forum/news_rss.php");
-        ("rss", 6,
-          "http://www.torrents.co.uk/backend.php");
-        ("rss", 6,
-          "http://varchars.com/rss/suprnova-movies.rss");
-      ];
+      web_infos_add "rss" 6 "http://www.ed2k-it.com/forum/news_rss.php";
+      web_infos_add "rss" 6 "http://www.torrents.co.uk/backend.php";
+      web_infos_add "rss" 6 "http://varchars.com/rss/suprnova-movies.rss";
       update 1
 
   | 1 ->
@@ -2051,34 +2093,19 @@ let rec update_options () =
       update 2
 
   | 2 ->
-      web_infos_remove
-        [
-          ("rss", 6,
-            "http://www.ed2k-it.com/forum/news_rss.php");
-          ("rss", 6,
-            "http://www.torrents.co.uk/backend.php");
-          ("rss", 6,
-            "http://varchars.com/rss/suprnova-movies.rss");
-        ];
-      if !!min_reask_delay = 720 then
-          min_reask_delay =:= 600;
+      web_infos_remove "http://www.ed2k-it.com/forum/news_rss.php";
+      web_infos_remove "http://www.torrents.co.uk/backend.php";
+      web_infos_remove "http://varchars.com/rss/suprnova-movies.rss";
+      if !!min_reask_delay = 720 then min_reask_delay =:= 600;
       update 3
 
   | 3 ->
-      web_infos_remove
-        [
-          ("ocl", 24,
-            "http://members.lycos.co.uk/appbyhp2/FlockHelpApp/contact-files/contact.ocl");
-        ];
+      web_infos_remove "http://members.lycos.co.uk/appbyhp2/FlockHelpApp/contact-files/contact.ocl";
       web_infos_add "contact.dat" 168 "http://www.overnet.org/download/contact.dat";
       update 4
 
   | 4 ->
-      web_infos_remove
-        [
-          ("server.met", 24,
-            "http://ocbmaurice.dyndns.org/pl/slist.pl/server.met?download/server-best.met");
-        ];
+      web_infos_remove "http://ocbmaurice.dyndns.org/pl/slist.pl/server.met?download/server-best.met";
       web_infos_add "server.met" 0 "http://www.gruk.org/server.met.gz";
       update 5
 
@@ -2102,38 +2129,18 @@ let rec update_options () =
       update 9
 
   | 9 ->
-      if web_infos_exists "http://www.gruk.org/server.met.gz" then
-      begin
-        web_infos_remove
-          [
-      ("server.met", 0, "http://www.gruk.org/server.met.gz")
-          ];
-  web_infos_add "server.met" 0 "http://www.jd2k.com/server.met";
-      end;
+      web_infos_remove "http://www.gruk.org/server.met.gz";
+      web_infos_add "server.met" 0 "http://www.jd2k.com/server.met";
       update 10
 
   | 10 ->
-      if web_infos_exists "http://www.overnet.org/download/contact.dat" then
-      begin
-        web_infos_remove
-          [
-      ("contact.dat", 672,
-        "http://www.overnet.org/download/contact.dat");
-          ];
-  web_infos_add "contact.dat" 168 "http://download.overnet.org/contact.dat";
-      end;
+      web_infos_remove "http://www.overnet.org/download/contact.dat";
+      web_infos_add "contact.dat" 168 "http://download.overnet.org/contact.dat";
       update 11
 
   | 11 ->
-      if web_infos_exists "http://www.bluetack.co.uk/config/antip2p.txt" then
-      begin
-        web_infos_remove
-          [
-      ("guarding.p2p", 96,
-        "http://www.bluetack.co.uk/config/antip2p.txt");
-          ];
-  web_infos_add "guarding.p2p" 0 "http://www.bluetack.co.uk/config/level1.gz";
-      end;
+      web_infos_remove "http://www.bluetack.co.uk/config/antip2p.txt";
+      web_infos_add "guarding.p2p" 0 "http://www.bluetack.co.uk/config/level1.gz";
       update 12
 
   | 12 ->
@@ -2141,12 +2148,35 @@ let rec update_options () =
       update 13
 
   | 13 ->
-      if web_infos_exists "http://www.jd2k.com/server.met" then
-      begin
-        web_infos_remove
-          [("server.met", 0, "http://www.jd2k.com/server.met")];
-	web_infos_add "server.met" 0 "http://www.gruk.org/server.met.gz";
-      end;
+      web_infos_remove "http://www.jd2k.com/server.met";
+      web_infos_add "server.met" 0 "http://www.gruk.org/server.met.gz";
       update 14
 
+  | 14 ->
+      (* set back to 5 because dynamic_loop_delay patch was removed *)
+      loop_delay =:= 5;
+      update 15
+
+  | 15 ->
+      if !!messages_filter = "Your client is connecting too fast" then
+        messages_filter =:= "DI-Emule|ZamBoR|Ketamine|eMule FX|AUTOMATED MESSAGE";
+      update 16
+
+  | 16 ->
+      if !!download_sample_size = 10 then download_sample_size =:= 100;
+      update 17
+
+  | 17 ->
+      web_infos_add "hublist" 0 "http://dchublist.com/hublist.config.bz2";
+      update 18
+
+  | 18 ->
+      if !!messages_filter = "DI-Emule|ZamBoR|Ketamine|eMule FX|AUTOMATED MESSAGE" then
+        messages_filter =:= "DI-Emule|ZamBoR|Ketamine|eMule FX|AUTOMATED MESSAGE|Hi Honey!|Do you live in my area|download HyperMule";
+      update 19
+
+  | 19 ->
+      if !!share_scan_interval = 5 then share_scan_interval =:= 30;
+      update 20
+
   | _ -> ()
