@@ -100,6 +100,7 @@ let impl_server_info impl =
     T.server_addr = Ip.addr_of_ip Ip.null;
     T.server_port = 0;
     T.server_realport = 0;
+    T.server_country_code = None;
     T.server_score = 0;
     T.server_tags = [];
     T.server_nusers = 0L;
@@ -272,7 +273,7 @@ let server_find (num : int) =
 
 let server_blocked s =
   let info = server_info s in
-  !Ip.banned (Ip.ip_of_addr info.G.server_addr) <> None
+  !Ip.banned (Ip.ip_of_addr info.G.server_addr, info.G.server_country_code) <> None
 
 let server_connect s =
   if not (server_blocked s) then
@@ -306,8 +307,8 @@ let server_sort () =
             (try impl.impl_server_ops.op_server_sort impl.impl_server_val
             with _ -> 0);
   ) servers_by_num;
-  Sort.list (fun s1 s2 ->
-      (as_server_impl s1).impl_server_sort >= (as_server_impl s2).impl_server_sort
+  List.sort (fun s1 s2 ->
+    compare (as_server_impl s2).impl_server_sort (as_server_impl s1).impl_server_sort
   ) !list
 
 let server_iter f =
@@ -385,7 +386,7 @@ let server_print_html_header buf ext =
     ( "0", "srh", "Network name", "Network" ) ;
     ( "0", "srh", "Connection status", "Status" ) ;
     ( "0", "srh br", "IP address", "IP address" ) ;
-    ] @ (if !Geoip.active then [( "0", "srh br", "Country Code/Name", "CC" )] else []) @ [
+    ] @ (if Geoip.active () then [( "0", "srh br", "Country Code/Name", "CC" )] else []) @ [
     ( "1", "srh ar", "Number of connected users", "Users" ) ;
     ( "1", "srh ar br", "Max number of users", "MaxUsers" ) ;
     ( "1", "srh ar br", "LowID users", "LowID" ) ;
@@ -408,7 +409,7 @@ let server_print s o =
           lprintf_nl "Exception %s in server_info (%s)\n"
             (Printexc2.to_string e) n.network_name;
           raise e in
-    let cc,cn = Geoip.get_country (Ip.ip_of_addr info.G.server_addr) in
+    let cc,cn = Geoip.get_country_code_name info.G.server_country_code in
     let buf = o.conn_buf in
   
   if use_html_mods o then begin
@@ -514,14 +515,13 @@ let server_print s o =
       | _ -> string_of_connection_state impl.impl_server_state
     in
 
-    let cc,cn = Geoip.get_country (Ip.ip_of_addr info.G.server_addr) in
     html_mods_td buf ([
       ("", "srb", if info.G.server_master then "M" else "-");
       (id_title, "sr", id_text);
       ("", "sr", n.network_name);
       ("", "sr", server_state_string);
       ("", "sr br", ip_port_string);
-      ] @ (if !Geoip.active then [(cn, "sr br", cc)] else []) @ [
+      ] @ (if Geoip.active () then [(cn, "sr br", CommonPictures.flag_html cc)] else []) @ [
       ("", "sr ar", if info.G.server_nusers = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_nusers);
       ("", "sr ar br", if info.G.server_max_users = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_max_users);
       ("", "sr ar br", if info.G.server_lowid_users = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_lowid_users);
@@ -559,7 +559,7 @@ onClick=\\\"location.href='submit?q=server_shares+%d'\\\"\\>%d\\</TD\\>"
             then "(" ^ (string_of_int info.G.server_realport) ^ ")" 
             else ""))
           (info.G.server_name)
-	  (if !Geoip.active then Printf.sprintf "%33s/%-2s%9s" cn cc "" else "")
+	  (if Geoip.active () then Printf.sprintf "%33s/%-2s%9s" cn cc "" else "")
           (info.G.server_nusers)
           (info.G.server_nfiles)
           (if server_blocked s 
