@@ -88,6 +88,10 @@ let file_string_of_uid uid =
   string_of_uid_sep uid "_"
 
 exception Illegal_urn of string
+exception Torrent_started of string
+exception Torrent_already_exists of string
+exception Torrent_can_not_be_used of string
+
 let uid_of_string s =
   let s = String.lowercase s in
   let urn = String2.before s 4 in
@@ -257,11 +261,13 @@ type field =
 | Field_Completesources (* "completesources" *)
 | Field_Filename (* "filename" *)
 | Field_Size
+| Field_Size_Hi
 | Field_Uid
 | Field_Filerating
 | Field_Lastseencomplete
 | Field_Mediacodec
 | Field_Medialength
+| Field_KNOWN of string
 | Field_UNKNOWN of string
   
 type tag = {
@@ -322,10 +328,10 @@ type host_state =
 | Connected of int    (* >= 0 Queued *)
 | Connected_downloading of int
 (* | ConnectionWaiting of int  *)
-  
 | NewHost
 | RemovedHost
 | BlackListedHost
+| ServerFull
 
 type compressor = 
   Deflate of Zlib.stream * Zlib.stream
@@ -347,7 +353,7 @@ type sharing_strategy = {
 
 type shared_directory = {
     shdir_dirname : string;
-    shdir_priority : int;
+    mutable shdir_priority : int;
     shdir_strategy : string;
     shdir_networks : string list;
   }
@@ -360,7 +366,17 @@ type connection_control = {
   }
 
 type output_type = TEXT | HTML | ANSI | XML | XHTML
-  
+
+type connection_type = TELNET | WEB | GUI | GIFT | CALENDAR
+
+let connection_type_to_text ct =
+  match ct with
+  | TELNET -> "Telnet"
+  | WEB -> "Web"
+  | GUI -> "GUI"
+  | GIFT -> "Gift"
+  | CALENDAR -> "Calendar"
+
 type sortvd_type = 
   BySize
 | ByName
@@ -376,6 +392,8 @@ type sortvd_type =
 | ByNet
 | ByAvail
 | ByComments
+| ByUser
+| ByGroup
 | NotSorted
   
 type room_state = 
@@ -442,17 +460,48 @@ type network_info = {
     network_netnum : int;
     network_config_filename : string;
     network_netflags : network_flag list;
-    mutable network_enabled : bool;
-    mutable network_uploaded : int64;
-    mutable network_downloaded : int64;
-    mutable network_connected : int; (* number of connected servers *)
+    network_enabled : bool;
+    network_uploaded : int64;
+    network_downloaded : int64;
+    network_connected_servers : int;
   }
 
 type extend_search =
   ExtendSearchLocally
 | ExtendSearchRemotely
   
-type network = {
+type network_stat_info  = {
+  mutable string_long : string;
+  mutable string_short : string;
+  mutable seen : int;
+  mutable banned : int;
+  mutable filerequest : int;
+  mutable download : Int64.t;
+  mutable upload : Int64.t;
+}
+
+type network_porttest =
+  PorttestNotAvailable
+| PorttestNotStarted
+| PorttestInProgress of int
+| PorttestResult of int * string
+
+type groupdb = {
+  group_name : string;
+  mutable group_admin : bool;
+}
+
+and userdb = {
+  user_name : string;
+  mutable user_pass : Md4.t;
+  mutable user_groups : groupdb list;
+  mutable user_default_group : groupdb option;
+  mutable user_mail : string;
+  mutable user_commit_dir : string;
+  mutable user_max_concurrent_downloads : int;
+}
+
+and network = {
     network_name : string;
     network_num : int;
     network_connection_manager : TcpBufferedSocket.connection_manager;
@@ -473,14 +522,14 @@ type network = {
     mutable op_network_add_server : 
       (Ip.addr -> int -> server);
     mutable op_network_file_of_option : 
-      int64 -> file_state -> ((string * Options.option_value) list -> file);
+      int64 -> file_state -> userdb -> groupdb option -> ((string * Options.option_value) list -> file);
     mutable op_network_client_of_option : 
       bool -> ((string * Options.option_value) list -> client);
     mutable op_network_recover_temp : (unit -> unit);
     mutable op_network_share : (
       string -> string -> int64 -> unit);
     mutable op_network_private_message : (string -> string -> unit);
-    mutable op_network_parse_url : (string -> string * bool);
+    mutable op_network_parse_url : (string -> userdb -> groupdb option -> string * bool);
     mutable op_network_connect_servers : (unit -> unit);
     
     mutable op_network_search : (search -> Buffer.t -> unit);
@@ -492,17 +541,20 @@ type network = {
     mutable op_network_info : (unit -> network_info);
     
     mutable op_network_connected : (unit -> bool);
-    mutable op_network_gui_message : (string -> unit);
+    mutable op_network_gui_message : (string -> userdb -> unit);
     
-    mutable op_network_download : (result_info -> file);
-    mutable op_network_display_stats : (Buffer.t -> ui_conn -> unit);
+    mutable op_network_download : (result_info -> userdb -> groupdb option -> file);
+    mutable op_network_display_stats : (ui_conn -> unit);
+    mutable op_network_stat_info_list : unit -> (string * int * (network_stat_info list)) list;
     mutable op_network_clean_exit : (unit -> bool);
     mutable op_network_reset : (unit -> unit);
     mutable op_network_ports : (unit -> (int * string) list);
+    mutable op_network_porttest_start : (unit -> unit);
+    mutable op_network_porttest_result : (unit -> network_porttest);
   }
 
-and   ui_user = {
-    ui_user_name : string;
+and ui_user = {
+    ui_user : userdb;
     mutable ui_user_searches : search list;
     mutable ui_last_search : search option;
     mutable ui_last_results : (int * result) list;
@@ -517,6 +569,7 @@ and ui_conn = {
     mutable conn_user : ui_user;
     mutable conn_width : int;
     mutable conn_height : int;
+    mutable conn_info : (connection_type * (Ip.t * int)) option;
   }
 
   
@@ -788,6 +841,7 @@ let is_connected state =
   | Connecting
   | NewHost
   | BlackListedHost
+  | ServerFull
   | RemovedHost -> false
 
 let string_of_connection_state s = 
@@ -811,6 +865,7 @@ let string_of_connection_state s =
   | RemovedHost -> "Removed"
   | BlackListedHost -> "Black"
   | NewHost -> "New"
+  | ServerFull -> "Server full"
       
 let short_string_of_connection_state s = 
   match s with
@@ -828,6 +883,7 @@ let short_string_of_connection_state s =
   | RemovedHost -> "Rem"
   | BlackListedHost -> "BL"
   | NewHost -> "New"
+  | ServerFull -> "Full"
     
 exception IgnoreNetwork
   
@@ -855,15 +911,23 @@ let string_of_kind kind =
     | Indirect_location (server_ip, server_port, ip, port) -> Ip.to_string ip
   with _ -> ""
 
-let string_of_kind_geo kind =
+let string_of_kind_geo kind cc =
+  let ip_cn_cc ip =
+    match cc with
+    | None ->
+        let ccode, cname = Geoip.get_country ip in
+        Ip.to_string ip, ccode, cname
+    | Some cc ->
+        Ip.to_string ip, 
+        Geoip.country_code_array.(cc),
+        Geoip.country_name_array.(cc)
+  in
   try
     match kind with
     | Known_location (ip,port) -> 
-        let cc,cn = Geoip.get_country ip in
-        (Ip.to_string ip),cc,cn
+        ip_cn_cc ip
     | Indirect_location (server_ip, server_port, ip, port) ->
-        let cc,cn = Geoip.get_country ip in
-        (Ip.to_string ip),cc,cn
+        ip_cn_cc ip
   with _ -> "","X","Country error"
 
 type brand_stat = {
@@ -883,7 +947,59 @@ let dummy_stats =
     brand_upload = 0L;
   }
 
+type country_stats =
+  {
+    country_code : string;
+    country_name : string;
+    country_continent : string;
+    mutable country_session_upload : int64;
+    mutable country_session_download : int64;
+    mutable country_session_seen : int64;
+    mutable country_total_upload : int64;
+    mutable country_total_download : int64;
+    mutable country_total_seen : int64;
+  }
+
 type kind_type = {
   f : string -> string -> unit;
   description : string
 }
+
+type slot_kind =
+  NoSlot
+| FriendSlot
+| ReleaseSlot
+| SmallFileSlot
+| NormalSlot
+| PrioSlot of string
+
+let string_of_slot_kind slot_kind short =
+  match slot_kind with
+    NoSlot -> "NoSlot"
+  | FriendSlot -> "FriendSlot"
+  | ReleaseSlot -> "ReleaseSlot"
+  | SmallFileSlot -> "SmallFileSlot"
+  | NormalSlot -> if short then "" else "NormalSlot"
+  | PrioSlot dir -> Printf.sprintf "Prio %s" dir
+
+type swarming_strategy =
+  LinearStrategy    (* one after the other one *)
+| AdvancedStrategy
+
+type web_infos_state =
+| DownloadStarted
+| FileLoaded
+
+type web_infos = {
+  kind : string;
+  period : int;
+  url : string;
+  mutable state : web_infos_state option;
+}
+
+let string_of_web_infos_state state =
+  match state with
+  | None -> "unknown"
+  | Some DownloadStarted -> "DL started"
+  | Some FileLoaded -> "File loaded"
+
