@@ -687,7 +687,7 @@ let add_shared full_name codedname size =
           impl_shared_ops = shared_ops;
           impl_shared_val = sh;
           impl_shared_requests = 0;
-          impl_shared_magic = None;
+          impl_shared_file = None;
           impl_shared_servers = [];
         }
       and sh = {
@@ -751,23 +751,6 @@ let streaming_amount () =
 let streaming_left = ref (streaming_amount ())
 let streaming_time = (ref None : float option ref)
 
-let has_upload = ref 0
-let upload_credit = ref 0
-
-
-let can_write_len sock len =
-  let bool1 = can_write_len sock len in
-(* changed 2.5.24: Don't put in a socket more than 10 seconds of upload. *)
-(* ...since the socket shares available bandwidth with others, 
-   that test has no physical meaning. Should it simply be removed ? *)
-  let bool2 = not_buffer_more sock (int_of_float !CommonGlobals.payload_bandwidth * 10) in
-(*  lprintf "can_write_len %b %b\n" bool1 bool2; *)
-  let b = bool1 && bool2 in
-(*  if not b then
-      lprintf "bool1 %b len %d bool2 %b upload %d\n" 
-        bool1 len bool2 upload_rate; *)
-  b
-
 let next_uploads () =
 
   let rec next_uploads_aux () =
@@ -811,11 +794,6 @@ let next_uploads () =
   streaming_time := Some new_streaming_time;
   next_uploads_aux ()
 
-let reset_upload_timer () = ()
-
-let reset_upload_timer _ =
-  reset_upload_timer ()
-
 let upload_credit_timer _ =
   if !has_upload = 0 then
     (if !upload_credit < 300 then incr upload_credit)
@@ -965,6 +943,7 @@ let dynamic_refill_upload_slots () =
 let turn = ref (-1)
 
 let refill_upload_slots () =
+  if !CommonGlobals.has_upload = 0 then begin
   incr turn;
   if !turn = 5 then
     turn := 0;
@@ -975,6 +954,7 @@ let refill_upload_slots () =
   end else
     (* call every 1s *)
     static_refill_upload_slots ()
+  end
 
 let consume_bandwidth len =
   streaming_left := !streaming_left - len
