@@ -21,6 +21,8 @@ open Md4
 
 open Printf2
 open CommonClient
+open CommonShared
+open CommonServer
 open CommonNetwork
 open CommonResult
 open CommonFile
@@ -35,7 +37,6 @@ open CommonTypes
 open CommonGlobals
 open CommonOptions
 open CommonUserDb
-open CommonTypes
 open Int64ops
 
 module VB = VerificationBitmap
@@ -49,19 +50,11 @@ let lprintf_n fmt =
   lprintf2 log_prefix fmt
 
 let verify_user_admin () =
-  let empty_pwd = ref false in
-  begin try
-    if user2_password admin_user = blank_password then
-      empty_pwd := true
-    with e ->
-      lprintf_nl (_b "SECURITY INFO: user 'admin' has to be present, creating...");
-      empty_pwd := true;
-      ignore (user2_add admin_user blank_password "")
-  end;
   let warning =
     "SECURITY WARNING: user admin has an empty password, use command: useradd admin password\n"
   in
-  if !empty_pwd && not !!enable_user_config then
+  if has_empty_password (admin_user ()) && !!allowed_ips <>
+    [(Ip.range_of_string (strings_of_option allowed_ips).option_default)] then
     begin
       lprintf_n "%s" warning;
       warning
@@ -83,6 +76,7 @@ let real_startup_message () =
   let s =
   !startup_message ^ (verify_user_admin ()) ^ (check_supported_os ()) 
   ^ (if not !dns_works then "DNS resolution does not work\n" else "")
+  ^ (if not !Charset.conversion_enabled then "Charset conversion does not work, disabled\n" else "")
   ^ (match !created_new_base_directory with
        None -> ""
      | Some dir -> (Printf.sprintf "MLDonkey created a new home directory in %s\n" dir))
@@ -239,6 +233,8 @@ if !!html_mods_use_relative_availability
 	then file_availability f
 	else string_availability f.file_availability
 
+let number_of_comments f =
+  List.length f.file_comments
 
 (* WARNING: these computations are much more expensive as they seem.
 We use the ShortLazy to avoid recomputing the result too many times,
@@ -316,6 +312,7 @@ let save_config () =
       Options.save_with_help_private users_ini;
       CommonComplexOptions.save ();
       CommonUploads.save ();
+      CommonStats.save ();
       networks_iter_all (fun r ->
           List.iter (fun opfile ->
               Options.save_with_help opfile
@@ -394,13 +391,10 @@ let print_table_text buf alignments titles lines =
 
 let print_table_html_mods buf lines =
 
-  let counter = ref 0 in
+  html_mods_cntr_init ();
 
   List.iter (fun line ->
-      if (!counter mod 2 == 0) then Printf.bprintf buf "\\<tr class=dl-1"
-      else Printf.bprintf buf "\\<tr class=dl-2";
-      incr counter;
-
+      Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"" (html_mods_cntr ());
       Array.iter (fun data ->
           Printf.bprintf buf "%s" data;
       ) line;
@@ -575,7 +569,7 @@ function cancelAll(x){for(i=0;i\\<document.selectForm.elements.length;i++){var j
   Printf.bprintf buf "\\</form\\>"
 
 
-  let print_file_html_mods buf guifiles =
+let print_file_html_mods buf guifiles =
 
     if (List.length guifiles) > 0 then begin
 	let tsize = ref Int64.zero in
@@ -636,7 +630,7 @@ function submitPriority(num,cp,sel) {
 \\<div class=main\\>";
 
 if !!html_mods_use_js_tooltips then Printf.bprintf buf 
-"\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:FFFFDD;color:black;border-color:black;border-width:20;font-size:8pt; visibility:show; left:25px; top:
+"\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:#FFFFDD;color:black;border-color:black;border-width:20px;font-size:8pt; visibility:visible; left:25px; top:
 -100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>";
 
 	Printf.bprintf buf "\\<form id=\\\"selectForm\\\" name=\\\"selectForm\\\" action=\\\"files\\\"\\>
@@ -660,7 +654,8 @@ if !!html_mods_use_js_tooltips then Printf.bprintf buf
 
 \\<td title=\\\"Pause\\\"  class=\\\"dlheader np\\\"\\>P\\</td\\>
 \\<td title=\\\"Resume\\\" class=\\\"dlheader np\\\"\\>R\\</td\\>
-\\<td title=\\\"Cancel\\\" class=\\\"dlheader brs\\\"\\>C\\</td\\>"
+\\<td title=\\\"Cancel\\\" class=\\\"dlheader brs\\\"\\>C\\</td\\>
+\\<td title=\\\"Click to switch release status\\\" class=\\\"dlheader brs\\\"\\>R\\</td\\>"
 (if !qnum > 0 then begin
 	Printf.sprintf "title=\\\"Active(%d): %s/%s | Queued(%d): %s/%s\\\""
 	(List.length guifiles - !qnum) (size_of_int64 (!tdl -- !qdl)) (size_of_int64 (!tsize -- !qsize))
@@ -670,17 +665,24 @@ else "")
 (List.length guifiles) (size_of_int64 !tdl) (size_of_int64 !tsize) (!trate /. 1024.)
 (let unread = ref 0 in
 Fifo.iter (fun (t,i,num,n,s) -> if t > !last_message_log then incr unread) chat_message_fifo;
-if !unread > 0 then Printf.sprintf "\\<td class=downloaded title=\\\"%d unread messages\\\"\\>(+%d)\\&nbsp;\\</td\\>" !unread !unread else "");
+if !unread > 0 then Printf.sprintf "\\<td onMouseOver=\\\"mOvr(this);\\\" onMouseOut=\\\"mOut(this);\\\" class=downloaded title=\\\"%d unread messages\\\"\\>\\<a onClick=\\\"mSub('fstatus','version');mSub('output','message')\\\"\\>(+%d)\\</a\\>\\&nbsp;\\</td\\>" !unread !unread else "");
 
 if !!html_mods_vd_network then Printf.bprintf buf
 "\\<td title=\\\"Sort by network\\\" class=dlheader\\>\\<input style=\\\"padding-left: 0px; padding-right: 0px;\\\" class=headbutton type=submit value=N name=sortby\\>\\</td\\>";
 
 Printf.bprintf buf
-"\\<td title=\\\"Sort by filename\\\" class=dlheader\\>\\<input class=headbutton type=submit value=File name=sortby\\>\\</td\\>
-\\<td title=\\\"Sort by size\\\" class=dlheader\\>\\<input class=headbutton type=submit value=Size name=sortby\\>\\</td\\>
+"\\<td title=\\\"Sort by filename\\\" class=dlheader\\>\\<input class=headbutton type=submit value=File name=sortby\\>\\</td\\>";
+
+if !!html_mods_vd_user then Printf.bprintf buf
+"\\<td title=\\\"Sort by user\\\" class=dlheader\\>\\<input class=headbutton type=submit value=User name=sortby\\>\\</td\\>";
+
+if !!html_mods_vd_group then Printf.bprintf buf
+"\\<td title=\\\"Sort by group\\\" class=dlheader\\>\\<input class=headbutton type=submit value=Group name=sortby\\>\\</td\\>";
+
+Printf.bprintf buf
+"\\<td title=\\\"Sort by size\\\" class=dlheader\\>\\<input class=headbutton type=submit value=Size name=sortby\\>\\</td\\>
 \\<td title=\\\"Sort by size downloaded\\\" class=dlheader\\>\\<input class=\\\"headbutton ar\\\" type=submit value=DLed name=sortby\\>\\</td\\>
 \\<td title=\\\"Sort by percent\\\" class=dlheader\\>\\<input class=headbutton type=submit value=%% name=sortby\\>\\</td\\>";
-
 if !!html_mods_vd_comments then Printf.bprintf buf
 "\\<td title=\\\"Sort by comments\\\" class=dlheader\\>\\<input style=\\\"padding-left: 0px; padding-right: 0px;\\\" class=headbutton type=submit value=Cm name=sortby\\>\\</td\\>";
 
@@ -715,26 +717,36 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
         [|
           (if !!html_mods_use_js_tooltips then
                         Printf.sprintf "
-                                onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s<br>%sFile#: %d<br>Network: %s%s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
-                        (Http_server.html_real_escaped file.file_name)
+                                onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s<br>%sFile#: %d<br>Network: %s<br>User%s %s%s%s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
+                        (Http_server.html_real_escaped (Charset.to_utf8 file.file_name))
 			(match file_magic (file_find file.file_num) with
 			   None -> ""
 			 | Some magic -> "File type: " ^ (Http_server.html_real_escaped magic) ^ "<br>")
 			file.file_num
                         (net_name file)
-			(let comments = file_comment (file_find file.file_num) in
-			   if comments = [] then "" else
+                        (if file.file_group = "none" then "" else ":Group")
+                        file.file_user
+                        (if file.file_group = "none" then "" else Printf.sprintf ":%s" file.file_group)
+
+      (if file.file_comments = [] then "" else
 			    begin
-			      let buf1 = Buffer.create 100 in
-			      Printf.bprintf buf1 "<br><br>Comments:<br>";
-			      List.iter (fun s -> Printf.bprintf buf1 "%s<br>" (Http_server.html_real_escaped s)) comments;
+           let num_comments = number_of_comments file in
+           let buf1 = Buffer.create (!!max_comment_length * num_comments) in
+           Printf.bprintf buf1 "<br><br>Comments(%d):<br>" (num_comments);
+	   let comments =
+	     if List.length file.file_comments > 5 then
+	       fst (List2.cut 5 file.file_comments) @ [Ip.null, "", 0, (_s "MLDonkey note: click file for more comments")]
+	     else
+	       file.file_comments
+	   in
+           List.iter (fun (_,_,_,s) -> Printf.bprintf buf1 "%s<br>" (Http_server.html_real_escaped (Charset.to_utf8 s))) comments;
 			      Buffer.contents buf1
 			    end)
+
 			!!html_mods_js_tooltips_wait
 			!!html_mods_js_tooltips_timeout
 			!!html_mods_js_tooltips_wait
-			 else Printf.sprintf "
-                               onMouseOver=\\\"mOvr(this);return true;\\\" onMouseOut=\\\"mOut(this);\\\"\\>");
+       else Printf.sprintf " onMouseOver=\\\"mOvr(this);return true;\\\" onMouseOut=\\\"mOut(this);\\\"\\>");
 
 	(if downloading file then
               Printf.sprintf "\\<td class=\\\"dl al np\\\"\\>\\<input class=checkbox name=pause type=checkbox value=%d\\>\\</td\\>
@@ -749,6 +761,10 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
                 file.file_num
                 file.file_num);
 
+	  Printf.sprintf "\\<td onClick=\\\"location.href='files?%s=%d';return true;\\\" class=\\\"dl al brs\\\"\\>\\%s\\</td\\>"
+	    (if file.file_release then "norelease" else "release")
+	    file.file_num (if file.file_release then "R" else "-");
+
           (if !!html_mods_vd_network then
 			Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d';return true;\\\"
 			title=\\\"%s\\\" class=\\\"dl al\\\"\\>%s\\</td\\>"
@@ -772,14 +788,14 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
             (truncate ( (1. -. downloaded /. size) *. 100.))
 	 else
 	     Printf.sprintf "\\<TD onClick=\\\"location.href='submit?q=vd+%d';return true;\\\"
-                        title=\\\"[File#: %d] [Net: %s]%s\\\" class=\\\"dl al\\\"\\>%s\\<br\\>
+                        title=\\\"[File#: %d] [Net: %s] [Comments: %d]%s\\\" class=\\\"dl al\\\"\\>%s\\<br\\>
                         \\<table cellpadding=0 cellspacing=0 width=100%%\\>\\<tr\\>
                         \\<td class=\\\"loaded\\\" style=\\\"height:%dpx\\\" width=\\\"%d%%\\\"\\> \\</td\\>
                         \\<td class=\\\"remain\\\" style=\\\"height:%dpx\\\" width=\\\"%d%%\\\"\\> \\</td\\>
                         \\</tr\\>\\</table\\>\\</td\\>"
             file.file_num
             file.file_num
-                        (net_name file)
+                        (net_name file) (number_of_comments file)
                         (if !!max_name_len < String.length file.file_name then " " ^ file.file_name else "")
             (short_name file)
             (!!html_vd_barheight)
@@ -788,14 +804,16 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
             (truncate ( (1. -. downloaded /. size) *. 100.)));
           );
 
+          (if !!html_mods_vd_user then ctd file.file_num file.file_user else "");
+          (if !!html_mods_vd_group then ctd file.file_num file.file_group else "");
+
           (ctd file.file_num (size_of_int64 file.file_size));
           (ctd file.file_num (size_of_int64 file.file_downloaded));
           (ctd file.file_num (Printf.sprintf "%.1f" (percent file)));
 
           (if !!html_mods_vd_comments then
-			Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d';return true;\\\"
-			class=\\\"dl al\\\"\\>%d\\</td\\>"
-			file.file_num (file_comment_length (file_find file.file_num)) else "");
+            ctd file.file_num (Printf.sprintf "%d" (number_of_comments file))
+            else "");
 
           (ctd file.file_num (Printf.sprintf "%d" (number_of_sources file)));
 
@@ -936,7 +954,7 @@ let simple_print_file_list finished buf files format =
     else
       print_table buf
         [|
-        Align_Left; Align_Left; Align_Right; Align_Right;
+        Align_Left; Align_Left; Align_Right; Align_Left; Align_Left; Align_Left;
         Align_Right; Align_Right; Align_Right |]
         (if format.conn_output = HTML then
           [|
@@ -951,6 +969,10 @@ let simple_print_file_list finished buf files format =
           |] else
           [|
             "$nNum";
+	    "Rele";
+	    "Comm";
+	    "User";
+	    "Group";
             "File";
             "    %";
             "    Done";
@@ -995,6 +1017,10 @@ let simple_print_file_list finished buf files format =
                     file.file_num
                       (if downloading file then "PAUSE" else "RESUME")
                   else ""));
+	      (Printf.sprintf "%s" (if file.file_release then "R" else "-"));
+	      (Printf.sprintf "%4d" (number_of_comments file));
+              file.file_user;
+              file.file_group;
               (short_name file);
               (Printf.sprintf "%3.1f" (percent file));
               (if !!improved_telnet then (print_human_readable file file.file_downloaded)
@@ -1086,7 +1112,8 @@ let display_active_file_list buf o list =
 
   if o.conn_output <> HTML && !!improved_telnet then
   begin
-    let list = Sort.list (fun f1 f2 -> percent f1 >= percent f2) list in
+    let list = 
+      List.sort (fun f1 f2 -> compare (percent f2) (percent f1)) list in
     simple_print_file_list false buf list o
   end
   else
@@ -1095,27 +1122,37 @@ let display_active_file_list buf o list =
       let sorter =
         match o.conn_sortvd with
 
-        | BySize -> (fun f1 f2 -> f1.file_size >= f2.file_size)
+        | BySize -> (fun f1 f2 -> compare f2.file_size f1.file_size)
         | ByRate -> (fun f1 f2 ->
-                if stalled f1 then false else
-                if stalled f2 then true else
-                  f1.file_download_rate >= f2.file_download_rate
-            )
-        | ByName -> (fun f1 f2 -> String.lowercase f1.file_name <= String.lowercase f2.file_name)
-        | ByDone -> (fun f1 f2 -> f1.file_downloaded >= f2.file_downloaded)
-	| ByPriority -> (fun f1 f2 -> f1.file_priority >= f2.file_priority)
-		| BySources -> (fun f1 f2 -> (number_of_sources f1) >= (number_of_sources f2))
-		| ByASources -> (fun f1 f2 -> (number_of_active_sources f1) >= (number_of_active_sources f2))
-        | ByPercent -> (fun f1 f2 -> percent f1 >= percent f2)
-        | ByETA -> (fun f1 f2 -> calc_file_eta f1 <= calc_file_eta f2)
-        | ByAge -> (fun f1 f2 -> f1.file_age >= f2.file_age)
-        | ByLast -> (fun f1 f2 -> f1.file_last_seen >= f2.file_last_seen)
-        | ByNet -> (fun f1 f2 -> net_name f1 <= net_name f2)
-        | ByAvail -> (fun f1 f2 -> get_file_availability f1 >= get_file_availability f2)
-        | ByComments -> (fun f1 f2 -> file_comment_length (file_find f1.file_num) >= file_comment_length (file_find f2.file_num))
+                if stalled f1 then 1 else
+                if stalled f2 then -1 else
+                  compare f2.file_download_rate f1.file_download_rate)
+        | ByName -> (fun f1 f2 -> String.compare 
+		       (String.lowercase f1.file_name) 
+		       (String.lowercase f2.file_name))
+        | ByDone -> (fun f1 f2 -> 
+		       compare f2.file_downloaded f1.file_downloaded)
+	| ByPriority -> (fun f1 f2 -> 
+			   compare f2.file_priority f1.file_priority)
+	| BySources -> (fun f1 f2 -> compare 
+			  (number_of_sources f2) (number_of_sources f1))
+	| ByASources -> (fun f1 f2 -> 
+			   compare (number_of_active_sources f2) 
+			           (number_of_active_sources f1))
+        | ByPercent -> (fun f1 f2 -> compare (percent f2) (percent f1))
+        | ByETA -> (fun f1 f2 -> compare (calc_file_eta f1) (calc_file_eta f2))
+        | ByAge -> (fun f1 f2 -> compare f2.file_age f1.file_age)
+        | ByLast -> (fun f1 f2 -> compare f2.file_last_seen f1.file_last_seen)
+        | ByNet -> (fun f1 f2 -> compare (net_name f1) (net_name f2))
+        | ByAvail -> (fun f1 f2 -> compare 
+			(get_file_availability f2) (get_file_availability f1))
+        | ByComments -> (fun f1 f2 -> compare 
+			   (number_of_comments f2) (number_of_comments f1))
+        | ByUser -> (fun f1 f2 -> compare f1.file_user f2.file_user)
+        | ByGroup -> (fun f1 f2 -> compare f1.file_group f2.file_group)
         | NotSorted -> raise Not_found
       in
-      Sort.list sorter list
+      List.sort sorter list
     with _ -> list
   in
   simple_print_file_list false buf list o
@@ -1150,15 +1187,18 @@ let get_tag_value tag =
   match tag.tag_value with
  | Uint64 i -> String.escaped (Int64.to_string i)
  | Fint64 i -> String.escaped (Int64.to_string i)
+ | Uint16 i | Uint8 i -> String.escaped (string_of_int i)
  | String s -> String.escaped s
- | _ -> ""
+ | Addr i -> Ip.to_string i
+ | Pair (x,y) -> Printf.sprintf "%Ld, %Ld" x y
 
 let old_print_search buf o results =
   let user = o.conn_user in
   let counter = ref 0 in
   if use_html_mods o then
   begin
-    if !!html_mods_use_js_tooltips then Printf.bprintf buf "\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:FFFFDD;color:black;border-color:black;border-width:20;font-size:8pt; visibility:show; left:25px; top:-100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>\n";
+    html_mods_cntr_init ();
+    if !!html_mods_use_js_tooltips then Printf.bprintf buf "\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:#FFFFDD;color:black;border-color:black;border-width:20px;font-size:8pt; visibility:visible; left:25px; top:-100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>\n";
     html_mods_table_header_colspan buf "resultsTable" "results" [
       ( "1", "0", "srh", "Network", "Network" ) ;
       ( "1", "0", "srh", "File", "File (mouseover)" ) ;
@@ -1178,10 +1218,7 @@ let old_print_search buf o results =
               if !counter >= !!max_displayed_results then raise Exit;
 
               if use_html_mods o then
-                begin
-                  if (!counter mod 2 == 0) then Printf.bprintf buf "\\<tr class=\\\"dl-1\\\"\\>"
-                  else Printf.bprintf buf "\\<tr class=\\\"dl-2\\\"\\>";
-                end;
+                Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
 
               user.ui_last_results <- (!counter, rs) :: user.ui_last_results;
               let network_name = 
@@ -1226,7 +1263,7 @@ let old_print_search buf o results =
                       let nl = ref false in
                       List.iter (fun t ->
                           match t.tag_name with
-                          | Field_UNKNOWN "FTH" | Field_UNKNOWN "urn" -> ()
+                          | Field_KNOWN "FTH" | Field_KNOWN "urn" -> ()
                           | _ ->
                               Buffer.add_string buf ((if !nl then "<br>" else begin nl := true;"" end) ^
                                 escaped_string_of_field t ^ ": " ^ get_tag_value t);
@@ -1242,7 +1279,7 @@ let old_print_search buf o results =
                       let nl = ref false in
                       List.iter (fun t ->
                           match t.tag_name with
-                          | Field_UNKNOWN "FTH" | Field_UNKNOWN "urn" -> ()
+                          | Field_KNOWN "FTH" | Field_KNOWN "urn" -> ()
                           | _ ->
                               Buffer.add_string buf ((if !nl then "\n" else begin nl := true;"" end) ^
                                   "|| (" ^
@@ -1298,8 +1335,8 @@ let old_print_search buf o results =
 	      let cformat = ref "" in
               List.iter (fun t ->
                   (match t.tag_name with
-                    | Field_UNKNOWN "urn"
-                    | Field_UNKNOWN "FTH"  -> hash := get_tag_value t
+                    | Field_KNOWN "urn"
+                    | Field_KNOWN "FTH"  -> hash := get_tag_value t
                     | Field_Availability -> cavail := get_tag_value t
                     | Field_Completesources -> csource := get_tag_value t
                     | Field_Length -> clength := get_tag_value t
@@ -1350,9 +1387,9 @@ let old_print_search buf o results =
                         | Field_Format
                         | Field_Bitrate
 (* TODO : "urn" shouldn't be some kind of Field_Uid of Gnutella ? *)
-                        | Field_UNKNOWN "urn"
+                        | Field_KNOWN "urn"
 (* TODO : "FTH" shouldn't be some kind of Field_Uid of Fasttrack ? *)
-                        | Field_UNKNOWN "FTH"  -> ()
+                        | Field_KNOWN "FTH"  -> ()
                         | _ ->
                             Buffer.add_string buf ("\\<span title=\\\"" ^
                                 get_tag_value t ^ "\\\"\\>(" ^
@@ -1457,9 +1494,9 @@ let print_search_html buf results o search_num =
               user.ui_last_results <- (!counter, rs) :: user.ui_last_results;
               files := [|
 
+                (Printf.sprintf "[%5d]\\<input name=d type=checkbox value=%d\\>" !counter r.result_num);
                 (Int64.to_string r.result_size);
                 (string_of_int avail);
-                (Printf.sprintf "[%5d]\\<input name=d type=checkbox value=%d\\>" !counter r.result_num);
 
                 (
                   let names = r.result_names in
@@ -1671,8 +1708,8 @@ let print_search buf s o =
   Intmap.iter (fun r_num (avail,rs) ->
       let r = IndexedResults.get_result rs in
       results := (rs, r, !avail) :: !results) s.search_results;
-  let results = Sort.list (fun (_, r1,_) (_, r2,_) ->
-        r1.result_size > r2.result_size
+  let results = List.sort (fun (_, r1,_) (_, r2,_) ->
+        compare r2.result_size r1.result_size
     ) !results in
 
   Printf.bprintf buf "Result of search %d\n" s.search_num;
@@ -1703,7 +1740,8 @@ let networks_header buf =
       ( "0", "srh br", "Has search", "Search" ) ;
       ( "0", "srh br", "Has chat", "Chat" ) ;
       ( "0", "srh br", "Has rooms", "Rooms" ) ;
-      ( "0", "srh", "Has multinet", "Multinet" ) ]
+      ( "0", "srh", "Has multinet", "Multinet" ) ;
+      ( "0", "srh", "Has porttest", "Porttest" ) ]
 
 let print_network_modules buf o =
   let buf = o.conn_buf in
@@ -1718,17 +1756,22 @@ let print_network_modules buf o =
           if not (List.mem VirtualNetwork n.network_flags) then
             try
               let net_has e = if List.mem e n.network_flags then "yes" else "" in
+              let net_has_porttest () =
+		match network_porttest_result n with
+		  PorttestNotAvailable -> ""
+		| _ -> "yes" in
               Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
                 ("", "sr br", n.network_name);
-                ("", "sr br", if n.op_network_is_enabled () then "Enabled" else "Disabled");
+                ("", "sr br", if network_is_enabled n then "Enabled" else "Disabled");
                 ("", "sr br", net_has NetworkHasUpload);
                 ("", "sr br", net_has NetworkHasServers);
                 ("", "sr br", net_has NetworkHasSupernodes);
                 ("", "sr br", net_has NetworkHasSearch);
                 ("", "sr br", net_has NetworkHasChat);
                 ("", "sr br", net_has NetworkHasRooms);
-                ("", "sr"   , net_has NetworkHasMultinet); ];
+                ("", "sr br", net_has NetworkHasMultinet);
+                ("", "sr"   , (net_has_porttest ())); ];
               Printf.bprintf buf "\\</tr\\>";
             with _ -> ()
         );
@@ -1743,16 +1786,15 @@ let print_network_modules buf o =
   else
     begin
       Printf.bprintf buf "Networks:";
-      Hashtbl.iter
-        (fun name n ->
+      networks_iter_all (fun n ->
           try
-            Printf.bprintf buf "\n   %2d %-30s %s" n.network_num name
-                (if n.op_network_is_enabled () then "Enabled" else "Disabled")
-          with _ -> ()
-        ) networks_by_name
+            Printf.bprintf buf "\n   %2d %-30s %s" n.network_num n.network_name
+                (if network_is_enabled n then "Enabled" else "Disabled")
+          with _ -> ())
     end
 
-let print_gdstats buf o =
+let print_gdstats o =
+  let buf = o.conn_buf in
   let picture_suffix () =
     if !!html_mods_vd_gfx_png then
       Printf.bprintf buf "png\\\"\\>"
@@ -1818,6 +1860,9 @@ let buildinfo html buf =
     (
       "Version:\t", 
       "MLNet Multi-Network p2p client version " ^ Autoconf.current_version 
+      ^ (match Filename.basename (Sys.executable_name) with
+         | "mlnet" | "mlnet.static" -> ""
+         | bin -> Printf.sprintf " (%s)" bin)
     );
   if Autoconf.scm_version <> "" then 
     tack list 
@@ -1828,7 +1873,7 @@ let buildinfo html buf =
   tack list
     (
       "Networks:\t", 
-      !networks_string 
+      !networks_string
     );
   tack list 
     (
@@ -1840,7 +1885,7 @@ let buildinfo html buf =
     );
   tack list 
     (
-      "Build on:\t", 
+      "Built on:\t", 
       Autoconf.build_system ^ " (" ^ Unix2.endianness () ^ ")" ^
       (if Autoconf.glibc_version = "" then "" 
       else
@@ -1867,7 +1912,7 @@ let buildinfo html buf =
   tack list 
     (
       "Features:\t",
-      (if BasicSocket.has_threads () then "threads" else "no-threads") ^
+      (if BasicSocket.has_threads () then " threads" else " no-threads") ^
       (let s = Zlib.zlib_version_num () in 
          Printf.sprintf " zlib%s" (if s <> "" then "-" ^ s else "")) ^
       (if Autoconf.bzip2 then
@@ -1886,7 +1931,10 @@ let buildinfo html buf =
 	    " gd(jpg)"
 	| _, false, false ->
 	    " gd(neither jpg nor png ?)") ^
-      (if Autoconf.has_iconv then " iconv" else " no-iconv") ^
+      (match Autoconf.has_iconv, !Charset.conversion_enabled with
+       | true, true -> " iconv(active)"
+       | true, false -> " iconv(inactive)"
+       | false, _ -> " no-iconv") ^
       (match Autoconf.magic, !Autoconf.magic_works with
        | true, true -> " magic(active)"
        | true, false -> " magic(inactive)"
@@ -1901,12 +1949,11 @@ let buildinfo html buf =
        ( "0", "srh", "", "" ) ]
     else
       Printf.bprintf buf "\n\t--Buildinfo--\n";
-    let counter = ref 0 in
+    html_mods_cntr_init ();
     List.iter (fun (desc, text) ->
-      incr counter;
       if html then 
-	Printf.bprintf buf "\\<tr class=\\\"%s\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
-          (if !counter mod 2 = 0 then "dl-1" else "dl-2") desc text
+	Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
+          (html_mods_cntr ()) desc text
       else
 	Printf.bprintf buf "%s %s\n" desc text;
     ) list;
@@ -1924,8 +1971,8 @@ let runinfo html buf o =
     (
       "User:\t\t", 
       Printf.sprintf "%s (%s) - uptime: %s" 
-	o.conn_user.ui_user_name
-	(if empty_password o.conn_user.ui_user_name then "Warning: empty Password"
+	o.conn_user.ui_user.user_name
+	(if has_empty_password o.conn_user.ui_user then "Warning: empty Password"
 	else "PW Protected")
 	(Date.time_to_string (last_time () - start_time) "verbose")
     );
@@ -1952,7 +1999,7 @@ let runinfo html buf o =
   tack list
     (
       "Geoip:\t\t", 
-      if !Geoip.active then "enabled, GeoLite data created by MaxMind, available from http://maxmind.com/"
+      if Geoip.active () then "enabled, GeoLite data created by MaxMind, available from http://maxmind.com/"
       else "disabled, to enable adjust web_infos in downloads.ini for automatic download"
     );
   tack list
@@ -1996,10 +2043,11 @@ let runinfo html buf o =
   tack list
     (
       "", 
-      Printf.sprintf "\t\t max_string_length: %d - word_size: %d - max_array_length: %d"
+      Printf.sprintf "\t\t max_string_length: %d - word_size: %d - max_array_length: %d - max_int: %d"
 	Sys.max_string_length
 	Sys.word_size
 	Sys.max_array_length
+	Pervasives.max_int
     );
   tack list
     (
@@ -2020,12 +2068,11 @@ let runinfo html buf o =
        ( "0", "srh", "", "" ) ]
     else
       Printf.bprintf buf "\n\t--Runinfo--\n";
-    let counter = ref 0 in
+    html_mods_cntr_init ();
     List.iter (fun (desc, text) ->
-      incr counter;
       if html then 
-	Printf.bprintf buf "\\<tr class=\\\"%s\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
-          (if !counter mod 2 = 0 then "dl-1" else "dl-2") desc text
+	Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
+          (html_mods_cntr ()) desc text
       else
 	Printf.bprintf buf "%s %s\n" desc text;
     ) list;
@@ -2089,14 +2136,14 @@ let diskinfo html buf =
   let len_dir = ref 9 in
   let len_strategy = ref 29 in (* "shared (incoming_directories)" *)
   List.iter ( fun (dir, strategy) ->
-    len_dir := maxi !len_dir (String.length dir);
-    len_strategy := maxi !len_strategy (String.length strategy)
+    len_dir := max !len_dir (String.length dir);
+    len_strategy := max !len_strategy (String.length strategy)
   ) !list;
   let fill_dir = String.make (!len_dir - 9) ' ' in
   let fill_dir_line = String.make (!len_dir - 9) '-' in
   let fill_strategy = String.make (!len_strategy - 4) ' ' in
   let fill_strategy_line = String.make (!len_strategy - 4) '-' in
-  let counter = ref 0 in
+  html_mods_cntr_init ();
   if html then
       html_mods_table_header buf "sharesTable" "shares" [
        ( "0", "srh", "Directory", "Directory" ) ;
@@ -2114,7 +2161,6 @@ let diskinfo html buf =
         fill_dir_line fill_strategy_line;
     end;
   List.iter (fun (dir, strategy) ->
-	incr counter;
 	let diskused =
     	  match Unix32.diskused dir with
 	  | None -> Printf.sprintf "---"
@@ -2132,15 +2178,14 @@ let diskinfo html buf =
 	in
 	let filesystem = Unix32.filesystem dir in
 	if html then
-	  begin
-	    Printf.bprintf buf "\\<tr class=\\\"%s\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
-	    (if !counter mod 2 = 0 then "dl-1" else "dl-2")
-	    dir strategy diskused diskfree percentfree filesystem
-	  end
+	    Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>
+	      \\<td class=\\\"sr ar\\\"\\>%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
+	      \\<td class=\\\"sr ar\\\"\\>%s\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
+	    (html_mods_cntr ()) dir strategy diskused diskfree percentfree filesystem
 	else
 	  Printf.bprintf buf "%-*s|%-*s|%8s|%8s|%5s|%-s\n"
-	    (maxi !len_dir (!len_dir - String.length dir)) dir
-	    (maxi !len_strategy (!len_strategy - String.length strategy)) strategy
+	    (max !len_dir (!len_dir - String.length dir)) dir
+	    (max !len_strategy (!len_strategy - String.length strategy)) strategy
 	    diskused diskfree percentfree filesystem
     	) !list;
   if html then
@@ -2186,11 +2231,11 @@ let dllink_print_result html url header results =
   if html then Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>\\</div\\>";
   Buffer.contents buf
 
-let dllink_query_networks html url =
+let dllink_query_networks html url user group =
   let result = ref [] in
   if not (networks_iter_until_true (fun n ->
     try
-      let s,r = network_parse_url n url in
+      let s,r = network_parse_url n url user group in
         if s = "" then
           r
         else
@@ -2208,7 +2253,7 @@ let dllink_query_networks html url =
   else
     dllink_print_result html url "Added link" !result
 
-let dllink_parse html url =
+let dllink_parse html url user =
   if (String2.starts_with url "http") then (
     let u = Url.of_string url in
     let module H = Http_client in
@@ -2242,21 +2287,14 @@ let dllink_parse html url =
       let concat_headers =
         (List.fold_right (fun (n, c) t -> n ^ ": " ^ c ^ "\n" ^ t) headers "")
       in
-      ignore (dllink_query_networks html concat_headers)
+      ignore (dllink_query_networks html concat_headers user user.user_default_group)
     );
     dllink_print_result html url "Parsing HTTP url" [])
   else
     if (String2.starts_with url "ftp") then
-      dllink_query_networks html (Printf.sprintf "Location: %s" url)
+      dllink_query_networks html (Printf.sprintf "Location: %s" url) user user.user_default_group
     else
-      dllink_query_networks html url
-
-let print_command_result o buf result =
-  if use_html_mods o then
-    html_mods_table_one_row buf "serversTable" "servers" [
-      ("", "srh", result); ]
-  else
-    Printf.bprintf buf "%s" result
+      dllink_query_networks html url user user.user_default_group
 
 module UnionFind = struct
   type t = int array
@@ -2378,3 +2416,123 @@ let filenames_variability o list =
 	shorten fileinfo.file_name 80;
 	string_of_int nc |]
     ) sorted_score_list)
+
+let print_upstats o list server =
+  let buf = o.conn_buf in
+  if use_html_mods o then
+    begin
+      if !!html_mods_use_js_tooltips then Printf.bprintf buf 
+"\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:#FFFFDD;color:black;border-color:black;border-width:20px;font-size:8pt; visibility:visible; left:25px; top:
+-100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>";
+
+      Printf.bprintf buf "\\<div class=\\\"upstats\\\"\\>";
+      match server with
+	None ->
+	  html_mods_table_one_row buf "upstatsTable" "upstats" [
+	    ("", "srh", Printf.sprintf "Session: %s uploaded | Shared(%d): %s\n"
+	    (size_of_int64 !upload_counter) !nshared_files (size_of_int64 !nshared_bytes)); ]
+      | Some s -> let info = server_info s in
+	  html_mods_table_one_row buf "upstatsTable" "upstats" [
+	    ("", "srh", Printf.sprintf "%d files shared on %s (%s:%s)"
+              info.G.server_published_files info.G.server_name
+              (Ip.string_of_addr info.G.server_addr)
+              (string_of_int info.G.server_port)); ]
+    end
+  else
+    begin
+      Printf.bprintf buf "Upload statistics:\n";
+      Printf.bprintf buf "Session: %s uploaded | Shared(%d): %s\n"
+	(size_of_int64 !upload_counter) !nshared_files (size_of_int64 !nshared_bytes)
+    end;
+
+  if use_html_mods o then
+    html_mods_table_header buf "upstatsTable" "upstats" [
+      ( "1", "srh", "Total file requests", "Reqs" ) ;
+      ( "1", "srh", "Total bytes sent", "Total" ) ;
+      ( "1", "srh", "Upload Ratio", "UPRatio" ) ;
+      ( "0", "srh", "Preview", "P" ) ;
+      ( "0", "srh", "Filename", "Filename" );
+      ( "0", "srh", "Statistic links", "Stats" );
+      ( "0", "srh", "Published on servers", "Publ" );
+      ( "0", "srh", "Share status", "Status" )
+    ]
+  else
+    begin
+      Printf.bprintf buf " Requests |  Bytes   | Uploaded | File\n";
+      Printf.bprintf buf "----------+----------+----------+----------------------------------------------------\n";
+    end;
+
+  html_mods_cntr_init ();
+  let list = List.sort (fun f1 f2 ->
+    let c = compare f2.impl_shared_requests f1.impl_shared_requests in
+    if c <> 0 then c else
+    compare f2.impl_shared_uploaded f1.impl_shared_uploaded
+    ) list in
+
+  List.iter (fun impl ->
+    if use_html_mods o then
+      begin
+	let published = List.length impl.impl_shared_servers in
+	let ed2k = file_print_ed2k_link
+	  (Filename.basename impl.impl_shared_codedname)
+	  impl.impl_shared_size impl.impl_shared_id in
+
+	Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"" (html_mods_cntr ());
+	(if !!html_mods_use_js_tooltips then
+	   Printf.bprintf buf " onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s<br>%s%s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
+	    (Http_server.html_real_escaped (Filename.basename (Charset.to_utf8 impl.impl_shared_codedname)))
+	    (match impl.impl_shared_file with
+	       None -> "no file info"
+	     | Some file -> match file_magic file with | None -> "no magic"
+	     | Some magic -> "File type: " ^ (Http_server.html_real_escaped magic) ^ "<br>")
+	    (if impl.impl_shared_servers = [] then "" else
+	       Printf.sprintf "<br>Published on %d %s<br>%s"
+		published (if published = 1 then "server" else "servers")
+		(let listbuf = Buffer.create 100 in
+		   List.iter (fun s -> let info = server_info s in
+		     Printf.bprintf listbuf "%s (%s:%s%s)<br>"
+			info.server_name
+			(Ip.string_of_addr info.server_addr)
+			(string_of_int info.server_port)
+			(if info.server_realport <> 0
+			   then "(" ^ (string_of_int info.server_realport) ^ ")" else "")
+			) impl.impl_shared_servers;
+		 Buffer.contents listbuf))
+		!!html_mods_js_tooltips_wait
+		!!html_mods_js_tooltips_timeout
+		!!html_mods_js_tooltips_wait
+	 else Printf.bprintf buf " onMouseOver=\\\"mOvr(this);return true;\\\" onMouseOut=\\\"mOut(this);\\\"\\>");
+
+	let uploaded = Int64.to_float impl.impl_shared_uploaded in
+	let size = Int64.to_float impl.impl_shared_size in
+	html_mods_td buf [
+	  ("", "sr ar", Printf.sprintf "%d" impl.impl_shared_requests);
+	  ("", "sr ar", size_of_int64 impl.impl_shared_uploaded);
+	  ("", "sr ar", Printf.sprintf "%5.1f" ( if size < 1.0 then 0.0 else (uploaded *. 100.) /. size));
+	  ("", "sr", Printf.sprintf "\\<a href=\\\"preview_upload?q=%d\\\"\\>P\\</a\\>" impl.impl_shared_num);
+	  ("", "sr", (if impl.impl_shared_id = Md4.null then
+		        (Filename.basename impl.impl_shared_codedname)
+		      else
+			Printf.sprintf "\\<a href=\\\"%s\\\"\\>%s\\</a\\>"
+			  ed2k (shorten (Filename.basename impl.impl_shared_codedname) !!max_name_len)));
+	  ("", "sr", (if impl.impl_shared_id = Md4.null then "" else
+			Printf.sprintf "\\<a href=\\\"http://tothbenedek.hu/ed2kstats/ed2k?hash=%s\\\"\\>%s\\</a\\>
+\\<a href=\\\"http://ed2k.titanesel.ws/ed2k.php?hash=%s\\\"\\>%s\\</a\\>
+\\<a href=\\\"http://bitzi.com/lookup/ed2k:%s\\\"\\>%s\\</a\\>"
+                      (Md4.to_string impl.impl_shared_id) "T1"
+                      (Md4.to_string impl.impl_shared_id) "T2"
+                      (Md4.to_string impl.impl_shared_id) "B"));
+	  ("", "sr ar", Printf.sprintf "%d" published);
+	  ("", "sr", shared_state (as_shared impl) o);
+        ];
+	Printf.bprintf buf "\\</tr\\>\n";
+      end
+    else
+      Printf.bprintf buf "%9d | %8s | %7s%% | %-50s\n"
+	(impl.impl_shared_requests)
+	(size_of_int64 impl.impl_shared_uploaded)
+	(Printf.sprintf "%3.1f" ((Int64.to_float impl.impl_shared_uploaded *. 100.) /. Int64.to_float impl.impl_shared_size))
+	(shorten (Filename.basename impl.impl_shared_codedname) !!max_name_len)
+  ) list;
+
+  if use_html_mods o then Printf.bprintf buf "\\</table\\>\\</div\\>\\</div\\>"
