@@ -92,8 +92,6 @@ let gui_can_write gui =
     None -> not !gui_reconnected
   | Some sock -> TcpBufferedSocket.can_write sock
           
-let restart_gui_server = ref (fun _ -> ())
-
 let update_events gui update user_num map =
   if not gui.gui_poll then
     let gui_num = gui.gui_num in
@@ -470,15 +468,13 @@ let gui_initialize gui =
           gui.gui_events.gui_new_events <- ev :: gui.gui_events.gui_new_events
       ) console_messages;                
       
-      if user2_is_admin gui.gui_conn.conn_user.ui_user then
       gui_send gui (
-        P.Options_info (simple_options "" downloads_ini));
+        P.Options_info (simple_options "" downloads_ini (user2_is_admin gui.gui_conn.conn_user.ui_user)));
 
-      if user2_is_admin gui.gui_conn.conn_user.ui_user then
       networks_iter_all (fun r ->
           List.iter (fun opfile ->
               let prefix = r.network_shortname ^ "-" in
-              let args = simple_options prefix opfile in
+              let args = simple_options prefix opfile (user2_is_admin gui.gui_conn.conn_user.ui_user) in
               gui_send gui (P.Options_info args)) r.network_config_file);
 
 (* Options panels defined in downloads.ini *)
@@ -623,11 +619,18 @@ let gui_reader (gui: gui_record) t _ =
               ) list
           
           | P.SetOption (name, value) ->
+              let o = gui.gui_conn in
+              let gui_type, ip, port =
+                match o.conn_info with
+                | None -> None, None, None
+                | Some (gui_type, (ip, port)) -> Some gui_type, Some ip, Some port
+              in
 	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
 		CommonInteractive.set_fully_qualified_options name value
+                  ~user:(Some o.conn_user.ui_user.user_name)
+                  ~ip:ip ~port:port ~gui_type:gui_type ()
 	      else
 	        begin
-                  let o = gui.gui_conn in
                   let buf = o.conn_buf in
                   Buffer.reset buf; 
                   Buffer.add_string buf "\nYou are not allowed to change options\n";
@@ -740,7 +743,11 @@ let gui_reader (gui: gui_record) t _ =
                 if not (networks_iter_until_true
                     (fun n ->
                        try
-                         let s,r = network_parse_url n url gui.gui_conn.conn_user.ui_user in r
+                         let s,r =
+                            network_parse_url n url
+                              gui.gui_conn.conn_user.ui_user
+                              gui.gui_conn.conn_user.ui_user.user_default_group
+                         in r
                        with e ->
                          lprintf "Exception %s for network %s\n"
                            (Printexc2.to_string e) (n.network_name);
@@ -789,9 +796,18 @@ let gui_reader (gui: gui_record) t _ =
               server_remove (server_find num)
           
           | P.SaveOptions_query list ->
+              let o = gui.gui_conn in
+              let gui_type, ip, port =
+                match o.conn_info with
+                | None -> None, None, None
+                | Some (gui_type, (ip, port)) -> Some gui_type, Some ip, Some port
+              in
 	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
               List.iter (fun (name, value) ->
-                  CommonInteractive.set_fully_qualified_options name value) list;
+                  CommonInteractive.set_fully_qualified_options name value
+                    ~user:(Some o.conn_user.ui_user.user_name)
+                    ~ip:ip ~port:port ~gui_type:gui_type ();
+              ) list;
               DriverInteractive.save_config ()
           
           | P.RemoveDownload_query num ->
@@ -1091,16 +1107,28 @@ let gui_reader (gui: gui_record) t _ =
               let s = server_find num in
               server_rename s name
           | P.ServerSetPreferred (num, preferred) ->
-              let s = server_find num in
-              server_set_preferred s preferred
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
+                server_set_preferred (server_find num) preferred
+	      else
+	        begin
+                  let o = gui.gui_conn in
+                  let buf = o.conn_buf in
+                  Buffer.reset buf; 
+                  Buffer.add_string buf "\nYou are not allowed to change preferred status\n";
+                  gui_send gui (P.Console (
+                      DriverControlers.dollar_escape o false
+                        (Buffer.contents buf)))
+		end
 
   with 
     Failure s ->
       gui_send gui (Console (Printf.sprintf "Failure: %s\n" s))
-  | Torrent_can_not_be_used ->
-      gui_send gui (Console (Printf.sprintf "\nError: This torrent does not have valid tracker URLs\n"))
-  | Torrent_already_exists ->
-      gui_send gui (Console (Printf.sprintf "\nError: This torrent is already in download queue\n"))
+  | Torrent_started s ->
+      gui_send gui (Console (Printf.sprintf "\nInfo: Torrent %s started\n" s))
+  | Torrent_can_not_be_used s ->
+      gui_send gui (Console (Printf.sprintf "\nError: Torrent %s does not have valid tracker URLs\n" s))
+  | Torrent_already_exists s ->
+      gui_send gui (Console (Printf.sprintf "\nError: Torrent %s is already in download queue\n" s))
   | e ->
       gui_send gui (Console (Printf.sprintf "from_gui: exception %s for message %s\n"
 	(Printexc2.to_string e) (GuiProto.string_of_from_gui t)))
@@ -1123,7 +1151,7 @@ let gui_events () =
     
   }
   
-let new_gui gui_send gui_auth sock  =
+let new_gui gui_send gui_auth sock gui_type =
   incr gui_counter;
   let gui = {
       gui_searches = [];
@@ -1145,8 +1173,12 @@ let new_gui gui_send gui_auth sock  =
         conn_sortvd = BySize;
         conn_filter = (fun _ -> ()); 
         conn_buf = Buffer.create 100;
-        conn_user = default_user;
+        conn_user = find_ui_user CommonUserDb.admin_user_name;
         conn_width = 80; conn_height = 25;
+        conn_info =
+          match sock with
+          | None -> Some (gui_type, (Ip.null, 0));
+          | Some t -> Some (gui_type, peer_addr t);
       };
       gui_send = gui_send;
     } in
@@ -1173,7 +1205,7 @@ let gui_handler t event =
             s in
         
         let gui = new_gui binary_gui_send 
-            false (Some sock) in
+            false (Some sock) GUI in
         gui.gui_result_handler <- binary_result_handler gui;
         TcpBufferedSocket.set_max_output_buffer sock !!interface_buffer;
         TcpBufferedSocket.set_lifetime sock 30.;
@@ -1211,7 +1243,7 @@ let gift_handler t event =
             "gui connection"
             s in
         
-        let gui = new_gui gift_gui_send true (Some sock) in
+        let gui = new_gui gift_gui_send true (Some sock) GIFT in
         gui.gui_result_handler <- gift_result_handler gui;
         guis := gui :: !guis;
         
@@ -1452,7 +1484,7 @@ let _ =
             if !gui_reconnected then begin
                 lprintf "gui_reconnected !\n"; 
                 gui_reconnected := false;
-                let gui = new_gui binary_gui_send false None in
+                let gui = new_gui binary_gui_send false None GUI in
                 gui.gui_result_handler <- binary_result_handler gui;
                 local_gui := Some gui;
                 ()
