@@ -54,7 +54,6 @@ let do_daily () =
 
 let minute_timer () =
   DriverInteractive.hdd_check ();
-  CommonShared.shared_check_files ();
   CommonUploads.upload_credit_timer ();
   CommonInteractive.force_download_quotas ();
   CommonResult.dummy_result.result_time <- last_time ();
@@ -88,7 +87,6 @@ let second_timer timer =
      CommonUploads.refill_upload_slots ()
    with e ->
         lprintf_nl (_b "Exception %s") (Printexc2.to_string e));
-  CommonUploads.reset_upload_timer ();
   CommonUploads.shared_files_timer ();
   ()
 
@@ -110,14 +108,32 @@ let start_interfaces () =
             (Printexc2.to_string e);
     end;
 
-  ignore (find_port  "telnet server" !!telnet_bind_addr
-      telnet_port DriverControlers.telnet_handler);
+  if !!telnet_port <> 0 then begin 
+      try
+        ignore (find_port "telnet server" !!telnet_bind_addr
+          telnet_port DriverControlers.telnet_handler);
+      with e ->
+          lprintf_nl (_b "Exception %s while starting Telnet interface")
+            (Printexc2.to_string e);
+    end;
 
-  gui_server_sock := find_port "gui server"  !!gui_bind_addr
-    gui_port gui_handler;
-  if !!gift_port <> 0 then
-    ignore (find_port "gift server"  !!gui_bind_addr
-        gift_port gift_handler);
+  if !!gui_port <> 0 then begin 
+      try
+        ignore (find_port "gui server" !!gui_bind_addr
+          gui_port gui_handler);
+      with e ->
+          lprintf_nl (_b "Exception %s while starting GUI interface")
+            (Printexc2.to_string e);
+    end;
+
+  if !!gift_port <> 0 then begin
+      try
+        ignore (find_port "gift server" !!gui_bind_addr
+          gift_port gift_handler);
+      with e ->
+          lprintf_nl (_b "Exception %s while starting GUI interface")
+            (Printexc2.to_string e);
+    end;
 
   add_infinite_option_timer update_gui_delay DriverInterface.update_gui_info;
   add_infinite_timer 1. second_timer
@@ -306,6 +322,23 @@ let load_config () =
       "-pid", Arg.String (fun s -> pid := s;
       ),
        _s ": directory for pid file";
+      "-useradd", Arg.Rest (fun s ->
+        (match String2.split s ' ' with
+        | user :: pass :: _ ->
+            if user2_user_exists user then
+              begin
+                user2_user_set_password (user2_user_find user) pass;
+                Printf.printf "%sPassword of user %s changed\n%!" (log_time ()) user
+              end
+            else
+              begin
+                user2_user_add user (Md4.Md4.string pass) ();
+                Printf.printf "%sUser %s added\n%!" (log_time ()) user
+              end;
+            Options.save_with_help_private users_ini;
+            Printf.printf "%sSaved changes to users.ini\n%!" (log_time ())
+        | _ -> raise (Arg.Bad "invalid syntax"));
+          exit 0), _s "\"<user> <pass>\" : create user/change password";
     ] @
       !more_args
       @
@@ -395,6 +428,7 @@ or getting a binary compiled with glibc %s.\n\n")
 (*  lprintf "(1) CommonComplexOptions.load\n"; *)
   CommonComplexOptions.load ();
   CommonUploads.load ();
+  CommonStats.load ();
 
 (*  lprintf "(2) CommonComplexOptions.load done\n"; *)
   begin
@@ -405,6 +439,22 @@ or getting a binary compiled with glibc %s.\n\n")
     save_results =:= old_save_results;
   end;
 
+  CommonGlobals.is_startup_phase := false;
+
+(* before activating network modules load all local files from web_infos/
+   to avoid security holes, especially for IP blocking *)
+  Hashtbl.iter (fun key w ->
+    let file = Filename.concat "web_infos" (Filename.basename w.url) in
+    if Sys.file_exists file then
+      try
+        lprintf_nl "loading %s from %s" w.kind file;
+        ((List.assoc w.kind !CommonWeb.file_kinds).f w.url) file;
+        w.state <- Some FileLoaded;
+      with _ -> ()
+  ) web_infos_table;
+
+  discover_ip false;
+
   lprintf_nl (_b "Check http://www.mldonkey.org for updates");
   networks_iter (fun r -> network_load_complex_options r);
   lprintf_nl (_b "enabling networks: ");
@@ -446,6 +496,11 @@ or getting a binary compiled with glibc %s.\n\n")
   add_infinite_timer 0.1 CommonUploads.upload_download_timer;
   add_infinite_timer !!buffer_writes_delay (fun _ -> Unix32.flush ());
 
+  if !!share_scan_interval <> 0 then
+  add_infinite_timer ((float_of_int !!share_scan_interval) *. 60.)
+    (fun _ -> CommonShared.shared_check_files ());
+  CommonShared.shared_check_files ();
+
   history_timeflag := (Unix.time()); 
   update_download_history (); 
   update_upload_history ();
@@ -478,16 +533,17 @@ or getting a binary compiled with glibc %s.\n\n")
 
   Options.prune_file downloads_ini;
   Options.prune_file users_ini;
-(*  Options.prune_file downloads_expert_ini; *)
-  add_timer 5. (fun _ -> try CommonWeb.load_web_infos true false with _ -> ());
-  lprintf_nl  (_b "To command: telnet %s %d")
+  add_timer 1. (fun _ -> try CommonWeb.load_web_infos true false with _ -> ());
+  if !!telnet_port <> 0 then lprintf_nl  (_b "To command: telnet %s %d")
 	(if !!telnet_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!telnet_bind_addr)  !!telnet_port;
-  lprintf_nl  (_b "Or with browser: http://%s:%d")
+  if !!http_port <> 0 then begin
+    lprintf_nl  (_b "Or with browser: http://%s:%d")
 	(if !!http_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!http_bind_addr)  !!http_port;
-  lprintf_nl  (_b "For a GUI check out http://sancho-gui.sourceforge.net");
-  lprintf_nl  (_b "Connect to IP %s, port %d")
+    lprintf_nl  (_b "For a GUI check out http://sancho-gui.sourceforge.net")
+  end;
+  if !!gui_port <> 0 then lprintf_nl  (_b "Connect to IP %s, port %d")
 	(if !!gui_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!gui_bind_addr)  !!gui_port;
   lprintf_nl  (_b "If you connect from a remote machine adjust allowed_ips");
@@ -541,11 +597,12 @@ or getting a binary compiled with glibc %s.\n\n")
 
   if not Autoconf.windows then
     MlUnix.set_signal  Sys.sighup
-      (Sys.Signal_handle (fun _ -> lprintf_nl (_b "Received SIGHUP, closing all files and client/server sockets");
+      (Sys.Signal_handle (fun _ -> lprintf_nl (_b "Received SIGHUP, closing all files and client/server sockets, start IP discovery");
          networks_iter (fun r -> CommonNetwork.network_reset r); (* stop_all_bt *)
 	 CommonServer.disconnect_all_servers ();
 	 CommonClient.disconnect_all_clients ();
-	 Unix32.close_all () (* close all files *)
+         Unix32.close_all (); (* close all files *)
+         discover_ip false;
          ));
 
   if not Autoconf.windows then
