@@ -44,7 +44,7 @@ let (=:=) = Options.(=:=)
 
 let short_name n =
   let n = if n="" then
-     "http://www.mldonkey.net/"
+     "http://www.mldonkey.org"
   else n
   in
   let len = String.length n in
@@ -60,6 +60,7 @@ let is_connected state =
   | NotConnected _
   | Connecting
   | RemovedHost
+  | ServerFull
   | BlackListedHost
   | NewHost -> false
 
