@@ -124,7 +124,7 @@ let must_share_file file codedname has_old_impl =
           impl_shared_ops = shared_ops;
           impl_shared_val = file;
           impl_shared_requests = 0;
-          impl_shared_magic = None;
+          impl_shared_file = Some (as_file file);
           impl_shared_servers = [];
         } in
         file.file_shared <- Some impl;
@@ -260,7 +260,7 @@ let rec set_trackers file file_trackers =
 	  file.file_trackers <-  t :: file.file_trackers;
 	set_trackers file q
 
-let new_file file_id t torrent_diskname file_temp file_state user =
+let new_file file_id t torrent_diskname file_temp file_state user group =
   try
     Hashtbl.find files_by_uid file_id
   with Not_found ->
@@ -270,7 +270,7 @@ let new_file file_id t torrent_diskname file_temp file_state user =
           file_file = file_impl;
           file_piece_size = t.torrent_piece_size;
           file_id = file_id;
-          file_name = Charset.safe_convert t.torrent_encoding t.torrent_name;
+          file_name = t.torrent_name;
           file_comment = t.torrent_comment;
           file_created_by = t.torrent_created_by;
           file_creation_date = t.torrent_creation_date;
@@ -290,7 +290,7 @@ let new_file file_id t torrent_diskname file_temp file_state user =
         } and file_impl =  {
           dummy_file_impl with
           impl_file_owner = user;
-          impl_file_group = user.user_default_group;
+          impl_file_group = group;
           impl_file_fd = Some file_fd;
           impl_file_size = t.torrent_length;
           impl_file_downloaded = Int64.zero;
@@ -417,6 +417,7 @@ let dot_string_of_string s =
   ) s;
   Buffer.contents buf
 
+(* check string s for char c (dec) at position l (list) *)
 let check_all s c l =
   let ch = char_of_int c in
   List.for_all (fun i -> s.[i] = ch) l
@@ -427,6 +428,14 @@ let check_int s p =
     true
   with _ -> false
 
+let strip_leading_zeroes s =
+  let l = String.length s in
+  let rec aux i =
+    if i = l then "0"
+    else if s.[i] <> '0' then String.sub s i (l - i)
+    else aux (i + 1) in
+  aux 0
+
 (* from azureus/gpl *)
 let decode_az_style s =
   if check_all s 45 [0;7] then begin
@@ -440,20 +449,21 @@ let decode_az_style s =
       | "BR" -> Brand_bitrocket
       | "BS" -> Brand_btslave
       | "BX" -> Brand_bittorrentx
-      | "CT" -> Brand_ctorrent
+      | "CT" (* ctorrent *)
       | "CD" -> Brand_ctorrent
+      | "lt" (* libtorrent *)
       | "LT" -> Brand_libtorrent
       | "MT" -> Brand_moonlighttorrent
       | "SB" -> Brand_swiftbit
       | "SN" -> Brand_sharenet
       | "SS" -> Brand_swarmscope
-      | "SZ" -> Brand_shareaza
+      | "SZ" (* shareaza *)
+      | "S~" -> Brand_shareaza
       | "TN" -> Brand_torrentdotnet
       | "TS" -> Brand_torrentstorm
       | "XT" -> Brand_xantorrent
       | "ZT" -> Brand_ziptorrent
       | "bk" -> Brand_bitkitten
-      | "lt" -> Brand_libtorrent
       | "MP" -> Brand_moopolice
       | "UT" -> Brand_utorrent
       | "KT" -> Brand_ktorrent
@@ -466,15 +476,30 @@ let decode_az_style s =
       | "qB" -> Brand_qbittorrent
       | "QT" -> Brand_qt4
       | "UL" -> Brand_uleecher
+      | "XX" -> Brand_xtorrent
+      | "AG" (* ares *)
+      | "A~" -> Brand_ares
+      | "AX" -> Brand_bitpump
+      | "DE" -> Brand_deluge
+      | "TT" -> Brand_tuotu
+      | "XL" -> Brand_xunlei
+      | "FT" -> Brand_foxtorrent
+      | "BF" -> Brand_bitflu
       | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None else
       let version = 
         match brand with
+          (* 4.56 *)
+          | Brand_bitpump
           | Brand_bitcomet -> (String.sub s 4 1) ^ "." ^ (String.sub s 5 2)
+          (* 3.45 *)
+          | Brand_tuotu
           | Brand_utorrent -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ "." ^ (String.sub s 5 1)
-          | Brand_ctorrent 
-          | Brand_transmission -> (String.sub s 3 2) ^ "." ^ (String.sub s 5 2)
+          (* 34.56 *)
+          | Brand_ctorrent
+          | Brand_transmission -> (strip_leading_zeroes (String.sub s 3 2)) ^ "." ^ (strip_leading_zeroes(String.sub s 5 2))
+          (* 3.4.5->[R=RC.6|D=Dev|''] *)
           | Brand_ktorrent -> 
               let x = match s.[5] with 
                 | 'R' -> " RC" ^ (String.sub s 6 1)
@@ -482,6 +507,13 @@ let decode_az_style s =
                 | _ -> ""
               in
               (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ x
+          (* 3.4(56) *)
+          | Brand_bitrocket -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ "(" ^ (String.sub s 5 2) ^ ")"
+          (* v 3456 *)
+          | Brand_xtorrent -> "v" ^ (strip_leading_zeroes (String.sub s 3 4))
+          (* BitFlu is too complicated YMDD (Y+M -> HEX) eg. 7224 is 2007.02.24 *)
+          | Brand_bitflu -> ""
+          (* 3.4.5.6 *)
           | _ -> (dot_string (String.sub s 3 4)) 
       in
       Some (brand, version)
@@ -489,40 +521,39 @@ let decode_az_style s =
   None
 
 let decode_tornado_style s =
-  if check_all s 45 [5] then begin
-    let check_brand s =
-     match s with
-     | "T" -> Brand_bittornado
-     | "S" -> Brand_shadow
-     | "A" -> Brand_abc
-     | "U" -> Brand_upnp
-     | "O" -> Brand_osprey
-     | "R" -> Brand_tribler
-     | _ -> Brand_unknown
+  if s.[5] = '-' then begin
+    let check_brand c =
+     match c with
+     | 'T' -> Brand_bittornado
+     | 'S' -> Brand_shadow
+     | 'A' -> Brand_abc
+     | 'U' -> Brand_upnp
+     | 'O' -> Brand_osprey
+     | 'R' -> Brand_tribler
+     | _   -> Brand_unknown
     in
     let bv = ref None in
 
-    if check_all s 45 [6;7;8] then begin
-      let brand = check_brand (String.sub s 0 1) in
+    if s.[5] ='-' && s.[6] ='-' && s.[7] ='-' then begin
+      let brand = check_brand s.[0] in
       if not (brand = Brand_unknown) then
         bv := Some (brand, (dot_string_h (String.sub s 1 3)));
     end
     else if s.[6] = (char_of_int 48) then begin
-      let brand = check_brand (String.sub s 0 1) in
+      let brand = check_brand s.[0] in
       if not (brand = Brand_unknown) then
         bv := Some (brand, ("LM " ^ dot_string_h (String.sub s 1 3)));
     end;
-
     !bv
   end else
   None
 
 let decode_mainline_style s =
   if check_all s 45 [2;7] && check_int s 1 then begin
-    let s_id = String.sub s 0 1 in
+    let c_id = s.[0] in
     let brand =
-      match s_id with
-     | "M" -> Brand_mainline
+      match c_id with
+     | 'M' -> Brand_mainline
      | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None
@@ -532,27 +563,38 @@ let decode_mainline_style s =
 
 let decode_simple_style s =
   let simple_list = ref
-    [ (0, "martini", Brand_martiniman, "");
-      (0, "oernu", Brand_btugaxp, "");
-      (0, "BTDWV-", Brand_deadmanwalking, "");
-      (0, "PRC.P---", Brand_btplus, "II");
-      (0, "P87.P---", Brand_btplus, "");
-      (0, "S587Plus", Brand_btplus, "");
-      (5, "Azureus", Brand_azureus, "2.0.3.2");
-      (0, "-G3", Brand_g3torrent, "");
-      (0, "-AR", Brand_arctic, "");
-      (4, "btfans", Brand_simplebt, "");
-      (0, "btuga", Brand_btugaxp, "");
-      (0, "BTuga", Brand_btugaxp, "");
-      (0, "DansClient", Brand_xantorrent, "");
-      (0, "Deadman Walking-", Brand_deadmanwalking, "");
-      (0, "346-", Brand_torrenttopia, "");
-      (0, "271-", Brand_greedbt, "2.7.1");
-      (10, "BG", Brand_btgetit, "");
-      (0, "a00---0", Brand_swarmy, "");
-      (0, "a02---0", Brand_swarmy, "");
-      (0, "10-------", Brand_jvtorrent, "");
-      (0, "T00---0", Brand_teeweety, "") ]
+    [ (0,  "martini",           Brand_martiniman,      "");
+      (0,  "oernu",             Brand_btugaxp,         "");
+      (0,  "BTDWV-",            Brand_deadmanwalking,  "");
+      (0,  "PRC.P---",          Brand_btplus,          "II");
+      (0,  "P87.P---",          Brand_btplus,          "");
+      (0,  "S587Plus",          Brand_btplus,          "");
+      (5,  "Azureus",           Brand_azureus,         "2.0.3.2");
+      (0,  "-G3",               Brand_g3torrent,       "");
+      (0,  "-AR",               Brand_arctic,          "");
+      (4,  "btfans",            Brand_simplebt,        "");
+      (0,  "btuga",             Brand_btugaxp,         "");
+      (0,  "BTuga",             Brand_btugaxp,         "");
+      (0,  "DansClient",        Brand_xantorrent,      "");
+      (0,  "Deadman Walking-",  Brand_deadmanwalking,  "");
+      (0,  "346-",              Brand_torrenttopia,    "");
+      (0,  "271-",              Brand_greedbt,         "2.7.1");
+      (10, "BG",                Brand_btgetit,         "");
+      (0,  "a00---0",           Brand_swarmy,          "");
+      (0,  "a02---0",           Brand_swarmy,          "");
+      (0,  "10-------",         Brand_jvtorrent,       "");
+      (0,  "T00---0",           Brand_teeweety,        "");
+      (0,  "LIME",              Brand_limewire,        "");
+      (0,  "AZ2500BT",          Brand_btyrant,         "");
+      (0,  "Mbrst",             Brand_burst,           (dot_string_of_list s [5;7;9]));
+      (0,  "Plus",              Brand_plus,            (dot_string_of_list s [4;5;6]));
+      (0,  "OP",                Brand_opera,           (dot_string(String.sub s 2 4)));
+      (0,  "eX",                Brand_exeem,           (String.sub s 2 18));
+      (0,  "turbobt",           Brand_turbobt,         (String.sub s 7 5));
+      (0,  "btpd",              Brand_btpd,            (dot_string(String.sub s 5 3)));
+      (0,  "XBT",               Brand_xbt,             (dot_string(String.sub s 3 3)));
+      (0,  "-FG",               Brand_flashget,        (dot_string(String.sub s 4 3)));
+    ]
   in
   let len = List.length !simple_list in
   let rec check pos =
@@ -564,21 +606,6 @@ let decode_simple_style s =
   in
   check 0
 
-let decode_mburst s =
-  if "Mbrst" = String.sub s 0 5 then
-     Some (Brand_burst, (dot_string_of_list s [5;7;9]))
-  else None
-
-let decode_plus s =
-  if "Plus" = String.sub s 0 4 then
-     Some (Brand_plus, (dot_string_of_list s [4;5;6]))
-  else None
-
-let decode_opera s =
-  if "OP" = String.sub s 0 2 then
-     Some (Brand_opera, (dot_string_of_list s [2;3;4;5]))
-  else None
-
 let decode_rufus s =
   let release s =
     let minor = Char.code s.[1] in
@@ -592,20 +619,10 @@ let decode_bow s =
   (check_all s 45 [0;7] && "BOW" = String.sub s 1 3) then
     Some (Brand_bitsonwheels, (String.sub s 4 3))
   else None
-
-let decode_exeem s =
-  if "eX" = String.sub s 0 2 then
-    Some (Brand_exeem, (String.sub s 2 18))
-  else None
-
-let decode_turbo s =
-  if "turbobt" = String.sub s 0 7 then
-    Some (Brand_turbobt, (String.sub s 7 5))
-  else None
-
-let decode_xbt s =
-  if "XBT" = String.sub s 0 3 then
-    Some (Brand_xbt, (dot_string (String.sub s 3 3)))
+  
+let decode_btuga s =
+  if ("BTM" = String.sub s 0 3) && ("BTuga" = String.sub s 5 5) then
+    Some (Brand_btuga, dot_string(String.sub s 3 2))
   else None
 
 let decode_shadow s =
@@ -639,7 +656,7 @@ let decode_bitspirit s =
   None
 
 let decode_upnp s =
-  if "U" = String.sub s 0 1 && s.[8] = '-' then
+  if 'U' = s.[0] && s.[8] = '-' then
     Some (Brand_upnp, (dot_string (String.sub s 1 3)))
   else None
 
@@ -704,7 +721,7 @@ let decode_non_zero s =
 
 (* format is : "-ML" ^ version ( of unknown length) ^ "-" ^ random bytes ( of unknown length) *)
 let decode_mldonkey_style s =
-  if check_all s 45 [0] then begin
+  if '-' = s.[0] then begin
     let s_id = String.sub s 1 2 in
     let brand =
      match s_id with
@@ -712,25 +729,21 @@ let decode_mldonkey_style s =
      | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None else
-      let len = 
-        (try String.index_from s 3 s.[0]
+      let len =
+        (try String.index_from s 3 '-'
         with _ -> 8) - 3
       in
       let version = String.sub s 3 len in
     Some (brand, version)
   end else None
 
+
 let decoder_list = [
     decode_az_style;
     decode_tornado_style;
     decode_mainline_style;
     decode_simple_style;
-    decode_mburst;
-    decode_turbo;
-    decode_plus;
-    decode_xbt;
     decode_bow;
-    decode_exeem;
     decode_shadow;
     decode_bitspirit;
     decode_upnp;
@@ -738,8 +751,8 @@ let decoder_list = [
     decode_shareaza;
     decode_non_zero;
     decode_mldonkey_style;
-    decode_opera;
     decode_rufus;
+    decode_btuga;
   ]
 
 let parse_software s =
@@ -749,8 +762,11 @@ let parse_software s =
       [] -> lprintf_nl "Unknown BT client software version, report the next line to http://mldonkey.sourceforge.net/UnknownBtClients%s\nBTUC:\"%s\"" Autoconf.current_version (String.escaped s);
             default
       | d :: t -> match (d s) with 
-                  | None -> iter t
-                  | Some bv -> bv
+                  | None    -> iter t
+                  | Some bv -> let (brand, version) = bv in
+                               if !verbose_msg_clienttags then
+                                 lprintf_nl "BTKC:\"%s\"; ID: \"%s\"; version:\"%s\"" (String.escaped s) (brand_to_string brand) version;
+                               bv
   in
   if Sha1.direct_of_string s = Sha1.null then
     default
@@ -758,10 +774,24 @@ let parse_software s =
     try iter decoder_list
     with _ -> default
 
-let new_client file peer_id kind =
+let check_client_country_code c =
+  if Geoip.active () then
+    match c.client_country_code with
+    | None ->
+        c.client_country_code <-
+          Geoip.get_country_code_option (fst c.client_host)
+    | _ -> ()
+
+let new_client file peer_id kind cc =
   try
     let c = Hashtbl.find file.file_clients kind in
+    let old_ip = fst c.client_host in
     c.client_host <- kind;
+    if old_ip <> Ip.null && old_ip <> fst c.client_host then
+      begin
+        c.client_country_code <- None;
+        check_client_country_code c
+      end;
     c
   with _ ->
       let brand, release = parse_software (Sha1.direct_to_string peer_id) in
@@ -772,6 +802,7 @@ let new_client file peer_id kind =
           client_connection_control = new_connection_control (());
           client_file = file;
           client_host = kind;
+          client_country_code = cc;
           client_choked = true;
           client_received_peer_id = false;
           client_sent_choke = false;
@@ -786,8 +817,10 @@ let new_client file peer_id kind =
           client_release = release;
           client_bitmap = None;
           client_allowed_to_write = zero;
-          client_uploaded = zero;
-          client_downloaded = zero;
+          client_total_uploaded = zero;
+          client_total_downloaded = zero;
+          client_session_uploaded = zero;
+          client_session_downloaded = zero;
           client_upload_rate = Rate.new_rate ();
           client_downloaded_rate = Rate.new_rate ();
           client_connect_time = last_time ();
@@ -813,6 +846,7 @@ let new_client file peer_id kind =
           impl_client_upload = None;
         } in
       c.client_connection_control.control_min_reask <- 120;
+      check_client_country_code c;
       new_client impl;
       Hashtbl.add file.file_clients kind c;
       file.file_clients_num <- file.file_clients_num + 1;
@@ -844,7 +878,9 @@ let remove_tracker url file =
 
 let tracker_is_enabled t =
   match t.tracker_status with
-    Enabled -> true
+  | Enabled -> true
+  | Disabled_failure (i,_) ->
+      if !!tracker_retries = 0 || i < !!tracker_retries then true else false
   | _ -> false
 
 let torrents_directory = "torrents"
