@@ -96,8 +96,9 @@ let tracked_files = Hashtbl.create 13
 let ntracked_files = ref 0
 
 let tracker_port = define_option bittorrent_section ["tracker_port"]
+  ~restart: true
   "The port to bind the tracker to"
-    int_option 6881
+    port_option 6881
 
 let max_tracked_files = define_option bittorrent_section ["max_tracked_files"]
   "The maximal number of tracked files (to prevend saturation attack)"
@@ -557,6 +558,7 @@ let start_tracker () =
           requests = [];
           addrs = Ip_set.of_list [ Ip.RangeCIDR (Ip.null, 0) ];
           base_ref = "";
+(* limit access to MLDonkey BT tracker based on IP blocklist *)
 	  use_ip_block_list = true;
           default = http_handler;
         } in
