@@ -49,6 +49,7 @@ type peer =
     mutable peer_ip : Ip.t;
     mutable peer_port : int;
     mutable peer_tcpport : int;
+    mutable peer_country_code : int option;
     mutable peer_kind : int;
     mutable peer_expire : int;
     mutable peer_last_send : int;
@@ -139,11 +140,11 @@ search for files and search for sources. *)
 let print_peer buf p =
   let hours = (last_time () - p.peer_created) / 3600 in
   let mins = (last_time () - p.peer_created) / 60 in
-  Printf.bprintf buf "   { md4 = %s ip = %s port = %d %s kind = %d l_send = %d expire = %d created = %d h = %d mins = %d %s}\n"
+  Printf.bprintf buf " {%s:%s:%d%s k=%d ls=%d ex=%d cr=%d h=%d m=%d%s}\n"
     (Md4.to_string p.peer_md4) (Ip.to_string p.peer_ip) p.peer_port
-    (if p.peer_tcpport <> 0 then Printf.sprintf "tcp = %d" p.peer_tcpport
+    (if p.peer_tcpport <> 0 then Printf.sprintf " tcp=%d" p.peer_tcpport
     else "") p.peer_kind p.peer_last_send p.peer_expire p.peer_created
-    hours mins (if p.peer_expire > last_time () && p.peer_last_send <> 0 then "OK " else "")
+    hours mins (if p.peer_expire > last_time () && p.peer_last_send <> 0 then " OK" else "")
     
 
 let message_to_string t =
@@ -151,61 +152,61 @@ let message_to_string t =
   begin
     match t with
       OvernetConnect p ->
-        Buffer.add_string buf "OvernetConnect\n";
+        Buffer.add_string buf "Connect\n";
         print_peer buf p
     | OvernetConnectReply peers ->
-        Buffer.add_string buf "OvernetConnectReply\n";
+        Buffer.add_string buf "ConnectReply\n";
         List.iter (print_peer buf) peers
     | OvernetPublicize p ->
-        Buffer.add_string buf "OvernetPublicize\n";
+        Buffer.add_string buf "Publicize\n";
         print_peer buf p
     | OvernetPublicized p ->
-        Buffer.add_string buf "OvernetPublicized\n";
+        Buffer.add_string buf "Publicized\n";
         (match p with
           Some p -> print_peer buf p
         | None -> ())
     | OvernetSearch (nresults, md4, _) ->
-        Buffer.add_string buf "OvernetSearch\n";
-        Printf.bprintf buf "   target = %s nresults = %d\n"
+        Buffer.add_string buf "Search\n";
+        Printf.bprintf buf " target=%s nresults=%d\n"
           (Md4.to_string md4) nresults
     | OvernetSearchReply (target, peers) ->
-        Buffer.add_string buf "OvernetSearchReply\n";
-        Printf.bprintf buf "  target = %s npeers = %d\n"
+        Buffer.add_string buf "SearchReply\n";
+        Printf.bprintf buf " target=%s npeers=%d\n"
           (Md4.to_string target) (List.length peers);
         List.iter (print_peer buf) peers
     | OvernetGetSearchResults (target, _, _, _) ->
-        Printf.bprintf buf "OvernetGetSearchResults %s\n" (Md4.to_string target)
+        Printf.bprintf buf "GetSearchResults %s\n" (Md4.to_string target)
     | OvernetSearchFilesResults (target, results) ->
-        Printf.bprintf buf "OvernetSearchFilesResults %s\n"
+        Printf.bprintf buf "SearchFilesResults %s\n"
           (Md4.to_string target);
         List.iter (fun (r_md4, r_tags) ->
-            Printf.bprintf buf "    %s\n       " (Md4.to_string r_md4);
+            Printf.bprintf buf "  %s\n   " (Md4.to_string r_md4);
             bprint_tags buf r_tags ;
             Printf.bprintf buf "\n";
         ) results
     | OvernetSearchSourcesResults (target, peers) ->
-        Printf.bprintf buf "OvernetSearchSourcesResults %s\n"
+        Printf.bprintf buf "SearchSourcesResults %s\n"
           (Md4.to_string target);
         List.iter (print_peer buf) peers
     | OvernetPublishFiles (target, results) ->
-        Printf.bprintf buf "OvernetPublish %s\n"
+        Printf.bprintf buf "Publish %s\n"
           (Md4.to_string target);
         List.iter (fun (r_md4, r_tags) ->
-            Printf.bprintf buf "    %s\n       " (Md4.to_string r_md4);
+            Printf.bprintf buf "  %s\n   " (Md4.to_string r_md4);
             bprint_tags buf r_tags ;
             Printf.bprintf buf "\n";
         ) results
     | OvernetPublishSources (target, peers) ->
-        Printf.bprintf buf "OvernetPublishSources %s\n"
+        Printf.bprintf buf "PublishSources %s\n"
           (Md4.to_string target);
         List.iter (print_peer buf) peers
     | OvernetPublished target ->
-        Printf.bprintf buf "OvernetPublished %s\n" (Md4.to_string target)
+        Printf.bprintf buf "Published %s\n" (Md4.to_string target)
     | OvernetPeerNotFound p ->
-        Printf.bprintf buf "OvernetPeerNotFound\n";
+        Printf.bprintf buf "PeerNotFound\n";
         print_peer buf p
     | OvernetUnknown21 p ->
-        Printf.bprintf buf "OvernetUnknown21\n";
+        Printf.bprintf buf "Unknown21\n";
         print_peer buf p
     | _ ->
         Buffer.add_string buf "unknown\n"
@@ -347,7 +348,7 @@ module Make(Proto: sig
 
     open Proto
 
-    let log_prefix = (if Proto.redirector_section = "DKKO" then "[Overnet]" else "[Kademlia]")
+    let log_prefix = (if Proto.redirector_section = "DKKO" then "[OV]" else "[KAD]")
 
     let lprintf_nl fmt =
       lprintf_nl2 log_prefix fmt
@@ -355,6 +356,9 @@ module Make(Proto: sig
     let lprintf_n fmt =
       lprintf2 log_prefix fmt
 
+    let lprintf fmt =
+      lprintf2 log_prefix fmt
+
 (********************************************************************
 
                      STATIC MEMORY
@@ -381,40 +385,15 @@ let overnet_store_size =
   define_option overnet_section [Proto.options_section_name; "store_size"] "Size of the filename storage used to answer queries"
     int_option 2000
 
-let overnet_max_known_peers =
-  define_option overnet_section [Proto.options_section_name; "max_known_peers"]
-  "maximal number of peers to keep overnet connected (should be >2048)"
-    int_option 8192
-
 let overnet_search_keyword =
   define_option overnet_section [Proto.options_section_name; "search_keyword"]
   "allow extended search to search on overnet" bool_option true
 
-let overnet_search_timeout =
-  define_option overnet_section [Proto.options_section_name; "search_timeout"]
-  "How long shoud a search on Overnet wait for the last answer before terminating"
-    int_option 140
-
-let overnet_query_peer_period =
-  define_option overnet_section [Proto.options_section_name; "query_peer_period"]
-  "Period between two queries in the overnet tree (should not be set under 5)"
-    float_option 5.
-
-let overnet_max_search_hits =
-  define_option overnet_section [Proto.options_section_name; "max_search_hits"]
-  "Max number of hits in a search on Overnet"
-    int_option 200
-
 let overnet_republish =
   define_option overnet_section [Proto.options_section_name; "republish"]
   "Interval (in seconds) before republish files"
     float_option 10800.
 
-let overnet_exclude_peers =
-  define_option overnet_section [Proto.options_section_name; "exclude_peers"]
-  "These IP addresses cannot be peers. Elements are separated by spaces, wildcard=255 ie: use 192.168.0.255 for 192.168.0.* "
-    ip_list_option [Ip.of_string "1.0.0.0"]
-
 let overnet_md4 = define_option overnet_section
   [Proto.options_section_name; "md4"]
   "The MD4 of the Overnet client" Md4.option (Md4.random ())
@@ -435,9 +414,6 @@ let gui_overnet_options_panel =
     "Enable Overnet", shortname enable_overnet, "B";
     "Port", shortname overnet_port, "T";
     "Search for keywords", shortname overnet_search_keyword, "B";
-    "Search Timeout", shortname overnet_search_timeout, "T";
-    "Search Internal Period", shortname overnet_query_peer_period, "T";
-    "Search Max Hits", shortname overnet_max_search_hits, "T";
   ]
 
 
@@ -455,6 +431,7 @@ let dummy_peer =
     peer_ip = Ip.null;
     peer_port = 0;
     peer_tcpport = 0;
+    peer_country_code = None;
     peer_kind = 0;
     peer_expire = 0;
     peer_last_send = 0;
@@ -464,9 +441,15 @@ let dummy_peer =
 let connected_peers = ref 0
 let pre_connected_peers = ref 0
 
-let is_overnet_ip ip =
+let check_peer_country_code p =
+  if Geoip.active () then
+    match p.peer_country_code with
+    | None -> p.peer_country_code <- Geoip.get_country_code_option p.peer_ip
+    | _ -> ()
+
+let is_overnet_ip ip cc =
   let is_not_banned ip =
-       match !Ip.banned ip with
+       match !Ip.banned (ip, cc) with
        None -> true
      | Some reason ->
          if !verbose_overnet then
@@ -496,7 +479,7 @@ module LimitedList = struct
 
     let add t key =
       let (ip, port) = key in
-      if ip <> Ip.localhost && is_overnet_ip ip &&
+      if ip <> Ip.localhost && is_overnet_ip ip None &&
         not (Hashtbl.mem t.objects_table key) then
         begin
           Hashtbl.add t.objects_table key key;
@@ -558,7 +541,7 @@ let boot_peers = define_option servers_section
     "List of IP addresses to use to boot Kademlia networks"
     LimitedList.option (LimitedList.create max_boot_peers)
 
-(* peers we may ping *)
+(* peers we may ping: We got only an ip and port *)
 let unknown_peers = LimitedList.create 2000
 
 (* the total number of buckets used. We must fill a bucket before using the
@@ -571,6 +554,9 @@ The buckets should preferably contain peers that have already send us a
   message, because we are not sure for other peers.
   *)
 
+(* FIXME: The function that adds the peers into the buckets should increase 
+   n_used_buckets when needed, but I have the feeling it does not work as expected, 
+   so I set n_used_buckets to some value.*)
 let n_used_buckets = ref 42
 
 (* We distinguish between buckets and prebuckets: peers in buckets are peers
@@ -579,18 +565,19 @@ are peers that we are not sure of. *)
 let buckets = Array.init 129 (fun _ -> Fifo.create ())
 let prebuckets = Array.init 129 (fun _ -> Fifo.create ())
 
+(* The peers we want to ping in the next 60 seconds *)
 let to_ping = ref []
+
+(* Every peer we have in the buckets und prebuckets goes into known_peers *)
 let known_peers = KnownPeers.create 1023
 
 (*
 We keep the data in buckets depending on the number of bits they have
 in common with our identifier. When we exceed the desired storage,
 we start removing associations from the buckets with the fewest common
-bits. Once every 30 minutes, we remove the associations which are older
-than 1 hour.
+bits. 
 *)
 
-
 (* Argh, we MUST verify how MD4s are compared, from left to right, or from
 right to left ?? *)
 
@@ -623,15 +610,19 @@ let common_bits m1 m2 =
 let bucket_number md4 =
   common_bits md4 !!overnet_md4
 
+(* Stats *)
 let search_hits = ref 0
 let source_hits = ref 0
 
-(* when we created the searches for all files last time*)
-let last_check_current_downloads = ref 0
+(* To know if we are can receive UDP packets *)
+let global_last_recv = ref 0
+let global_last_send = ref 0
 
 let udp_sock = ref None
 
 let overnet_searches = ref []
+let current_files = ref []
+
 
 (********************************************************************
 
@@ -734,6 +725,15 @@ module PublishedFiles = Publish(struct
       let item_name = "Files"
     end)
 
+let remove_current_file file =
+  current_files := List.filter (fun (f,start,last) ->
+    if (f != file) then true else false
+  ) !current_files
+
+
+let add_current_file file =
+  current_files := (file, last_time (), 0) :: !current_files
+
 let debug_client ip = false
 (*  Ip.matches ip !!overnet_debug_clients *)
 
@@ -767,27 +767,32 @@ let udp_send_direct ip port msg =
     None -> ()
   | Some sock ->
 (* Why check this? Because it may have been blocked since it was added *)    
-     if ip <> Ip.localhost && is_overnet_ip ip && port <> 0 then
-      Proto.udp_send sock ip port false msg
+     if ip <> Ip.localhost && is_overnet_ip ip None && port <> 0 then begin
+      Proto.udp_send sock ip port false msg;
+      global_last_send := last_time ()
+     end
 
 let udp_send_ping ip port msg =
   match !udp_sock with
     None -> ()
   | Some sock ->
 (* Why check this? Because it may have been blocked since it was added *)    
-     if ip <> Ip.localhost && is_overnet_ip ip && port <> 0 then
-      Proto.udp_send sock ip port true msg
+     if ip <> Ip.localhost && is_overnet_ip ip None && port <> 0 then begin
+      Proto.udp_send sock ip port true msg;
+      global_last_send := last_time ()
+     end
 
 let udp_send p msg =
   p.peer_last_send <- last_time ();
   udp_send_direct p.peer_ip p.peer_port msg
 
 let bootstrap ip port =
-  if !!overnet_update_nodes && is_overnet_ip ip &&
+  if !!overnet_update_nodes && is_overnet_ip ip None &&
      port <> 0 && not (KnownPeers.mem known_peers { dummy_peer with peer_port = port ; peer_ip = ip } ) then
        LimitedList.add unknown_peers (ip,port)
 
 let new_peer p =
+  check_peer_country_code p;
   let ip = p.peer_ip in
   let port = p.peer_port in
  
@@ -797,7 +802,7 @@ let new_peer p =
       pp
     with _ ->
 
-   if ip <> Ip.localhost && is_overnet_ip ip 
+   if ip <> Ip.localhost && is_overnet_ip ip p.peer_country_code
      && port <> 0 && p.peer_created <> 0 then
    
         let bucket = bucket_number p.peer_md4 in
@@ -837,7 +842,8 @@ restart. *)
                 p.peer_md4 <> Md4.null then begin
             Fifo.put prebuckets.(!n_used_buckets) p;
             incr pre_connected_peers;
-
+(* increase n_used_buckets if the buckets and prebuckets are full *)
+(* FIXME: Does it work correctly? Until sure, n_used_buckets is set to some value *)
             while !n_used_buckets < 128 &&
               Fifo.length prebuckets.(!n_used_buckets)
               = max_peers_per_bucket do
@@ -870,10 +876,10 @@ let get_closest_peers md4 nb =
            yet removed because of timeouts *)
         (* TODO: Keep order? Then we need a in_use flag? *)
         if p.peer_kind < 3 && p.peer_expire > last_time () then begin
-            if !verbose_overnet then begin
-            lprintf_nl "Adding good search peer %s:%d"
+            (* if !verbose_overnet then begin
+             lprintf_nl "Adding search peer %s:%d" 
               (Ip.to_string p.peer_ip) p.peer_port;
-	    end;
+            end; *)
             decr nb;
             list := p :: !list;
           end;
@@ -911,6 +917,7 @@ let my_peer () =
     peer_port = !!overnet_port;
     peer_kind = 0;
     peer_tcpport = !!overnet_tcpport;
+    peer_country_code = None;
     peer_last_send = 0;
     peer_expire = 0;
     peer_created = 0;
@@ -943,7 +950,7 @@ let get_any_peers  nb =
   !list
 
 let add_search_peer s p =
-  if p.peer_ip <> Ip.localhost && is_overnet_ip p.peer_ip && 
+  if p.peer_ip <> Ip.localhost && is_overnet_ip p.peer_ip p.peer_country_code && 
      p.peer_port <> 0 && p.peer_created <> 0 then begin
     let nbits = common_bits p.peer_md4 s.search_md4 in
 (* Don't add ourself *)
@@ -962,7 +969,7 @@ let add_search_peer s p =
 
 let create_search kind md4 =
   if !verbose_overnet then lprintf_nl "create_search";
-  let starttime = last_time () + (3 * List.length !overnet_searches) in
+  let starttime = last_time () + (List.length !overnet_searches) in
   let s = ref {
       search_md4 = md4;
       search_kind = kind;
@@ -1069,17 +1076,18 @@ let udp_client_handler t p =
   let other_ip = ip_of_udp_packet p in
   let other_port = port_of_udp_packet p in
   if !verbose_overnet then
-    lprintf_nl "UDP FROM %s:%d type %s"
+    lprintf_nl "UDP from %s:%d type %s"
       (Ip.to_string other_ip) other_port
       (message_to_string t);
   (* Emule uses other_ip:other_port, so do we *)
   (* Update known_peers by other_port:other_ip *)
   let sender = new_peer { dummy_peer with peer_port = other_port ; peer_ip = other_ip } in
   new_peer_message sender;
+  global_last_recv := last_time ();
   match t with
 
   | OvernetConnect p ->
-      if is_overnet_ip sender.peer_ip && sender.peer_port <> 0 then
+      if is_overnet_ip sender.peer_ip p.peer_country_code && sender.peer_port <> 0 then
         let sender = new_peer { p with peer_ip = other_ip; peer_kind = 2 } in
         (* let sender = new_peer { p with peer_port = other_port ; peer_ip = other_ip } in *)
         new_peer_message sender;
@@ -1109,7 +1117,7 @@ let udp_client_handler t p =
   | OvernetPublicize p ->
       let sender = new_peer { p with peer_ip = other_ip; peer_kind = 2 } in
       new_peer_message sender;
-      if is_overnet_ip sender.peer_ip && sender.peer_port <> 0 then
+      if is_overnet_ip sender.peer_ip p.peer_country_code && sender.peer_port <> 0 then
         udp_send sender (OvernetPublicized (Some (my_peer ())))
        else begin
 	      if !verbose_overnet then
@@ -1127,6 +1135,7 @@ let udp_client_handler t p =
       ()
 
   | OvernetSearch (nresults, md4, from_who) ->
+      let nresults = min 10 nresults in
       let peers = get_closest_peers md4 nresults in
       udp_send sender (OvernetSearchReply (md4,peers))
 
@@ -1189,9 +1198,9 @@ let udp_client_handler t p =
                   List.iter (fun p ->
                       let ip = p.peer_ip in
                       let port = p.peer_tcpport in
-                      if is_overnet_ip ip && port <> 0 then
-                        let s = DonkeySources.find_source_by_uid
-                            (Direct_address (ip, port))  in
+                      if is_overnet_ip ip p.peer_country_code && port <> 0 then
+                        let s = DonkeySources.create_source_by_uid
+                            (Direct_address (ip, port)) None in
 			if !verbose_overnet then
 			  lprintf_nl "added new source %s:%d for file %s"
 			    (Ip.to_string ip) port (Md4.to_string md4);
@@ -1323,11 +1332,7 @@ let overnet_search_tick () =
   ) !overnet_searches
 
 let recover_file file =
-  if file_state file = FileDownloading then
-    ignore (create_search (FileSearch file) file.file_md4)
-
-let check_current_downloads () =
-  List.iter recover_file !DonkeyGlobals.current_files
+  add_current_file file
 
 let update_buckets () =
 
@@ -1340,7 +1345,8 @@ let update_buckets () =
       let p = Fifo.take b in
       (* bad peers have kind = 4 and did not respond within peer_expire *)
       (* Why check is_overnet_ip? Because it may have been blocked since it was added *)
-      if not (p.peer_kind = 4 && p.peer_expire <= last_time ()) && is_overnet_ip p.peer_ip then
+      if not (p.peer_kind = 4 && p.peer_expire <= last_time ()) &&
+             is_overnet_ip p.peer_ip p.peer_country_code then
         Fifo.put b p 
       else
       begin
@@ -1372,7 +1378,7 @@ let update_buckets () =
             (* bad peers are removed *)    
             (* Why check is_overnet_ip? Because it may have been blocked since it was added *)
             end else if (p.peer_kind = 4 && p.peer_expire <= last_time ()) || 
-                        not (is_overnet_ip p.peer_ip) then begin
+                        not (is_overnet_ip p.peer_ip p.peer_country_code) then begin
               decr pre_connected_peers;
               KnownPeers.remove known_peers p;
               if !verbose_overnet then lprintf_nl "update_bucket2: removing %s:%d" (Ip.to_string p.peer_ip) p.peer_port;
@@ -1438,6 +1444,27 @@ let compute_to_ping () =
           end;
           ()
 
+(* start max three searches for sources *)
+let start_max_source_searches () =
+      begin
+          let current_files2 = ref [] in
+          (try
+            while (List.length !overnet_searches) < 3 && !current_files <> [] do
+              let (file, start, last) = List.hd !current_files in
+	      current_files := List.tl !current_files;
+              if file_state file = FileDownloading &&
+	         (last + 900) < last_time () then begin
+	        ignore (create_search (FileSearch file) file.file_md4);
+   	        current_files2 := (file, start, last_time ()) :: !current_files2;
+	      end else
+   	        current_files2 := (file, start, last) :: !current_files2;
+	    done;
+	  with Failure "hd" -> ()
+	     | Failure "tl" -> (); );
+          List.iter (fun (file, start, last) ->
+            current_files := (file, start, last) :: !current_files
+	  ) !current_files2;
+       end	
 
 let enable () =
   if !!enable_overnet && not !is_enabled then begin
@@ -1455,7 +1482,7 @@ let enable () =
 
 (* copy all boot_peers to unknown_peers *)
       LimitedList.iter (fun (ip, port) ->
-          if ip <> Ip.localhost && is_overnet_ip ip && port <> 0 then
+          if ip <> Ip.localhost && is_overnet_ip ip None && port <> 0 then
             LimitedList.add unknown_peers (ip, port)
           ) !!boot_peers;
 
@@ -1520,12 +1547,42 @@ let enable () =
 
           compute_to_ping ();
 
+(* dump searches older than lifetime to the logfile *)
+          if !verbose_overnet then List.iter (fun s ->
+            if s.search_lifetime + s.search_start <= last_time () then begin
+              lprintf_nl "Removing search %s for %s (requests:%d queries:%d seconds alive:%d lifetime:%d)"
+                (match s.search_kind with
+                  KeywordSearch _ -> "keyword"
+                  | FileSearch _ -> "file"
+                  | FillBuckets -> "fillbuckets" )
+                (Md4.to_string s.search_md4) s.search_requests s.search_queries (last_time ()-s.search_start) s.search_lifetime;
+                for i = 128 downto 0 do
+                  let npeers = Fifo.length s.search_peers.(i) in
+                  let count = ref 0 in
+                  let cp p = if (common_bits p.peer_md4 s.search_md4) = i then count := !count + 1 in
+                  KnownPeers.iter cp s.search_asked_peers;
+                  let nasked = !count in
+                  count := 0;
+                  KnownPeers.iter cp s.search_ok_peers;
+                  let nok = !count in
+                  count := 0;
+                  KnownPeers.iter cp s.search_result_asked_peers;
+                  let nres = !count in
+                  if npeers > 0 || nasked > 0 then
+                    lprintf_nl
+                    " nbits[%d] = %d peer(s) total, %d peer(s) asked, %d peer(s) ok, %d peer(s) result asked"
+                      i npeers nasked nok nres
+                done;
+            end;
+          ) !overnet_searches;
+
 (* remove searches that are older than their lifetime *)
           overnet_searches := List.filter (fun s ->
-              (* s.search_lifetime + s.search_start > last_time () *)
-(*DEBUG:show longer *)              360 + s.search_start > last_time ()
+              s.search_lifetime + s.search_start > last_time ()
           ) !overnet_searches;
 
+          start_max_source_searches ();
+
           (* FIXE: Dump latencies to logfile *)
           if !verbose_overnet then ignore (UdpSocket.get_latencies (ref true));
 
@@ -1544,21 +1601,36 @@ let enable () =
 (* every 15min for light operations *)
       add_session_timer enabler 900. (fun _ ->
           if !!enable_overnet then begin
-              let _ = create_search FillBuckets !!overnet_md4 in
-              check_current_downloads ();
-              last_check_current_downloads := last_time ();
+              ignore(create_search FillBuckets !!overnet_md4);
+	      (* Remove all files not actuall downloading/pause/new etc *)
+                (* TODO: Are there states enough? There are:  FileDownloading
+                | FileQueued
+                | FilePaused
+                | FileDownloaded
+                | FileShared
+                | FileCancelled
+                | FileNew
+                | FileAborted of string
+                *)
+              current_files := List.filter (fun (f,_,_) ->
+                match file_state f with
+                    FileDownloading
+                  | FileQueued
+                  | FilePaused
+                  | FileNew -> true;
+                  | _ -> false;
+              ) !current_files;
             end
       );
 
-      begin
-        check_current_downloads ();
-(* Delay the first normal searches, so we can fill the buckets first *)
-        List.iter (fun s ->
-          s.search_start <- s.search_start + 30;
-        ) !overnet_searches;
+      begin 
+        (* First fill the buckets *)
         ignore (create_search FillBuckets !!overnet_md4);
-        last_check_current_downloads := last_time ();
+        (* start with all actual files *)
+        List.iter add_current_file !DonkeyGlobals.current_files
       end;
+
+      start_max_source_searches ();
       
       add_infinite_timer 1800. (fun _ ->
           if !!enable_overnet then begin
@@ -1589,9 +1661,7 @@ let _ =
     (fun _ ->
       if !CommonOptions.start_running_plugins then
         if !!enable_overnet = false then disable() else enable ()
-  );
-  option_hook overnet_query_peer_period
-    (fun _ -> if !!overnet_query_peer_period < 5. then overnet_query_peer_period =:= 5.)
+  )
 
 let load_contact_dat filename =
   try
@@ -1709,16 +1779,16 @@ let _ =
         let buf = o.conn_buf in
         html_mods_commands buf "commandsTable" "commands" ([
           ("bu bbig", command_prefix_to_net ^ " boots list",
-           "top.output.location.href='submit\\?q=" ^ command_prefix ^ "boots'",
+           "mSub('output','" ^ command_prefix ^ "boots')",
            command_prefix_to_net ^ " boots");
           ("bu bbig", command_prefix_to_net ^ " buckets list",
-           "top.output.location.href='submit\\?q=" ^ command_prefix ^ "buckets'",
+           "mSub('output','" ^ command_prefix ^ "buckets')",
            command_prefix_to_net ^ " buckets");
           ("bu bbig", command_prefix_to_net ^ " stats",
-           "top.output.location.href='submit\\?q=" ^ command_prefix ^ "stats'",
+           "mSub('output','" ^ command_prefix ^ "stats')",
            command_prefix_to_net ^ " stats");
           ("bu bbig", command_prefix_to_net ^ " store list",
-           "top.output.location.href='submit\\?q=" ^ command_prefix ^ "store'",
+           "mSub('output','" ^ command_prefix ^ "store')",
            command_prefix_to_net ^ " store");
           ]);
         "";
@@ -1728,26 +1798,56 @@ let _ =
         let buf = o.conn_buf in
         if o.conn_output = HTML then
           begin
+            Printf.bprintf buf "\\<div class=results\\>";
+            html_mods_table_header buf "ovstatsTable" "sources" [];
+            Printf.bprintf buf "\\<tr\\>";
+            html_mods_td buf [
+              ("", "srh", Printf.sprintf "%s %s\n" (command_prefix_to_net)
+                  (if !!enable_overnet then "is enabled" else "is DISABLED"));
+              ("", "srh", Printf.sprintf "%s Connectivity: %s%s" (command_prefix_to_net)
+                   (if (!connected_peers + !pre_connected_peers) > 20 then "Got enough online peers" else "NOT enough online peers")
+                   (if (abs(!global_last_recv - !global_last_send)) > 60 then " ,there maybe a problem with incoming upd packets" else "" ));
+
+            ];
+            Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>\n";
+
             Printf.bprintf buf "\\<div class=results\\>";
             html_mods_table_header buf "ovstatsTable" "sources" [];
             Printf.bprintf buf "\\<tr\\>";
             html_mods_td buf [
               ("", "srh", Printf.sprintf "%s statistics" command_prefix_to_net);
-              ("", "srh", Printf.sprintf "Last file search started %d seconds ago\n" 
-                          (last_time () - !last_check_current_downloads));
               ("", "srh", Printf.sprintf "Search hits: %d\n" !search_hits);
-              ("", "srh", Printf.sprintf "Source hits: %d\n" !source_hits); ];
+              ("", "srh", Printf.sprintf "Source hits: %d\n" !source_hits); 
+              ("", "srh", Printf.sprintf "Current files: %d\n" (List.length !current_files)); ];
+            if !verbose_overnet then begin
+              Printf.bprintf buf "\\</tr\\>\\<tr\\>\n";
+              html_mods_td buf [
+                ("", "srh", Printf.sprintf "Verbose:");
+                ("", "srh", Printf.sprintf "last upd send: %d\n" !global_last_send);
+                ("", "srh", Printf.sprintf "last upd recv: %d\n" !global_last_recv);
+                ("", "srh", Printf.sprintf "diff: %d\n" (abs(!global_last_recv - !global_last_send)));
+              ];
+            end;
             Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>\n";
           end
         else
           begin
+            Printf.bprintf buf "%s %s\n" (command_prefix_to_net) 
+               (if !!enable_overnet then "is enabled" else "is DISABLED");
+            Printf.bprintf buf "%s Connectivity: %s%s\n" (command_prefix_to_net)
+                   (if (!connected_peers + !pre_connected_peers) > 20 then "Got enough online peers" else "NOT enough online peers")
+                   (if (abs(!global_last_recv - !global_last_send)) > 60 then " ,there maybe a problem with incoming upd packets" else "");
             Printf.bprintf buf "%s statistics:\n"
 	    (command_prefix_to_net);
-            Printf.bprintf buf "  Last file search started %d seconds ago\n"
-            (last_time () - !last_check_current_downloads);
             Printf.bprintf buf "  Search hits: %d\n" !search_hits;
             Printf.bprintf buf "  Source hits: %d\n" !source_hits;
+            Printf.bprintf buf "  Current files: %d\n" (List.length !current_files);
           end;
+(* Only for debuging current_file
+          List.iter (fun (file,start,last) ->
+          Printf.bprintf buf "current_file %s (%d,%d)\n" (file_disk_name file) start last
+        ) !current_files;
+*)
         List.iter (fun s ->
             if o.conn_output = HTML then
               begin
@@ -1816,15 +1916,21 @@ let _ =
         let urls =
           match args with
             [] -> let list = ref [] in
-              List.iter (fun (kind,_, url) ->
-                  if kind = web_info then list := url :: !list
-              )!!web_infos;
+              Hashtbl.iter (fun key w ->
+                  if w.kind = web_info then list := w :: !list
+              ) web_infos_table;
               !list
-          | _ -> args
+          | _ -> List.map (fun url -> 
+                    { 
+                      url = url;
+                      kind = web_info;
+                      period = 0;
+                      state = None;
+                    }) args
         in
-        List.iter (fun url ->
-            Printf.bprintf o.conn_buf "Loading %s\n" url;
-            CommonWeb.load_url true web_info url) urls;
+        List.iter (fun w ->
+            Printf.bprintf o.conn_buf "Loading %s\n" w.url;
+            CommonWeb.load_url true w) urls;
         "web boot started"
     ), "<urls> : \t\t\tdownload .ocl URLs (no arg load default)";
 
@@ -1975,11 +2081,12 @@ let cancel_recover_file file =
    overnet_searches := List.filter (fun s ->
       match s.search_kind with
         FileSearch f when f == file -> false
-      | _ -> true) !overnet_searches
-   end   
+      | _ -> true) !overnet_searches;
+   remove_current_file file;
+   end  
 
 let _ =
-  CommonWeb.add_web_kind web_info web_info_descr (fun _ filename ->
+  CommonWeb.add_web_kind web_info web_info_descr (fun url filename ->
       let s = File.to_string filename in
       let s = String2.replace s '"' "" in
       let lines = String2.split_simplify s '\n' in
@@ -1993,7 +2100,7 @@ let _ =
                     let port = int_of_string port in
                     if !verbose_overnet then
                       lprintf_nl "Adding %s peer %s:%d" command_prefix_to_net name port;
-                    bootstrap ip port)
+                    bootstrap ip port) (fun _ -> ())
             | _ -> lprintf_nl "BAD LINE ocl: %s" s;
           with _ -> lprintf_nl "DNS failed";
       ) lines
@@ -2008,10 +2115,10 @@ let _ =
             lprintf_nl "contact.dat loaded from %s, added %d peers" url n;
 	else
 	  if not !!enable_overnet then
-      lprintf_nl "Overnet module is disabled, ignoring..."
+            lprintf_nl "Overnet module is disabled, ignoring..."
 	  else
-      lprintf_nl "Overnet_update_nodes is disabled, ignoring..."
-        );
+            lprintf_nl "Overnet_update_nodes is disabled, ignoring..."
+      );
 
 (*************************************************************
 
