@@ -39,67 +39,110 @@ let get_emule_version () =
     (int_of_string(Autoconf.minor_version) lsl 10) lor
     (int_of_string(Autoconf.sub_version) lsl 7)
 
-(* TODO : update this
-I downgraded some of those to get better results :
-We don't use emule udp extension, client_md4 in sourceexchange or complete sources in
-file request *)
 let mldonkey_emule_proto = 
   {
-    emule_comments = 1;
     emule_version = get_emule_version (); 
     emule_release = "";
-    emule_secident = 3; (* Emule uses v1 if advertising both, v2 if only advertising 2 *)
-    emule_noviewshared = 0;
-    emule_supportpreview = 0;
     emule_osinfosupport = 1;
-    emule_compression = 1; (* 1 *)
+    emule_features = 3;
+
+(* emule_miscoptions1 *)
+    received_miscoptions1 = false;
+    emule_aich = 0;
+    emule_unicode = 0;
+    emule_udpver = 0;
+    emule_compression = 1;
+    emule_secident = 3; (* Emule uses v1 if advertising both, v2 if only advertising 2 *)
     emule_sourceexchange = 2; (* 2 : +client_md4 3 : +IdHybrid (emule Kademlia?)*)
-    emule_multipacket = 0; (* 1 *)
     emule_extendedrequest = 1; (* 1: +file_status 2: +ncomplete_sources*)
-    emule_features = 3; (* 3 *)
-    emule_udpver = 0; (* 4 *)
+    emule_comments = 1;
+    emule_peercache = 0;
+    emule_noviewshared = 0;
+    emule_multipacket = 0;
+    emule_supportpreview = 0;
+
+(* emule_miscoptions2 *)
+    received_miscoptions2 = false;
+    emule_require_crypt = 0;
+    emule_request_crypt = 0;
+    emule_support_crypt = 0;
+    emule_extmultipacket = 0;
+    emule_largefiles = 1;
+    emule_kad_version = 0;
   }
 
 let emule_miscoptions1 m =
   let o =
+    (m.emule_aich lsl 29) lor
+    (m.emule_unicode lsl 28) lor
     (m.emule_udpver lsl 24) lor
     (m.emule_compression lsl 20) lor
     (m.emule_secident lsl 16) lor
     (m.emule_sourceexchange lsl 12) lor
     (m.emule_extendedrequest lsl 8) lor
     (m.emule_comments lsl 4) lor
+    (m.emule_peercache lsl 3) lor
     (m.emule_noviewshared lsl 2) lor
     (m.emule_multipacket lsl 1) lor
     (m.emule_supportpreview lsl 0)
   in
   Int64.of_int o
 
+let update_emule_proto_from_miscoptions1 m o =
+  let o = Int64.to_int o in
+  m.emule_aich            <- (o lsr 29) land 0x7;
+  m.emule_unicode         <- (o lsr 28) land 0xf;
+  m.emule_udpver          <- (o lsr 24) land 0xf;
+  m.emule_compression     <- (o lsr 20) land 0xf;
+  m.emule_secident        <- (o lsr 16) land 0xf;
+  m.emule_sourceexchange  <- (o lsr 12) land 0xf;
+  m.emule_extendedrequest <- (o lsr  8) land 0xf;
+  m.emule_comments        <- (o lsr  4) land 0xf;
+  m.emule_peercache       <- (o lsr  3) land 0x1;
+  m.emule_noviewshared    <- (o lsr  2) land 0x1;
+  m.emule_multipacket     <- (o lsr  1) land 0x1;
+  m.emule_supportpreview  <- (o lsr  0) land 0x1
+
+let print_emule_proto_miscoptions1 m =
+  let buf = Buffer.create 50 in
+  if m.emule_aich <> 0 then Printf.bprintf buf " aich %d\n" m.emule_aich;
+  if m.emule_unicode <> 0 then Printf.bprintf buf " unicode %d\n" m.emule_unicode;
+  if m.emule_udpver <> 0 then Printf.bprintf buf " udpver %d\n" m.emule_udpver;
+  if m.emule_compression <> 0 then Printf.bprintf buf " compression %d\n" m.emule_compression;
+  if m.emule_secident <> 0 then Printf.bprintf buf " secident %d\n" m.emule_secident;
+  if m.emule_sourceexchange <> 0 then Printf.bprintf buf " sourceexchange %d\n" m.emule_sourceexchange;
+  if m.emule_extendedrequest <> 0 then Printf.bprintf buf " extendedrequest %d\n" m.emule_extendedrequest;
+  if m.emule_comments <> 0 then Printf.bprintf buf " comments %d\n" m.emule_comments;
+  if m.emule_peercache <> 0 then Printf.bprintf buf " peercache %d\n" m.emule_peercache;
+  if m.emule_noviewshared <> 0 then Printf.bprintf buf " noviewshared %d\n" m.emule_noviewshared;
+  if m.emule_multipacket <> 0 then Printf.bprintf buf " multipacket %d\n" m.emule_multipacket;
+  if m.emule_supportpreview <> 0 then Printf.bprintf buf " supportpreview %d\n" m.emule_supportpreview;
+  Buffer.contents buf
+
 let emule_miscoptions2 m =
-(*
   let o =
     (m.emule_largefiles lsl 4)
   in
   Int64.of_int o
-*)
-  Int64.zero
 
-let update_emule_proto_from_miscoptions1 m o =
+let update_emule_proto_from_miscoptions2 m o =
   let o = Int64.to_int o in
-  m.emule_udpver <- (o lsr 24) land 0xf;
-  m.emule_compression <- (o lsr 20) land 0xf;
-  m.emule_secident <- (o lsr 16) land 0xf;
-  m.emule_sourceexchange <- (o lsr 12) land 0xf;
-  m.emule_extendedrequest <- (o lsr 8) land 0xf;
-  m.emule_comments <- (o lsr 4) land 0xf;
-  m.emule_noviewshared <- (o lsr 2) land 0x1;
-  m.emule_multipacket <- (o lsr 1) land 0x1;
-  m.emule_supportpreview <- (o lsr 0) land 0x1
+  m.emule_require_crypt  <- (o lsr 9) land 0x1;
+  m.emule_request_crypt  <- (o lsr 8) land 0x1;
+  m.emule_support_crypt  <- (o lsr 7) land 0x1;
+  m.emule_extmultipacket <- (o lsr 5) land 0x1;
+  m.emule_largefiles     <- (o lsr 4) land 0x1;
+  m.emule_kad_version    <- (o lsr 0) land 0xf
 
-let update_emule_proto_from_miscoptions2 m o = ()
-(*
-  let o = Int64.to_int o in
-  m.emule_largefiles <- (o lsr 4) land 0x1
-*)
+let print_emule_proto_miscoptions2 m =
+  let buf = Buffer.create 50 in
+  if m.emule_require_crypt <> 0 then Printf.bprintf buf " require_crypt %d\n" m.emule_require_crypt;
+  if m.emule_request_crypt <> 0 then Printf.bprintf buf " request_crypt %d\n" m.emule_request_crypt;
+  if m.emule_support_crypt <> 0 then Printf.bprintf buf " support_crypt %d\n" m.emule_support_crypt;
+  if m.emule_extmultipacket <> 0 then Printf.bprintf buf " extmultipacket %d\n" m.emule_extmultipacket;
+  if m.emule_largefiles <> 0 then Printf.bprintf buf " largefiles %d\n" m.emule_largefiles;
+  if m.emule_kad_version <> 0 then Printf.bprintf buf " kad_version %d\n" m.emule_kad_version;
+  Buffer.contents buf
 
 let emule_compatoptions m =
   (m.emule_osinfosupport lsl 0)
@@ -146,23 +189,10 @@ module Connect  = struct
         left_bytes : string;
       }
 
+    let names_of_tag = client_common_tags
+
     let names_of_tag =
-      [
-        "\001", Field_UNKNOWN "name";
-        "\015", Field_UNKNOWN "port";
-        "\017", Field_UNKNOWN "version";
-        "\031", Field_UNKNOWN "udpport";
-        "\060", Field_UNKNOWN "downloadtime";
-        "\061", Field_UNKNOWN "incompleteparts";
-        "\085", Field_UNKNOWN "mod_version";
-        "\239", Field_UNKNOWN "emule_compatoptions";
-        "\249", Field_UNKNOWN "emule_udpports";
-        "\250", Field_UNKNOWN "emule_miscoptions1";
-        "\251", Field_UNKNOWN "emule_version";
-        "\252", Field_UNKNOWN "buddy_ip";
-        "\253", Field_UNKNOWN "buddy_udp";
-        "\254", Field_UNKNOWN "emule_miscoptions2";
-      ]
+      List.map (fun (v, name) -> (v, Field_KNOWN name)) names_of_tag
 
     let parse reply len s =
       let hash_len, pos = if not reply then get_uint8 s 1, 2 else -1, 1 in
@@ -388,7 +418,6 @@ module QueryChunksReply = struct (* Request 80 *)
 
     let print t =
       lprintf_nl "CHUNKS for %s" (Md4.to_string t.md4);
-      lprint_string "   ";
       lprintf_nl "%s\n" (Bitv.to_string t.chunks)
 
     let write buf t =
@@ -423,7 +452,7 @@ module QueryChunkMd4Reply = struct (* Request 80 *)
       }
 
     let print t =
-      lprintf_nl "CHUNKS for %s" (Md4.to_string t.md4);
+      lprintf_nl "CHUNKSMd4 for %s" (Md4.to_string t.md4);
       lprint_string "   ";
       Array.iter (fun b ->
           lprintf "  %s" (Md4.to_string b))
@@ -462,6 +491,7 @@ module QueryFileReply  = struct
 module Bloc  = struct
     type t = {
         md4 : Md4.t;
+        usesixtyfour : bool;
         start_pos : int64;
         end_pos: int64;
         bloc_str: string;
@@ -469,14 +499,15 @@ module Bloc  = struct
         bloc_len : int;
       }
 
-    let parse len s =
+    let parse usesixtyfour len s =
       {
         md4 = get_md4 s 1;
-        start_pos = get_uint64_32 s 17;
-        end_pos = get_uint64_32 s 21;
+        usesixtyfour = usesixtyfour;
+        start_pos = if usesixtyfour then get_int64 s 17 else get_uint64_32 s 17;
+        end_pos   = if usesixtyfour then get_int64 s 25 else get_uint64_32 s 21;
         bloc_str = s;
-        bloc_begin = 25;
-        bloc_len = len - 25;
+        bloc_begin = if usesixtyfour then 33 else 25;
+        bloc_len = if usesixtyfour then len - 33 else len - 25;
       }
 
     let print t =
@@ -487,14 +518,15 @@ module Bloc  = struct
 
     let write buf t =
       buf_md4 buf t.md4;
-      buf_int64_32 buf t.start_pos;
-      buf_int64_32 buf t.end_pos;
+      if t.usesixtyfour then buf_int64 buf t.start_pos else buf_int64_32 buf t.start_pos;
+      if t.usesixtyfour then buf_int64 buf t.end_pos else buf_int64_32 buf t.end_pos;
       Buffer.add_substring buf t.bloc_str t.bloc_begin t.bloc_len
   end
 
 module QueryBloc  = struct
     type t = {
         md4 : Md4.t;
+        usesixtyfour : bool;
         start_pos1 : int64; (* 180 ko *)
         end_pos1: int64;
         start_pos2 : int64;
@@ -503,15 +535,16 @@ module QueryBloc  = struct
         end_pos3: int64;
       }
 
-    let parse len s =
+    let parse usesixtyfour len s =
       {
         md4 = get_md4 s 1;
-        start_pos1 = get_uint64_32 s 17;
-        end_pos1 = get_uint64_32 s 29;
-        start_pos2 = get_uint64_32 s 21;
-        end_pos2 = get_uint64_32 s 33;
-        start_pos3 = get_uint64_32 s 25;
-        end_pos3 = get_uint64_32 s 37;
+        usesixtyfour = usesixtyfour;
+        start_pos1 = if usesixtyfour then get_int64 s 17 else get_uint64_32 s 17;
+        end_pos1   = if usesixtyfour then get_int64 s 41 else get_uint64_32 s 29;
+        start_pos2 = if usesixtyfour then get_int64 s 25 else get_uint64_32 s 21;
+        end_pos2   = if usesixtyfour then get_int64 s 49 else get_uint64_32 s 33;
+        start_pos3 = if usesixtyfour then get_int64 s 33 else get_uint64_32 s 25;
+        end_pos3   = if usesixtyfour then get_int64 s 57 else get_uint64_32 s 37;
       }
 
     let print t =
@@ -523,12 +556,12 @@ module QueryBloc  = struct
 
     let write buf t =
       buf_md4 buf t.md4;
-      buf_int64_32 buf t.start_pos1;
-      buf_int64_32 buf t.start_pos2;
-      buf_int64_32 buf t.start_pos3;
-      buf_int64_32 buf t.end_pos1;
-      buf_int64_32 buf t.end_pos2;
-      buf_int64_32 buf t.end_pos3
+      if t.usesixtyfour then buf_int64 buf t.start_pos1 else buf_int64_32 buf t.start_pos1;
+      if t.usesixtyfour then buf_int64 buf t.start_pos2 else buf_int64_32 buf t.start_pos2;
+      if t.usesixtyfour then buf_int64 buf t.start_pos3 else buf_int64_32 buf t.start_pos3;
+      if t.usesixtyfour then buf_int64 buf t.end_pos1 else buf_int64_32 buf t.end_pos1;
+      if t.usesixtyfour then buf_int64 buf t.end_pos2 else buf_int64_32 buf t.end_pos2;
+      if t.usesixtyfour then buf_int64 buf t.end_pos3 else buf_int64_32 buf t.end_pos3
   end
 
 let unit = ()
@@ -554,7 +587,7 @@ module NoArg = functor(M: sig val m : string end) -> (struct
 
 module AvailableSlot = NoArg(struct let m = "AvailableSlot" end)
 module ReleaseSlot = NoArg(struct let m = "ReleaseSlot" end)
-module CloseSlot = NoArg(struct let m = "CloseSlot" end)
+module OutOfParts = NoArg(struct let m = "OutOfParts" end)
 module ViewFiles = NoArg(struct let m = "VIEW FILES" end)
 module ViewDirs = NoArg(struct let m = "VIEW DIRS" end)
 
@@ -787,49 +820,10 @@ module EmuleClientInfo = struct
         mutable tags : tag list;
       }
 
-    let names_of_tag =
-      [
-        "\032", "compression";
-        "\033", "udpport";
-        "\034", "udpver";
-        "\035", "sourceexchange";
-        "\036", "comments";
-        "\037", "extendedrequest";
-        "\038", "compatibleclient";
-        "\039", "features";
-        "\060", "downloadtime";
-        "\061", "incompleteparts";
-        "\062", "l2hac";
-        "\065", "mod_unknown41";
-        "\066", "mod_unknown42";
-        "\067", "mod_unknown43";
-        "\084", "mod_featureset";
-        "\086", "mod_protocol";
-        "\085", "mod_version";
-        "\090", "mod_bowlfish";
-        "\092", "mod_secure_community";
-        "\102", "mod_fusion";
-        "\103", "mod_fusion_version";
-        "\119", "mod_tarod";
-        "\120", "mod_tarod_version";
-        "\121", "mod_morph";
-        "\128", "mod_morph_version";
-        "\130", "mod_mortillo";
-        "\131", "mod_mortillo_version";
-        "\132", "chanblard_version";
-        "\133", "signature";
-        "\134", "cache";
-        "\135", "mod_lsd";
-        "\136", "mod_lsd_version";
-        "\144", "mod_lovelace_version";
-        "\148", "os_info"; (* reused by aMule to transfer client OS type *)
-        "\153", "mod_plus";
-        "\160", "mod_wombat";
-        "\161", "dev_wombat";
-      ]
+    let names_of_tag = client_common_tags
 
     let names_of_tag =
-      List.map (fun (v, name) -> (v, Field_UNKNOWN name)) names_of_tag
+      List.map (fun (v, name) -> (v, Field_KNOWN name)) names_of_tag
 
     let parse len s =
       let version = get_uint8 s 1 in
@@ -979,6 +973,7 @@ module EmuleRequestSourcesReply = struct
         mutable src_server_ip : Ip.t;
         mutable src_server_port : int;
         mutable src_md4 : Md4.t;
+        mutable src_cc : int option;
       }
 
     type t = {
@@ -992,6 +987,7 @@ module EmuleRequestSourcesReply = struct
         src_server_ip = Ip.null;
         src_server_port = 0;
         src_md4 = Md4.null;
+        src_cc = None;
       }
 
     let parse e len s =
@@ -1005,10 +1001,13 @@ module EmuleRequestSourcesReply = struct
         let sources = Array.create ncount dummy_source in
         let rec iter pos i =
           if i < ncount then
-            let ss = {
+            let ss =
+              let ip = get_ip s pos in
+              {
                 dummy_source with
-                src_ip = get_ip s pos;
+                src_ip = ip;
                 src_port = get_int16 s (pos+4);
+                src_cc = Geoip.get_country_code_option ip
               } in
             let pos =
               if slen > 6 then begin
@@ -1090,6 +1089,35 @@ module EmuleFileDesc = struct
       buf_string buf t.comment
   end
 
+module EmuleCompressedPart = struct
+
+    type t = {
+        md4 : Md4.t;
+        usesixtyfour : bool;
+        statpos : int64;
+        newsize : int64;
+        bloc : string;
+      }
+
+    let parse usesixtyfour len s =
+      {
+        md4 = get_md4 s 1;
+        usesixtyfour = usesixtyfour;
+        statpos = if usesixtyfour then get_int64 s 17 else get_uint64_32 s 17;
+        newsize = if usesixtyfour then get_uint64_32 s 25 else get_uint64_32 s 21;
+    	bloc = if usesixtyfour then String.sub s 29 (len-29) else String.sub s 25 (len-25)
+      }
+
+    let print t =
+      lprintf_nl "EmuleCompressedPart for %s %Ld %Ld len %d"
+	(Md4.to_string t.md4) t.statpos t.newsize (String.length t.bloc)
+
+    let write buf t =
+      buf_md4 buf t.md4;
+      if t.usesixtyfour then buf_int64 buf t.statpos else buf_int64_32 buf t.statpos;
+      buf_int64_32 buf t.newsize;
+      Buffer.add_string buf t.bloc
+  end
 
 module EmulePortTestReq = struct
 
@@ -1115,7 +1143,7 @@ type t =
 | JoinQueueReq of JoinQueue.t (* sent before queryBloc *)
 | AvailableSlotReq of AvailableSlot.t
 | ReleaseSlotReq of ReleaseSlot.t
-| CloseSlotReq of CloseSlot.t
+| OutOfPartsReq of OutOfParts.t
 | QueryChunksReq of QueryChunks.t
 | QueryChunksReplyReq of QueryChunksReply.t
 | QueryChunkMd4Req of QueryChunkMd4.t
@@ -1147,7 +1175,7 @@ type t =
 | EmuleSecIdentStateReq  of EmuleSecIdentStateReq.t
 | EmuleMultiPacketReq of Md4.t * t list
 | EmuleMultiPacketAnswerReq of Md4.t * t list
-| EmuleCompressedPart of Md4.t * int64 * int64 * string
+| EmuleCompressedPart of EmuleCompressedPart.t
 | EmulePortTestReq of EmulePortTestReq.t
 
 let rec print t =
@@ -1162,7 +1190,7 @@ let rec print t =
     | JoinQueueReq t -> JoinQueue.print t
     | AvailableSlotReq t -> AvailableSlot.print t
     | ReleaseSlotReq t -> ReleaseSlot.print t
-    | CloseSlotReq t -> CloseSlot.print t
+    | OutOfPartsReq t -> OutOfParts.print t
     | QueryChunksReq t -> QueryChunks.print t
     | QueryChunksReplyReq t -> QueryChunksReply.print t
     | QueryChunkMd4Req t -> QueryChunkMd4.print t
@@ -1216,9 +1244,8 @@ let rec print t =
         EmuleSignatureReq.print t
     | EmulePublicKeyReq t ->
         EmulePublicKeyReq.print t
-    | EmuleCompressedPart (md4, statpos, newsize, bloc) ->
-        lprintf_nl "EmuleCompressedPart for %s %Ld %Ld len %d"
-          (Md4.to_string md4) statpos newsize (String.length bloc)
+    | EmuleCompressedPart t ->
+        EmuleCompressedPart.print t
     | EmulePortTestReq t ->
         EmulePortTestReq.print t
     | UnknownReq (opcode, s) ->
@@ -1265,11 +1292,7 @@ let rec parse_emule_packet emule opcode len s =
 
     | 0x40 (* 64 *) ->
 (* OP_COMPRESSEDPART *)
-        let md4 = get_md4 s 1 in
-        let statpos = get_uint64_32 s 17 in
-        let newsize = get_uint64_32 s 21 in
-        let bloc = String.sub s 25 (len-25) in
-        EmuleCompressedPart (md4, statpos, newsize, bloc)
+        EmuleCompressedPart (EmuleCompressedPart.parse false len s)
 
     | 0x85 (* 133 *) ->
         EmulePublicKeyReq(EmulePublicKeyReq.parse len s)
@@ -1352,6 +1375,10 @@ let rec parse_emule_packet emule opcode len s =
         in
         EmuleMultiPacketAnswerReq (md4, iter s 17 len)
 
+    | 0xa1 (* 161 *) -> (* OP_COMPRESSEDPART_I64 *)
+        EmuleCompressedPart (EmuleCompressedPart.parse true len s)
+    | 0xa2 -> BlocReq (Bloc.parse true len s) (* OP_SENDINGPART_I64 *)
+    | 0xa3 -> QueryBlocReq (QueryBloc.parse true len s) (*OP_REQUESTPARTS_I64 *)
     | 0xfe (* 254 *) ->
         EmulePortTestReq s
 
@@ -1377,8 +1404,8 @@ and parse emule_version magic s =
         begin
           match opcode with
           | 1 -> ConnectReq (Connect.parse false len s)
-          | 70 -> BlocReq (Bloc.parse len s)
-          | 71 -> QueryBlocReq (QueryBloc.parse len s)
+          | 70 -> BlocReq (Bloc.parse false len s)
+          | 71 -> QueryBlocReq (QueryBloc.parse false len s)
           | 72 -> NoSuchFileReq (NoSuchFile.parse len s)
           | 73 -> EndOfDownloadReq (EndOfDownload.parse len s)
           | 74 -> ViewFilesReq (ViewFiles.parse len s)
@@ -1396,8 +1423,8 @@ and parse emule_version magic s =
           | 85 -> AvailableSlotReq (AvailableSlot.parse len s)
 (* ReleaseSlot: the upload is finished *)
           | 86 -> ReleaseSlotReq (ReleaseSlot.parse len s)
-(* CloseSlot: the upload slot is not available *)
-          | 87 -> CloseSlotReq (CloseSlot.parse len s)
+(* OutOfParts: the upload slot is not available *)
+          | 87 -> OutOfPartsReq (OutOfParts.parse len s)
           | 88 -> QueryFileReq (QueryFile.parse emule_version len s)
           | 89 -> QueryFileReplyReq (QueryFileReply.parse len s)
           | 92 -> QueueRankReq (QueueRank.parse len s)
@@ -1419,7 +1446,7 @@ and parse emule_version magic s =
 
     | 0xD4 -> (* 212 *)
 
-          let s = Autoconf.zlib__uncompress_string2 (String.sub s 1 (len-1)) in
+          let s = Zlib.uncompress_string2 (String.sub s 1 (len-1)) in
           let s = Printf.sprintf "%c%s" (char_of_int opcode) s in
     begin try
             parse_emule_packet emule_version opcode (String.length s) s
@@ -1470,6 +1497,8 @@ let write emule buf t =
     | EmuleQueueRankingReq _
     | EmuleCompressedPart _
       -> 0xC5
+    | QueryBlocReq t when t.QueryBloc.usesixtyfour -> 0xC5
+    | BlocReq t when t.Bloc.usesixtyfour -> 0xC5
     | _
       ->  227
   in
@@ -1491,10 +1520,10 @@ let write emule buf t =
         buf_int8 buf 77;
         OtherLocations.write buf t
     | QueryBlocReq t ->
-        buf_int8 buf 71;
+        buf_int8 buf (if t.QueryBloc.usesixtyfour then 0xa3 else 71);
         QueryBloc.write buf t
     | BlocReq t ->
-        buf_int8 buf 70;
+        buf_int8 buf (if t.Bloc.usesixtyfour then 0xa2 else 70);
         Bloc.write buf t
     | JoinQueueReq t ->
         buf_int8 buf 84;
@@ -1517,9 +1546,9 @@ let write emule buf t =
     | ReleaseSlotReq t ->
         buf_int8 buf 86;
         ReleaseSlot.write buf t
-    | CloseSlotReq t ->
+    | OutOfPartsReq t ->
         buf_int8 buf 87;
-        CloseSlot.write buf t
+        OutOfParts.write buf t
     | ViewFilesReq t ->
         buf_int8 buf 74;
         ViewFiles.write buf t
@@ -1578,13 +1607,9 @@ let write emule buf t =
     | EmuleFileDescReq t ->
         buf_int8 buf 0x61;
         EmuleFileDesc.write buf t
-    | EmuleCompressedPart (md4, statpos, newsize, bloc) ->
-        buf_int8 buf 0x40;
-        buf_md4 buf md4;
-        buf_int64_32 buf statpos;
-        buf_int64_32 buf newsize;
-        Buffer.add_string buf bloc
-
+    | EmuleCompressedPart t ->
+        buf_int8 buf (if t.EmuleCompressedPart.usesixtyfour then 0xa1 else 0x40);
+        EmuleCompressedPart.write buf t
     | EmuleMultiPacketReq (md4, list) ->
         buf_int8 buf 0x92;
         buf_md4 buf md4;
