@@ -28,31 +28,6 @@ open CommonGlobals
 open DonkeyTypes
 open DonkeyMftp
 
-(*
-let field_of_tagname s =
-  match s with
-  | "size" -> Field_Size
-  | "filename" -> Field_Filename
-  | "Artist" -> Field_Artist
-  | "Album" -> Field_Album
-  | "Title" -> Field_Title
-  | "format" -> Field_Format
-  | "type" -> Field_Type
-  | s -> Field_UNKNOWN s
-
-let tagname_of_field field =
-  match field with
-    Field_Size -> "size"
-  | Field_Filename -> "filename"
-  | Field_Artist -> "Artist"
-  | Field_Album -> "Album"
-  | Field_Title -> "Title"
-  | Field_Format -> "format"
-  | Field_Type -> "type"
-  | Field_Uid -> "uid"
-  | Field_unknown s -> s
-*)
-
 module Connect = struct
     type t = {
         md4 : Md4.t;
@@ -63,10 +38,10 @@ module Connect = struct
 
     let names_of_tag =
       [
-       "\001", Field_UNKNOWN "name";          (* CT_NAME          0x01 *)
-       "\017", Field_UNKNOWN "version";       (* CT_VERSION       0x11 *)
-       "\032", Field_UNKNOWN "extended";      (* CT_SERVER_FLAGS  0x20 *)
-       "\251", Field_UNKNOWN "emule_version"; (* CT_EMULE_VERSION 0xfb *)
+       "\001", Field_KNOWN "name";          (* CT_NAME          0x01 *)
+       "\017", Field_KNOWN "version";       (* CT_VERSION       0x11 *)
+       "\032", Field_KNOWN "extended";      (* CT_SERVER_FLAGS  0x20 *)
+       "\251", Field_KNOWN "emule_version"; (* CT_EMULE_VERSION 0xfb *)
       ]
 
     let parse len s =
@@ -388,8 +363,8 @@ module ServerInfo = struct
 
     let names_of_tag =
       [
-        "\001", Field_UNKNOWN "name";
-        "\011", Field_UNKNOWN "description";
+        "\001", Field_KNOWN "name";
+        "\011", Field_KNOWN "description";
       ]
 
     let parse len s =
@@ -760,9 +735,9 @@ module QueryUsersReply = struct (* request 67 *)
 
     let names_of_tag =
       [
-        "\001", Field_UNKNOWN "name";
-        "\017", Field_UNKNOWN "version";
-        "\015", Field_UNKNOWN "port";
+        "\001", Field_KNOWN "name";
+        "\017", Field_KNOWN "version";
+        "\015", Field_KNOWN "port";
       ]
 
     let rec parse_clients s pos nclients left =
@@ -833,7 +808,13 @@ module QueryLocation  = struct
       Printf.bprintf oc "QUERY LOCATION OF %s [%Ld]\n" (Md4.to_string t.md4) t.size
 
     let write buf t =
-      buf_md4 buf t.md4; buf_int64_32 buf t.size 
+      buf_md4 buf t.md4;
+      if t.size > old_max_emule_file_size then
+        begin
+	  buf_int64_32 buf 0L; buf_int64 buf t.size 
+        end
+      else
+        buf_int64_32 buf t.size 
   end
 
 module QueryLocationReply  = struct
@@ -1143,7 +1124,7 @@ let rec parse magic s =
               raise Not_found
         end
     | 0xD4 -> (* 212 *)
-          let s = Autoconf.zlib__uncompress_string2 (String.sub s 1 (len-1)) in
+          let s = Zlib.uncompress_string2 (String.sub s 1 (len-1)) in
           let s = Printf.sprintf "%c%s" (char_of_int opcode) s in
           parse 227 s
 
@@ -1384,8 +1365,8 @@ let to_string m =
   (*
 let _ =
   let s = "abcdefghijklmnopqrstuvwxyz" in
-  let compressed = Autoconf.zlib__compress_string s in
-  let ss = Autoconf.zlib__uncompress_string2 compressed in
+  let compressed = Zlib.compress_string s in
+  let ss = Zlib.uncompress_string2 compressed in
   lprintf "[%s] <> [%s]\n" s (String.escaped ss);
   assert (s = ss);
   exit 2
