@@ -184,6 +184,14 @@ let find_server ip port =
     h.host_server
   with _ -> None
 
+let check_server_country_code s =
+  if Geoip.active () then
+    match s.server_country_code with
+    | None ->
+        s.server_country_code <-
+          Geoip.get_country_code_option (Ip.ip_of_addr s.server_host.host_addr)
+    | _ -> ()
+
 let new_server ip port =
   let h = H.new_host ip port Ultrapeer in
   match h.host_server with
@@ -193,6 +201,7 @@ let new_server ip port =
           server_server = server_impl;
           server_ciphers = None;
           server_host = h;
+          server_country_code = None;
           server_sock = NoConnection;
           server_agent = "<unknown>";
           server_description = "";
@@ -220,6 +229,7 @@ let new_server ip port =
       server_add server_impl;
       h.host_server <- Some s;
       h.host_on_remove <- (fun _ -> server_remove (as_server server_impl));
+      check_server_country_code s;
       s
       
 let extract_uids arg = Uid.expand [Uid.of_string arg]
@@ -310,7 +320,7 @@ List.iter iter_uid other_uids;
 let megabyte = Int64.of_int (1024 * 1024)
 let megabytes10 = Int64.of_int (10 * 1024 * 1024)
       
-let new_file file_temporary file_name file_size file_uids = 
+let new_file file_temporary file_name file_size file_uids user group =
   let file_temp = Filename.concat !!temp_directory file_temporary in
   let t = Unix32.create_rw file_temp in
   let rec file = {
@@ -329,6 +339,8 @@ let new_file file_temporary file_name file_size file_uids =
       impl_file_fd = Some t;
       impl_file_size = file_size;
       impl_file_downloaded = Int64.zero;
+      impl_file_owner = user;
+      impl_file_group = group;
       impl_file_val = file;
       impl_file_ops = file_ops;
       impl_file_age = last_time ();          
@@ -337,7 +349,7 @@ let new_file file_temporary file_name file_size file_uids =
     } 
   in
   if !verbose then
-    lprintf_nl "SET SIZE : %Ld\n" file_size;
+    lprintf_nl "SET SIZE : %Ld" file_size;
   let kernel = CommonSwarming.create_swarmer file_temp file_size in
   let swarmer = CommonSwarming.create kernel (as_file file) megabyte in
   CommonSwarming.set_verifier swarmer ForceVerification;
@@ -357,7 +369,7 @@ let new_file file_temporary file_name file_size file_uids =
 
 exception FileFound of file
   
-let new_file file_id file_name file_size file_uids =
+let new_file file_id file_name file_size file_uids user group =
 (*  if file_uids = [] then 
     try Hashtbl.find files_by_key (file_name, file_size) with
       _ -> 
@@ -370,7 +382,7 @@ let new_file file_id file_name file_size file_uids =
         try  raise (FileFound (Hashtbl.find files_by_uid uid))
         with Not_found -> ()
     ) file_uids;
-    let file = new_file file_id file_name file_size file_uids in
+    let file = new_file file_id file_name file_size file_uids user group in
     List.iter (fun uid -> 
         if !verbose then
           lprintf "Adding file %s\n" (Uid.to_string uid);
@@ -411,7 +423,17 @@ let new_user kind =
       user_add user_impl;
       Hashtbl.add users_by_uid kind user;
       user
-  
+
+let check_client_country_code c =
+  if Geoip.active () then
+    match c.client_country_code with
+    | None ->
+        (match c.client_host with
+        | Some (ip,port) ->
+            c.client_country_code <- Geoip.get_country_code_option ip
+        | _ -> ())
+    | _ -> ()
+
 let new_client kind =
   try
     Hashtbl.find clients_by_uid kind 
@@ -433,6 +455,7 @@ client_error = false;
           client_connection_control = new_connection_control (());
           client_downloads = [];
           client_host = None;
+          client_country_code = None;
           client_reconnect = false;
           client_in_queues = [];
           client_connected_for = None;
