@@ -77,8 +77,8 @@ let recover_file file =
       if file_state file = FileDownloading then
         GnutellaServers.really_recover_file file        
 
-let download_file r =
-  let file = GnutellaServers.really_download_file r in
+let download_file r user group =
+  let file = GnutellaServers.really_download_file r user group in
   recover_file file;
   as_file file
 
@@ -219,6 +219,7 @@ that we can reuse queries *)
     [
     !!client_port, "client_port TCP+UDP";
     ]);
+  network.op_network_porttest_result <- (fun _ -> PorttestNotAvailable);
   network.op_network_share <- (fun fullname codedname size ->
       (*
       lprintf "*************** op_network_share %s\n
@@ -241,8 +242,8 @@ that we can reuse queries *)
         end
   );
 (* TODO RESULT *)
-  network.op_network_download <- (fun r ->
-      result_download r
+  network.op_network_download <- (fun r user ->
+      result_download r user
   )
   
 let file_num file =
@@ -284,11 +285,21 @@ let _ =
           GnutellaServers.get_file_from_source c file
       ) file.file_clients
   );
-  
+  file_ops.op_file_download_order <- (fun file strategy ->
+      match file.file_swarmer with
+      | None -> None
+      | Some s ->
+          (match strategy with
+          (* return current strategy *)
+          | None -> Some (CommonSwarming.get_strategy s)
+          | Some strategy -> CommonSwarming.set_strategy s strategy;
+                      Some (CommonSwarming.get_strategy s))
+  )
+
 module P = GuiTypes
   
 let _ =
-  file_ops.op_file_print_html <- (fun file buf -> ());
+  file_ops.op_file_print <- (fun file buf -> ());
   file_ops.op_file_cancel <- (fun file ->
       CommonSwarming.remove_swarmer file.file_swarmer;
       file.file_swarmer <- None;
@@ -307,6 +318,10 @@ let _ =
          | None -> None 
 	 | Some swarmer -> 
 	     Some (CommonSwarming.chunks_verified_bitmap swarmer));
+        P.file_chunk_size = (match file.file_swarmer with
+         | None -> None
+	 | Some t ->
+	     Some (List.map (fun t -> t.CommonSwarming.t_chunk_size) t.CommonSwarming.t_s.CommonSwarming.s_networks));
         P.file_availability =   [network.network_num,
            (match file.file_swarmer with
            None -> "" | Some swarmer ->
@@ -320,12 +335,14 @@ let _ =
   
 let _ =
   server_ops.op_server_info <- (fun s ->
+      check_server_country_code s;
       if !!enable_gnutella then
         { (impl_server_info s.server_server) with
 
           P.server_network = network.network_num;
           P.server_addr = s.server_host.host_addr;
           P.server_port = s.server_host.host_port;
+          P.server_country_code = s.server_country_code;
           P.server_nusers = s.server_nusers;
           P.server_max_users = s.server_maxnusers;
           P.server_nfiles = s.server_nfiles;
@@ -370,7 +387,7 @@ let _ =
       List2.tail_map (fun s -> as_server s.server_server)
       !connected_servers
   );
-  network.op_network_parse_url <- (fun url ->
+  network.op_network_parse_url <- (fun url user group ->
       match String2.split (String.escaped url) '|' with
       | "gnut://" :: "server" :: ip :: port :: _ ->  
           let ip = Ip.addr_of_string ip in
@@ -402,7 +419,7 @@ let _ =
               (* Start a download for this file *)
               let rs = new_result name size [] uids [] in
               let r = IndexedResults.get_result rs in
-              let file = download_file r in
+              let file = download_file r user group in
               CommonInteractive.start_download file;
               "started Gnutella download", true
             end
@@ -411,6 +428,7 @@ let _ =
   
 let _ =
   client_ops.op_client_info <- (fun c ->
+      check_client_country_code c;
       { (impl_client_info c.client_client) with
 
         P.client_network = network.network_num;
@@ -419,6 +437,7 @@ let _ =
         P.client_name = if c.client_user.user_speed > 0 
                           then Printf.sprintf "%s (%d)" c.client_user.user_nick c.client_user.user_speed
                           else c.client_user.user_nick;
+        P.client_country_code = c.client_country_code;
         P.client_software = c.client_user.user_software;
 
       }
@@ -441,8 +460,8 @@ let _ =
         client_print cc o; 
         Printf.bprintf buf "client: %s downloaded: %s uploaded: %s"
           "gN" (* cinfo.GuiTypes.client_software *)
-          (Int64.to_string cinfo.GuiTypes.client_downloaded)
-        (Int64.to_string cinfo.GuiTypes.client_uploaded);
+          (Int64.to_string cinfo.GuiTypes.client_total_downloaded)
+        (Int64.to_string cinfo.GuiTypes.client_total_uploaded);
         Printf.bprintf buf "\nfilename: %s\n\n" info.GuiTypes.file_name;
     );    
     client_ops.op_client_dprint_html <- (fun c o file str ->
@@ -450,6 +469,11 @@ let _ =
         let buf = o.conn_buf in
         let cc = as_client c in
         let cinfo = client_info cc in
+        let ccode,cname =
+          match c.client_host with
+          | None -> Geoip.unknown_country
+          | Some _ -> Geoip.get_country_code_name cinfo.GuiTypes.client_country_code
+        in
         Printf.bprintf buf " \\<tr onMouseOver=\\\"mOvr(this);\\\"
     onMouseOut=\\\"mOut(this);\\\" class=\\\"%s\\\"\\>" str;
         
@@ -475,9 +499,11 @@ let _ =
           ("", "sr", "D");
           ("", "sr", "N");
           ("", "sr", (string_of_kind cinfo.GuiTypes.client_kind));
-          ] @ (if !Geoip.active then [("?", "sr", "?")] else []) @ [
-          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
-          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_downloaded));
+          ] @ (if Geoip.active () then [( cname, "sr br", CommonPictures.flag_html ccode)] else []) @ [
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_uploaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_downloaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_session_uploaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_session_downloaded));
           ("", "sr", info.GuiTypes.file_name); ]);
         true
     )     
