@@ -573,10 +573,10 @@ utf8_validate (const char  *str,
 #define SUBLANG_AZERI_CYRILLIC 0x02
 #endif
 #ifndef SUBLANG_BENGALI_INDIA
-#define SUBLANG_BENGALI_INDIA 0x00
+#define SUBLANG_BENGALI_INDIA 0x01
 #endif
 #ifndef SUBLANG_BENGALI_BANGLADESH
-#define SUBLANG_BENGALI_BANGLADESH 0x01
+#define SUBLANG_BENGALI_BANGLADESH 0x02
 #endif
 #ifndef SUBLANG_CHINESE_MACAU
 #define SUBLANG_CHINESE_MACAU 0x05
@@ -669,16 +669,16 @@ utf8_validate (const char  *str,
 #define SUBLANG_NEPALI_INDIA 0x02
 #endif
 #ifndef SUBLANG_PUNJABI_INDIA
-#define SUBLANG_PUNJABI_INDIA 0x00
+#define SUBLANG_PUNJABI_INDIA 0x01
 #endif
 #ifndef SUBLANG_PUNJABI_PAKISTAN
-#define SUBLANG_PUNJABI_PAKISTAN 0x01
+#define SUBLANG_PUNJABI_PAKISTAN 0x02
 #endif
 #ifndef SUBLANG_ROMANIAN_ROMANIA
-#define SUBLANG_ROMANIAN_ROMANIA 0x00
+#define SUBLANG_ROMANIAN_ROMANIA 0x01
 #endif
 #ifndef SUBLANG_ROMANIAN_MOLDOVA
-#define SUBLANG_ROMANIAN_MOLDOVA 0x01
+#define SUBLANG_ROMANIAN_MOLDOVA 0x02
 #endif
 #ifndef SUBLANG_SERBIAN_LATIN
 #define SUBLANG_SERBIAN_LATIN 0x02
@@ -686,12 +686,12 @@ utf8_validate (const char  *str,
 #ifndef SUBLANG_SERBIAN_CYRILLIC
 #define SUBLANG_SERBIAN_CYRILLIC 0x03
 #endif
-#ifndef SUBLANG_SINDHI_INDIA
-#define SUBLANG_SINDHI_INDIA 0x00
-#endif
 #ifndef SUBLANG_SINDHI_PAKISTAN
 #define SUBLANG_SINDHI_PAKISTAN 0x01
 #endif
+#ifndef SUBLANG_SINDHI_AFGHANISTAN
+#define SUBLANG_SINDHI_AFGHANISTAN 0x02
+#endif
 #ifndef SUBLANG_SPANISH_GUATEMALA
 #define SUBLANG_SPANISH_GUATEMALA 0x04
 #endif
@@ -749,14 +749,14 @@ utf8_validate (const char  *str,
 #ifndef SUBLANG_TAMAZIGHT_ARABIC
 #define SUBLANG_TAMAZIGHT_ARABIC 0x01
 #endif
-#ifndef SUBLANG_TAMAZIGHT_LATIN
-#define SUBLANG_TAMAZIGHT_LATIN 0x02
+#ifndef SUBLANG_TAMAZIGHT_ALGERIA_LATIN
+#define SUBLANG_TAMAZIGHT_ALGERIA_LATIN 0x02
 #endif
 #ifndef SUBLANG_TIGRINYA_ETHIOPIA
-#define SUBLANG_TIGRINYA_ETHIOPIA 0x00
+#define SUBLANG_TIGRINYA_ETHIOPIA 0x01
 #endif
 #ifndef SUBLANG_TIGRINYA_ERITREA
-#define SUBLANG_TIGRINYA_ERITREA 0x01
+#define SUBLANG_TIGRINYA_ERITREA 0x02
 #endif
 #ifndef SUBLANG_URDU_PAKISTAN
 #define SUBLANG_URDU_PAKISTAN 0x01
@@ -1079,8 +1079,8 @@ ml_win32_get_default_language (void)
     case LANG_SINDHI: l = "sd";
       switch (sub)
 	{
-	case SUBLANG_SINDHI_INDIA: sl = "IN"; break;
 	case SUBLANG_SINDHI_PAKISTAN: sl = "PK"; break;
+	case SUBLANG_SINDHI_AFGHANISTAN: sl = "AF"; break;
 	}
       break;
     case LANG_SINHALESE: l = "si"; sl = "LK"; break;
