@@ -21,12 +21,17 @@ type t
 
 val verbose : bool ref
 
+val get_max_cache_size : unit -> int
+val set_max_cache_size : int -> unit
+val max_cache_size_default : int
+
 val external_start : unit -> unit
 val external_exit : unit -> unit
 val uname : unit -> string
 val os_supported : unit -> bool
 
-val create_dir_mask : string ref
+val create_file_mode : int ref
+val create_dir_mode : int ref
 val close : t -> unit
 (* val force_fd : t -> Unix.file_descr *)
   
@@ -41,7 +46,6 @@ val close_all : unit -> unit
 val fds_size : int
 val filename : t -> string
 val rename : t -> string -> unit 
-val max_cache_size : int ref
 val mtime : string -> float
 val mtime64 : t -> float
 val owner : string -> (string * string)
@@ -94,7 +98,7 @@ val bsize : string -> int64 option
 val blocks : string -> int64 option
 val bfree : string -> int64 option
 val bavail : string -> int64 option
-val fnamelen : string -> int64 option
+val fnamelen : string -> int option
 val disktotal : string -> int64 option
 val diskfree : string -> int64 option
 val diskused : string -> int64 option
