@@ -27,6 +27,7 @@ open BasicSocket
 open Unix
 open Url
 open TcpBufferedSocket
+open Int64ops
 
 
 type http_request =
@@ -54,7 +55,7 @@ type request = {
   }
 
 type content_handler = 
-  int -> (string * string) list -> TcpBufferedSocket.t -> int -> unit
+  int64 -> (string * string) list -> TcpBufferedSocket.t -> int -> unit
 
 let log_prefix = "[HTTPcl]"
 
@@ -62,7 +63,7 @@ let lprintf_nl fmt =
   lprintf_nl2 log_prefix fmt
 
 let basic_request = {
-    req_url = Url.of_string "http://www.mldonkey.net/";
+    req_url = Url.of_string "http://www.mldonkey.org/";
     req_referer = None;
     req_save_to_file_time = 0.;
     req_request = GET;
@@ -170,7 +171,7 @@ let read_header header_handler sock nread =
   let b = TcpBufferedSocket.buf sock in
   let end_pos = b.pos + b.len in
   let new_pos = end_pos - nread in
-  let new_pos = maxi 0 (new_pos - 1) in
+  let new_pos = max 0 (new_pos - 1) in
   (*
   lprintf "received [%s]" (String.escaped
       (String.sub b.buf new_pos nread));
@@ -255,13 +256,8 @@ let rec get_page r content_handler f ferr =
         TcpBufferedSocket.set_reader sock (http_reply_handler nread
             (default_headers_handler url level));
         set_rtimeout sock 5.;
-      (*
-        TcpBufferedSocket.set_closer sock (fun _ _ -> ()
-        lprintf "Connection closed nread:%b\n" !nread; 
-        )
-      *)
-
     )
+    ferr;
   with e -> 
     lprintf_nl "error in get_url"; 
     raise Not_found
@@ -281,10 +277,10 @@ let rec get_page r content_handler f ferr =
               f ()
             );
 
-        let content_length = ref (-1) in
+        let content_length = ref (-1L) in
         List.iter (fun (name, content) ->
             if String.lowercase name = "content-length" then
-            try content_length := int_of_string content
+            try content_length := Int64.of_string content
             with _ -> lprintf_nl "bad content length [%s]" content;
         ) headers;
         let location = "Location", Url.to_string old_url in
@@ -334,6 +330,14 @@ let rec get_page r content_handler f ferr =
           raise Not_found
         end
           
+    | 400 when r.req_request = HEAD ->
+        lprintf_nl "Error 400 received for HEAD %s, re-try GET" (Url.to_string_no_args r.req_url);
+        let r2 = {
+          r with
+          req_request = GET;
+        } in
+        get_page r2 content_handler f ferr
+
     | 404 ->
         lprintf_nl "404: Not found for: %s" (Url.to_string_no_args r.req_url);
         close sock (Closed_for_error "bad reply");
@@ -355,6 +359,7 @@ let rec get_page r content_handler f ferr =
     end
         else begin
           lprintf_nl "more than %d retries, aborting." r.req_max_retry;
+          ferr ans_code;
           raise Not_found
         end
           
@@ -370,7 +375,7 @@ let rec get_page r content_handler f ferr =
 let wget r f = 
   
   let file_buf = Buffer.create 1000 in
-  let file_size = ref 0 in
+  let file_size = ref 0L in
   
   try
   get_page r (fun maxlen headers sock nread ->
@@ -379,13 +384,13 @@ let wget r f =
       
       if nread > 0 then begin
           let left = 
-            if maxlen >= 0 then
-              mini (maxlen - !file_size) nread
+            if maxlen >= 0L then
+              min (Int64.to_int (maxlen -- !file_size)) nread
             else nread
           in
           Buffer.add_string file_buf (String.sub buf.buf buf.pos left);
           buf_used buf left;
-          file_size := !file_size + left;
+          file_size := !file_size ++ (Int64.of_int left);
           if nread > left then
             TcpBufferedSocket.close sock Closed_by_user
         end
@@ -443,7 +448,7 @@ let whead r f = whead2 r f def_ferr
 let wget_string r f progress =
     
   let file_buf = Buffer.create 1000 in
-  let file_size = ref 0 in
+  let file_size = ref 0L in
 
   get_page r
     (fun maxlen headers sock nread ->
@@ -451,14 +456,14 @@ let wget_string r f progress =
         
         if nread > 0 then begin
             let left = 
-              if maxlen >= 0 then
-                mini (maxlen - !file_size) nread
+              if maxlen >= 0L then
+                min (Int64.to_int (maxlen -- !file_size)) nread
               else nread
           in
           Buffer.add_string file_buf (String.sub buf.buf buf.pos left);
           progress left maxlen;
           buf_used buf left;
-          file_size := !file_size + left;
+          file_size := !file_size ++ (Int64.of_int left);
           if nread > left then
             TcpBufferedSocket.close sock Closed_by_user
         end)
